<?php

namespace TeamBooking\Functions;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Database, TeamBooking\Toolkit;

/////////////////////////////////
//  INCLUDING AJAX CALLBACKS   //
/////////////////////////////////
include_once dirname(TEAMBOOKING_FILE_PATH) . '/includes/tb_ajax_calls.php';

/////////////////////////////////
//  INCLUDING FILE GENERATORS  //
/////////////////////////////////
include_once dirname(TEAMBOOKING_FILE_PATH) . '/includes/tb_file_generators.php';

/////////////////////////
//  INCLUDING TOOLKIT  //
/////////////////////////
include_once dirname(TEAMBOOKING_FILE_PATH) . '/includes/tb_toolkit.php';

/**
 * Returns the plugin settings object
 *
 * @return \TeamBookingSettings Plugin settings object
 */
function getSettings()
{
    global $team_booking_settings;
    if (!($team_booking_settings instanceof \TeamBookingSettings)) $team_booking_settings = get_option('team_booking');

    return $team_booking_settings;
}

/**
 * Get the Coworker ID list
 *
 * @return array Coworkers ID list
 */
function getCoworkersIdList()
{
    $all_users = get_users();
    $coworkers = array();
    foreach ($all_users as $user) {
        if ($user->has_cap('tb_can_sync_calendar')) {
            $coworkers[] = $user->ID;
        }
    }
    if (NULL !== $coworkers) {
        return array_values($coworkers);
    } else {
        return array();
    }
}

/**
 * Get the authorized Coworker ID list
 *
 * @return array Coworkers ID list
 */
function getAuthCoworkersIdList()
{
    $coworkers_data = getSettings()->getCoworkersData();
    $list = array();
    if (NULL !== $coworkers_data) {
        foreach ($coworkers_data as $coworker_id => $data) {
            $token = $data->getAccessToken();
            if (!empty($token)) {
                $list[] = $coworker_id;
            }
        }
    }

    return $list;
}

/**
 * Get the authorized Coworkers list
 *
 * @return array
 */
function getAuthCoworkersList()
{
    $results = array();
    $coworkers_data = getSettings()->getCoworkersData();
    foreach ($coworkers_data as $id => $data) {
        /* @var $data \TeamBookingCoworker */
        if ($data->getAccessToken()) {
            $results[ $id ]['name'] = $data->getDisplayName();
            $results[ $id ]['email'] = $data->getEmail();
            $results[ $id ]['roles'] = $data->getRoles();
            $results[ $id ]['tokens'] = $data->getAccessToken();
            $results[ $id ]['calendars'] = $data->getCalendars();
        }
    }

    return $results;
}

/**
 * Get the Coworkers list
 *
 * @return array
 */

function getAllCoworkersList()
{
    $results = array();
    // Get the ids of users with TB capability right now
    $ids = getCoworkersIdList();
    // Get the eventual list of coworkers already present
    $present_data = getSettings()->getCoworkersData();
    foreach ($ids as $id) {
        if (isset($present_data[ $id ])) {
            $coworker = $present_data[ $id ];
            unset($present_data[ $id ]);
        } else {
            $coworker = new \TeamBookingCoworker($id);
        }
        $results[ $id ]['name'] = $coworker->getDisplayName();
        $results[ $id ]['email'] = $coworker->getEmail();
        $results[ $id ]['calendars'] = $coworker->getCalendars();
        $results[ $id ]['roles'] = $coworker->getRoles();
        if (isset(json_decode($coworker->getAccessToken())->refresh_token)) {
            $results[ $id ]['token'] = 'refresh';
        } elseif (isset(json_decode($coworker->getAccessToken())->access_token)) {
            $results[ $id ]['token'] = 'access';
        } else {
            $results[ $id ]['token'] = '';
        }
    }
    // There are coworkers without TB capability left?
    if (!empty($present_data)) {
        foreach ($present_data as $id => $data) {
            if (get_userdata($id) == FALSE) {
                // User not exists anymore
                $settings = getSettings();
                $settings->dropCoworkerData($id);
                $settings->save();
                continue;
            }
            $results[ $id ]['name'] = $data->getDisplayName();
            $results[ $id ]['email'] = $data->getEmail();
            $results[ $id ]['calendar'] = $data->getCalendars();
            $results[ $id ]['roles'] = $data->getRoles();
            if (isset(json_decode($data->getAccessToken())->refresh_token)) {
                $results[ $id ]['token'] = 'refresh';
            } elseif (isset(json_decode($data->getAccessToken())->access_token)) {
                $results[ $id ]['token'] = 'access';
            } else {
                $results[ $id ]['token'] = '';
            }
            $results[ $id ]['allowed_no_more'] = TRUE;
        }
    }

    return $results;
}

/**
 * Renders the field validation modal in service settings tab
 *
 * @param \TeamBookingFormTextField $field
 * @param string                    $fieldname
 *
 * @return string
 */
function getValidationModal(\TeamBookingFormTextField $field, $fieldname)
{
    ob_start();
    ?>
    <div class="ui small modal" id="tb-field-regex-modal-<?= $field->getHook() ?>">
        <i class="close tb-icon"></i>

        <div class="content" style="display:block;width: initial;">
            <div style="font-style: italic;font-weight: 300;">
                <?= __('Validation rule', 'team-booking') ?>
            </div>
            <select style="margin-bottom:10px;width:99%;" name="<?= $fieldname ?>[<?= $field->getHook() ?>_validation]">
                <option
                    value="none" <?php selected('none', $field->getValidationRule(), TRUE); ?>><?= __('No validation', 'team-booking') ?></option>
                <option
                    value="email" <?php selected('email', $field->getValidationRule(), TRUE); ?>><?= __('Email', 'team-booking') ?></option>
                <option
                    value="alphanumeric" <?php selected('alphanumeric', $field->getValidationRule(), TRUE); ?>><?= __('Alphanumeric', 'team-booking') ?></option>
                <option
                    value="phone" <?php selected('phone', $field->getValidationRule(), TRUE); ?>><?= __('Phone number (US only)', 'team-booking') ?></option>
                <option
                    value="custom" <?php selected('custom', $field->getValidationRule(), TRUE); ?>><?= __('Custom', 'team-booking') ?></option>
            </select>

            <div style="font-style: italic;font-weight: 300;">
                <?= __('Custom validation regex (works if "custom" is selected)', 'team-booking') ?>
            </div>
            <input type="text" class="large-text" style="margin-bottom:10px;"
                   name="<?= $fieldname ?>[<?= $field->getHook() ?>_regex]"
                   value="<?= $field->getValidationRegex(TRUE) ?>">

            <div class="ui button tb-close" style="height:auto">
                <?= __('Close', 'team-booking') ?>
            </div>
        </div>
    </div>
    <script>
        jQuery('#tb-field-regex-open-<?= $field->getHook() ?>').on('click', function (e) {
            e.preventDefault();
            jQuery('#tb-field-regex-modal-<?= $field->getHook() ?>')
                .uiModal({detachable: false})
                .uiModal('attach events', '.tb-close', 'hide')
                .uiModal('show')
            ;
        });
    </script>
    <?php
    return ob_get_clean();
}

/**
 * Checks if there is a booking with that ID already
 *
 * @param string $id
 *
 * @return boolean TRUE if exists
 */
function checkServiceIdExistance($id)
{
    $services = getSettings()->getServices();

    return array_key_exists($id, $services);
}

/**
 * Checks if there is a booking with that name already
 *
 * @param string $name
 *
 * @return boolean
 */
function checkServiceNameExistance($name)
{
    $services = getSettings()->getServices();
    $response = FALSE;
    foreach ($services as $service) {
        /* @var $service \TeamBookingType */
        if (strtolower($service->getName()) == strtolower($name)) {
            $response = TRUE;
            break;
        }
    }

    return $response;
}

/**
 *
 * Error log
 *
 * @param \TeamBooking_ErrorLog $log
 */
function errorLog(\TeamBooking_ErrorLog $log)
{
    $settings = getSettings();
    $error_logs = $settings->getErrorLogs();
    $now = current_time('timestamp');
    $log->setTimestamp($now);
    if (count($error_logs) >= 10) {
        array_shift($error_logs);
    }
    $error_logs[] = $log;
    $settings->setErrorLogs($error_logs);
    $settings->save();
}

/**
 * Returns appropriate text color
 *
 * @param string     $color (hexadecimal color value)
 * @param bool|FALSE $prefer_white
 *
 * @return string
 */
function getRightTextColor($color, $prefer_white = FALSE)
{
    $brightness_limit = $prefer_white ? 185 : 145;
    $rgb = Toolkit\hex2RGB($color);
    if (!$rgb) {
        return 'inherit';
    }
    $brightness = sqrt(
        $rgb['red'] * $rgb['red'] * .299 +
        $rgb['green'] * $rgb['green'] * .587 +
        $rgb['blue'] * $rgb['blue'] * .114);
    if ($brightness < $brightness_limit) {
        return '#FFFFFF';
    } else {
        return '#414141';
    }
}

/**
 * Returns appropriate hover color
 *
 * @param string $color (hexadecimal color value)
 *
 * @return string
 */
function getRightHoverColor($color)
{
    $rgb = Toolkit\hex2RGB($color);
    $brightness = sqrt(
        $rgb['red'] * $rgb['red'] * .299 +
        $rgb['green'] * $rgb['green'] * .587 +
        $rgb['blue'] * $rgb['blue'] * .114);
    if ($brightness < 145) {
        return 'rgba(255, 255, 255, 0.15)';
    } else {
        return 'rgba(0, 0, 0, 0.15)';
    }
}

/**
 * Returns appropriate background label color
 *
 * @param string $color (hexadecimal color value)
 *
 * @return string
 */
function getRightBackgroundColor($color)
{
    $rgb = Toolkit\hex2RGB($color);
    $brightness = sqrt(
        $rgb['red'] * $rgb['red'] * .299 +
        $rgb['green'] * $rgb['green'] * .587 +
        $rgb['blue'] * $rgb['blue'] * .114);
    if ($brightness < 145) {
        return '#2B2B2B';
    } else {
        return '#F4F4F4';
    }
}

function currencyCodeToSymbol($cc, $amount = 0, $keep_code = FALSE, $only_position = FALSE)
{
    $currencies = Toolkit\getCurrencies();
    if (!isset($currencies[ $cc ])) {
        $symbol = "$";
        $position = 'before';
    } else {
        if ($keep_code) {
            $symbol = $cc;
        } else {
            $symbol = $currencies[ $cc ]['symbol'];
        }
        $position = $currencies[ $cc ]['format'];
    }

    if ($only_position) {
        return $position;
    }

    if (NULL === $amount) {
        return $symbol;
    } else {
        if ($position === 'after') {
            return priceFormat($amount) . $symbol;
        } else {
            return $symbol . priceFormat($amount);
        }
    }
}

function priceFormat($value)
{
    return number_format((float)$value, 2);
}

function cleanReservations($all = FALSE, $just_check = FALSE, $timeout_override = FALSE)
{
    global $wpdb;
    $table_name = $wpdb->prefix . 'teambooking_reservations';
    if ($timeout_override) {
        $timeout = $timeout_override;
    } else {
        $timeout = 30 * DAY_IN_SECONDS;
        #$timeout = tbGetSettings()->getDatabaseReservationTimeout();
    }
    if ($all) {
        $wpdb->query("TRUNCATE TABLE $table_name");
    } else {
        $now = Toolkit\getNowInSecondsUTC(); // UTC, seconds
        $reservations = Database\Reservations::getAll();
        $count = 0;
        foreach ($reservations as $id => $reservation) {
            /* @var $reservation \TeamBooking_ReservationData */
            if ($reservation instanceof \TeamBooking_ReservationData) {
                $age = $now - $reservation->getStart(); // UTC, seconds
                if ($age > $timeout && $timeout) {
                    if (!$just_check) {
                        Database\Reservations::delete($id);
                    } else {
                        $count++;
                    }
                }
            }
        }
        if ($count) {
            return $count;
        } else {
            return FALSE;
        }
    }
}

function tbCleanReservationsRoutine()
{
    cleanReservations();

    return TRUE;
}

function isReservationTimedOut(\TeamBooking_ReservationData $reservation)
{
    $timeout = getSettings()->getMaxPendingTime(); // seconds
    $now = Toolkit\getNowInSecondsUTC(); // UTC, seconds
    $reservation_time = $reservation->getCreationInstant(); // UTC, seconds
    if (($now - $reservation_time) > $timeout) {
        return TRUE;
    } else {
        return FALSE;
    }
}

function isReservationPastInTime(\TeamBooking_ReservationData $reservation)
{
    $now = Toolkit\getNowInSecondsUTC(); // UTC, seconds
    $reservation_time = $reservation->getStart(); // UTC, seconds
    if (!$reservation_time) return FALSE; // it is unscheduled
    if ($now > $reservation_time) {
        return TRUE;
    } else {
        return FALSE;
    }
}

/**
 * Handler for uploaded files
 *
 * Returns associative array:
 * $movefile[file] The local path to the uploaded file.
 * $movefile[url] The public URL for the uploaded file.
 * $movefile[type] The MIME type.
 *
 * @param $file
 *
 * @return array|bool
 */
function handleFileUpload($file)
{
    if (!function_exists('wp_handle_upload')) {
        require_once ABSPATH . 'wp-admin/includes/file.php';
    }
    $upload_overrides = array('test_form' => FALSE);
    $movefile = wp_handle_upload($file, $upload_overrides);
    if ($movefile) {
        return $movefile;
    } else {
        return FALSE;
    }
}

function date_i18n_tb($format, $value)
{
    $prev = date_default_timezone_get();
    date_default_timezone_set('UTC');
    $return = date_i18n($format, $value);
    date_default_timezone_set($prev);

    return $return;
}

function isThereOneCouponsAtLeast($service_id)
{
    $promotions = Database\Promotions::getByClassAndService('coupon', $service_id, TRUE, TRUE);
    if (empty($promotions)) {
        return FALSE;
    } else {
        return TRUE;
    }
}

/**
 * @param \TeamBookingType $service
 * @param                  $code
 *
 * @return array|bool
 */
function getPriceWithCoupon(\TeamBookingType $service, $code)
{
    $promotions = Database\Promotions::getByClassAndService('coupon', $service->getId(), TRUE, TRUE);
    foreach ($promotions as $id => $promotion) {
        if ($promotion->getName() == $code) {
            $discount_used = array(
                'name'  => $promotion->getName(),
                'value' => $promotion->getDiscount(),
                'type'  => $promotion->getDiscountType(),
            );
            $service_price = getDiscountedPrice($service);
            if ($promotion->getDiscountType() === 'percentage') {
                $value = $service_price - ($service->getPrice() - Toolkit\applyPercentage($service->getPrice(), $promotion->getDiscount()));
            } else {
                $value = $service_price - $promotion->getDiscount();
                if ($value < 0) {
                    $value = 0;
                }
            }

            return array(
                'discounted' => $value,
                'promotion'  => $discount_used
            );
        }
    }

    return FALSE; // nothing to return? The coupon is invalid or expired
}

/**
 * Only for campaigns!
 *
 * @param \TeamBookingType $service
 *
 * @return int
 */
function getDiscountedPrice(\TeamBookingType $service)
{
    $promotions = Database\Promotions::getByService($service->getId(), TRUE, TRUE);
    $base_price = $service->getPrice();
    $discounted_price = $base_price;
    foreach ($promotions as $promotion) {
        if (!($promotion instanceof \TeamBooking_Promotions_Campaign)) {
            continue;
        }
        if ($promotion->getDiscountType() === 'percentage') {
            $discount = $discounted_price - Toolkit\applyPercentage($base_price, $promotion->getDiscount());
        } else {
            $discount = $promotion->getDiscount();
        }
        $discounted_price -= $discount;
    }
    if ($discounted_price < 0) {
        $discounted_price = 0;
    }

    return $discounted_price;
}

/**
 * Check if the (current|given) user is
 * either an Admin or a Coworker
 *
 * @param null $user_id
 *
 * @return bool
 */
function isAdminOrCoworker($user_id = NULL)
{
    if (NULL === $user_id) {
        if (current_user_can('manage_options')
            || current_user_can('tb_can_sync_calendar')
        ) {
            return TRUE;
        } else {
            return FALSE;
        }
    } else {
        $user = get_user_by('id', $user_id);
        if (!$user) {
            return FALSE;
        }
        if ($user->has_cap('manage_options')
            || $user->has_cap('tb_can_sync_calendar')
        ) {
            return TRUE;
        } else {
            return FALSE;
        }
    }
}

/**
 * Sends the reminder e-mail messages to customers
 *
 * @return bool
 */
function tbSendEmailReminder()
{
    $reservations = Database\Reservations::getAll();
    $local_time = Toolkit\getNowInSecondsUTC();
    foreach ($reservations as $db_id => $data) {
        if (!$data->isConfirmed()) continue;
        try {
            $service = getSettings()->getService($data->getServiceId());
        } catch (\Exception $e) {
            continue;
        }
        if ($service->isSendReminderEmail()) {
            $reference_time = $data->getStart(); // Unix
            $creation_time = $data->getCreationInstant(); // Unix
            $timeframe = $service->getSendReminderEmail();
            $timeframe = isset($timeframe['timeframe']) ? ($timeframe['timeframe'] * DAY_IN_SECONDS) : DAY_IN_SECONDS;
            if ($reference_time - $local_time < $timeframe
                && $reference_time - $local_time > 0
                && !$data->isEmailReminderSent()
            ) {
                if ($reference_time - $creation_time < DAY_IN_SECONDS
                    && $service->getConfirm()
                ) {
                    continue; // too early
                }
                $email = new \TeamBooking\EmailHandler();
                $subject = Toolkit\findAndReplaceHooks($service->getReminderEmail()->getSubject(), $data->getHooksArray(TRUE));
                $body = Toolkit\findAndReplaceHooks($service->getReminderEmail()->getBody(), $data->getHooksArray(TRUE));
                $email->setSubject($subject);
                $email->setBody($body);
                $email->setFrom($service->getEmailForNotifications(), get_bloginfo('name'));
                $email->setTo($data->getCustomerEmail());
                $email->send();
                $data->setEmailReminderSent(TRUE);
                Database\Reservations::update($data);
            }
        }
    }

    return TRUE;
}

/**
 * Sends the reminder e-mail message to customer (manual)
 *
 * @param $reservation_id
 *
 * @return bool
 */
function sendEmailReminderManually($reservation_id)
{
    $reservation = Database\Reservations::getById($reservation_id);
    if (!$reservation->isEmailReminderSent()
        && $reservation->isConfirmed()
    ) {
        try {
            $service = getSettings()->getService($reservation->getServiceId());
        } catch (\Exception $e) {
            return FALSE;
        }
        if ($service->isSendReminderEmail()) {
            if (!isReservationPastInTime($reservation)) {
                $email = new \TeamBooking\EmailHandler();
                $subject = Toolkit\findAndReplaceHooks($service->getReminderEmail()->getSubject(), $reservation->getHooksArray(TRUE));
                $body = Toolkit\findAndReplaceHooks($service->getReminderEmail()->getBody(), $reservation->getHooksArray(TRUE));
                $email->setSubject($subject);
                $email->setBody($body);
                $email->setFrom($service->getEmailForNotifications(), get_bloginfo('name'));
                $email->setTo($reservation->getCustomerEmail());
                $email->send();
                $reservation->setEmailReminderSent(TRUE);
                Database\Reservations::update($reservation);

                return TRUE;
            }

            return FALSE;
        }

        return FALSE;
    }

    return FALSE;
}

/**
 * Retrieve a Coworker by his API token
 *
 * @param $ctoken
 *
 * @return bool|\TeamBookingCoworker
 */
function getCoworkerFromApiToken($ctoken)
{
    foreach (getSettings()->getCoworkersData() as $coworker) {
        if ($coworker->getApiToken() == $ctoken) return $coworker;
    }

    return FALSE;
}

/////////////////////
//  CUSTOM HOOKS   //
/////////////////////

/**
 * @param $params \TeamBooking\RenderParameters
 */
function calendar_click_on_day($params)
{
    do_action('tbk_calendar_click_on_day', $params);
}

/**
 * @param $slot \TeamBooking\Slot
 */
function schedule_slot_parse($slot)
{
    do_action('tbk_schedule_slot_parse', $slot);
}

/**
 * @param $slot \TeamBooking\Slot
 */
function schedule_slot_render($slot)
{
    do_action('tbk_schedule_slot_render', $slot);
}

/**
 * @param $data \TeamBooking_ReservationData
 */
function reservation_before_processing($data)
{
    do_action('tbk_reservation_before_processing', $data);
}

/**
 * @param \TeamBooking\Frontend\Form $form
 */
function reservation_form_header($form)
{
    do_action('tbk_reservation_form_header', $form);
}

/**
 * @param \TeamBooking\Frontend\Form $form
 */
function reservation_form_description($form)
{
    do_action('tbk_reservation_form_description', $form);
}

/**
 * @param \TeamBooking\Frontend\Form $form
 */
function reservation_form_map($form)
{
    do_action('tbk_reservation_form_map', $form);
}

function reservation_review_header($text)
{
    do_action('tbk_reservation_review_header', $text);
}

function reservation_review_details($data)
{
    do_action('tbk_reservation_review_details', $data);
}

function reservation_review_footer($service)
{
    do_action('tbk_reservation_review_footer', $service);
}

function api_page_main_content($content)
{
    echo apply_filters('tbk_api_page_main_content', $content);
}