<?php

namespace TeamBooking\Admin\Framework;

class Modal implements Element
{
    protected $id = '';
    protected $show_buttons = TRUE;
    protected $close_only = FALSE;
    protected $header_text = array(
        'main' => '',
        'sub'  => ''
    );
    protected $button_text = array(
        'approve'   => '',
        'deny'      => '',
        'secondary' => ''
    );
    protected $content = array();

    public function __construct($id)
    {
        $this->id = $id;
        $this->button_text['approve'] = __('Save', 'team-booking');
        $this->button_text['deny'] = __('Close', 'team-booking');
    }

    public function setButtonText(array $text)
    {
        if (isset($text['approve'])) $this->button_text['approve'] = $text['approve'];
        if (isset($text['deny'])) $this->button_text['deny'] = $text['deny'];
        if (isset($text['secondary'])) $this->button_text['secondary'] = $text['secondary'];
    }

    public function setHeaderText(array $text)
    {
        if (isset($text['main'])) $this->header_text['main'] = $text['main'];
        if (isset($text['sub'])) $this->header_text['sub'] = $text['sub'];
    }

    public function setButtons($bool)
    {
        $this->show_buttons = (bool)$bool;
    }

    public function closeOnly($bool)
    {
        $this->close_only = (bool)$bool;
    }

    public function addContent($content)
    {
        $this->content[] = $content;
    }

    public function render()
    {
        echo '<div class="ui small modal tbk-modal" id="' . $this->id . '">';
        echo '<i class="close tb-icon"></i>';
        echo '<div class="header">' . esc_html($this->header_text['main']);
        if (!empty($this->header_text['sub'])) {
            echo ' <div class="sub header">'
                . ($this->header_text['sub'] instanceof Element ? $this->header_text['sub']->get() : esc_html($this->header_text['sub']))
                . '</div>';
        }
        echo '</div>';
        echo '<div class="content" style="width: calc(100% - 3rem);">';
        foreach ($this->content as $item) {
            if ($item instanceof Element) {
                $item->render();
            } else {
                echo $item;
            }
        }
        echo '</div>';
        if ($this->show_buttons) {
            echo '<div class="actions">';
            if (!$this->close_only) {
                echo '<div class="ui positive button" style="height:auto;position: relative;">';
                echo esc_html($this->button_text['approve']);
                if (!empty($this->button_text['secondary'])) {
                    echo '</div><div class="ui ok blue button tbk-secondary" style="height:auto;position: relative;">';
                    echo esc_html($this->button_text['secondary']);
                }
                echo '</div>';
            }
            echo '<div class="ui deny black button" style="height:auto;position: relative;">';
            echo esc_html($this->button_text['deny']);
            echo '</div></div>';
        }
        echo '</div>';
    }
}