<?php

namespace TeamBooking\Admin;
defined('ABSPATH') or die("No script kiddies please!");

class Misc
{
    /**
     * @var Misc
     */
    private static $_instance;

    //------------------------------------------------------------

    public static function render()
    {
        if (!(self::$_instance instanceof self)) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }

    /**
     * Generate the tab wrapper for admin page
     *
     * @param string $active_tab
     */
    public function getTabWrapper($active_tab)
    {
        $header = new Framework\Header();
        $header->setPluginData(TEAMBOOKING_FILE_PATH);
        $header->setMainText('Hi, ' . wp_get_current_user()->user_firstname);
        $header->addTab('overview', __('Overview', 'team-booking'), 'dashicons-chart-area', $active_tab == 'overview');
        $header->addTab('events', __('Services', 'team-booking'), 'dashicons-clipboard', $active_tab == 'events');
        $header->addTab('coworkers', __('Coworkers', 'team-booking'), 'dashicons-groups', $active_tab == 'coworkers');
        $header->addTab('customers', __('Customers', 'team-booking'), 'dashicons-id', $active_tab == 'customers');
        $header->addTab('personal', __('Personal', 'team-booking'), 'dashicons-businessman', $active_tab == 'personal');
        $header->addTab('aspect', __('Frontend style', 'team-booking'), 'dashicons-art', $active_tab == 'aspect');
        $header->addTab('general', __('Core settings', 'team-booking'), 'dashicons-admin-generic', $active_tab == 'general');
        $header->addTab('payments', __('Payment gateways', 'team-booking'), 'dashicons-cart', $active_tab == 'payments');
        $header->addTab('pricing', __('Promotions', 'team-booking'), 'dashicons-money', $active_tab == 'pricing');
        $header->render();
    }

    /**
     * Generate the tab wrapper for admin page (coworker)
     *
     * @param string $active_tab
     */
    public function getTabWrapperCoworker($active_tab)
    {
        $header = new Framework\Header();
        $header->setPluginData(TEAMBOOKING_FILE_PATH);
        $header->setMainText('Hi, ' . wp_get_current_user()->user_firstname);
        $header->addTab('overview', __('Overview', 'team-booking'), 'dashicons-chart-area', $active_tab == 'overview');
        $header->addTab('events', __('Services', 'team-booking'), 'dashicons-clipboard', $active_tab == 'events');
        $header->addTab('personal', __('Personal', 'team-booking'), 'dashicons-businessman', $active_tab == 'personal');
        $header->render();
    }

    public function getWhatsnewPage()
    {
        ?>
        <div class="wrap about-wrap">
        <h1><?= esc_html__('Welcome to TeamBooking 2.1', 'team-booking') ?></h1>
        <p class="about-text">
            <?= esc_html__('Thank you for updating to the latest version.', 'team-booking') ?>
        </p>
        <div class="wp-badge"
             style="    background: url(<?= TEAMBOOKING_URL . '/images/logo-white.png' ?>) center 45px no-repeat #0073aa;">
            Ver. <?= TEAMBOOKING_VERSION ?>
        </div>

        <div class="changelog point-releases">
            <h3><?= esc_html__('What you must do now', 'team-booking') ?></h3>
            <p>
                <?= esc_html__('This version applies major changes to the database.', 'team-booking') ?>
                <a href="<?= admin_url('admin.php?page=team-booking&tab=coworkers') ?>"><?= esc_html__('All the Google Calendars of your Coworkers must be re-synced after the upgrade.', 'team-booking') ?></a>
            </p>
        </div>

        <div class="feature-section two-col">
            <h2><?= esc_html__('Frontend improvements', 'team-booking') ?></h2>
            <div class="col">
                <div>
                    <img src="<?= TEAMBOOKING_URL . '/images/feature-directions.png' ?>" alt="Directions"
                         style="border: 5px solid lightgrey;box-sizing: border-box;">
                </div>
                <h3><?= esc_html__('Google Maps real-time directions', 'team-booking') ?></h3>
                <p>
                    <?= esc_html__('If your service has a location and the reservation form asks the customer for his address, then as he type TeamBooking shows real-time directions.', 'team-booking') ?>
                </p>
            </div>
            <div class="col">
                <img src="<?= TEAMBOOKING_URL . '/images/feature-stripe.png' ?>" alt="Stripe"
                     style="border: 5px solid lightgrey;box-sizing: border-box;">
                <h3><?= esc_html__('Trustworthy credit card form', 'team-booking') ?></h3>
                <p>
                    <?= esc_html__('A little step towards, with Stripe. Auto-formatting and real-time auto detection of the credit card type makes the customer feel more secure.', 'team-booking') ?>
                </p>
            </div>
        </div>

        <div class="feature-section two-col">
            <h2><?= esc_html__('Backend improvements', 'team-booking') ?></h2>
            <div class="col">
                <div>
                    <img src="<?= TEAMBOOKING_URL . '/images/feature-overview.png' ?>" alt="Overview"
                         style="border: 5px solid lightgrey;box-sizing: border-box;">
                </div>
                <h3><?= esc_html__('A more informative reservations list', 'team-booking') ?></h3>
                <p>
                    <?= esc_html__('The reservations list now shows a lot of details at glance and presents a more polished design.', 'team-booking') ?>
                </p>
            </div>
            <div class="col">
                <img src="<?= TEAMBOOKING_URL . '/images/feature-calendars.png' ?>" alt="Stripe"
                     style="border: 5px solid lightgrey;box-sizing: border-box;">
                <h3><?= esc_html__('Google Calendars interdependence', 'team-booking') ?></h3>
                <p>
                    <?= esc_html__('Slots and events inside an independent calendar cannot affect (and cannot be affected by) slots and events that reside in another Google Calendar. Viceversa, a non-independent calendar affects (and is affected by) all other non-independent ones.', 'team-booking') ?>
                </p>
            </div>
        </div>

        <div class="changelog">
            <h2><?= esc_html__('Under the hood', 'team-booking') ?></h2>

            <div class="under-the-hood three-col">
                <div class="col">
                    <h3><?= esc_html__('Database optimization', 'team-booking') ?></h3>
                    <p>
                        <?= esc_html__('A more efficient event storage, for a smaller database footprint and memory usage.', 'team-booking') ?>
                    </p>
                </div>
                <div class="col">
                    <h3><?= esc_html__('Translations', 'team-booking') ?></h3>
                    <p>
                        <?= esc_html__("The built-in fields label are now translated on-the-fly like any other string. TeamBooking is also now compatible with .po translation plugins like Loco Translate.", 'team-booking') ?>
                    </p>
                </div>
                <div class="col">
                    <h3><?= esc_html__('Multisite compatibility', 'team-booking') ?></h3>
                    <p>
                        <?= esc_html__("Networks are now recognized and handled properly during the upgrade phase.", 'team-booking') ?>
                    </p>
                </div>
            </div>
        </div>

        <?php
    }

}
