<?php

namespace TeamBooking\Admin;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Toolkit;

class Overview
{
    public $new_reservation_ids;
    /** @var FilterObject */
    public $filter_object;
    private $settings;
    /** @var \TeamBooking_ReservationData[] */
    private $reservations;

    public function __construct($reservations)
    {
        $this->reservations = $reservations;
        $this->settings = Functions\getSettings();
    }

    //------------------------------------------------------------

    public function getPostBody()
    {
        ob_start();
        ?>
        <div class="tbk-wrapper" xmlns="http://www.w3.org/1999/html">

            <div class="tbk-row">
                <?= $this->getDataExportForms() ?>
                <?php if (!$this->filter_object->getPendingReservations()) { ?>
                    <div class="tbk-column tbk-span-12">
                        <?php $this->getReservationList()->render() ?>
                    </div>
                <?php } else { ?>
                    <div class="tbk-column tbk-span-12">
                        <?php $this->getReservationList()->render() ?>
                    </div>
                <?php } ?>
            </div>

        </div>
        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    public function getReservationList()
    {
        $timezone = Toolkit\getTimezone();

        $panel = new Framework\PanelForList(__('Reservations', 'team-booking'));
        $panel_button = new Framework\PanelTitleAddNewButton(__('Export CSV', 'team-booking'));
        $panel_button->addClass('tb-get-csv');
        $panel->addTitleContent($panel_button);
        $panel_button = new Framework\PanelTitleAddNewButton(__('Export XLSX', 'team-booking'));
        $panel_button->addClass(class_exists('ZipArchive') ? 'tb-get-xlsx' : 'inactive');
        $panel->addTitleContent($panel_button);

        if (current_user_can('manage_options')) {
            $panel_button = new Framework\PanelTitleAddNewButton(__('Delete All', 'team-booking'));
            $panel_button->addClass('delete_all_tb_reservations');
            $panel->addTitleContent($panel_button);
            $panel_button = new Framework\PanelTitleAddNewButton(__('Pending payments', 'team-booking'));
            $panel_button->addClass('tbk-show-pending-payments');
            $filter_entries = clone $this->filter_object;
            $filter_entries->setPendingReservations(!$this->filter_object->getPendingReservations());
            $panel_button->setHref(admin_url('/admin.php?page=team-booking&tab=overview&filter=' . $filter_entries->encode()));
            if ($this->filter_object->getPendingReservations()) $panel_button->addClass('selected');
            $panel->addTitleContent($panel_button);
        }

        ob_start();
        ?>
        <script>
            jQuery('.delete_all_tb_reservations').click(function (e) {
                e.preventDefault();
            });
        </script>
        <?php
        $panel->addElement(Framework\ElementFrom::content(ob_get_clean()));

        ob_start();
        ?>
        <div style="position: absolute;top: 30px;right: 20px;">
            <span class="dashicons dashicons-search tbk-table-search-icon" style="vertical-align: middle;"></span>
            <input type="search" class="regular-text tbk-table-search" style="vertical-align: middle;" data-column="1-7"
                   placeholder="<?= __('Search...', 'team-booking') ?>">
        </div>
        <?php
        $panel->addElement(Framework\ElementFrom::content(ob_get_clean()));

        if ($this->filter_object->getPendingReservations()) {
            ob_start();
            ?>
            <div class="tbk-setting-notice">
                <span><?= esc_html__('Currently showing pending payments reservations.', 'team-booking') ?></span>
            </div>
            <?php
            $panel->addElement(Framework\ElementFrom::content(ob_get_clean()));
        }

        // Confirmation Modal
        $modal = new Framework\Modal('tb-delete-all-reservations-modal');
        $modal->setHeaderText(array('main' => __('Are you sure?', 'team-booking')));
        $modal->setButtonText(array('approve' => __('Yes', 'team-booking'), 'deny' => __('No', 'team-booking')));
        $modal->addContent(Framework\Html::paragraph(__('All the reservations for all the services, regardless of current filtering options, will be removed from the database. This action is permanent.', 'team-booking')));
        $modal->addContent(Framework\Html::paragraph(__('Please note: non-expired-nor-cancelled Appointments Classes will stay booked, non-expired-nor-cancelled Events Classes will regain the tickets, but customers and coworkers will not be notified of that.', 'team-booking')));
        $panel->addElement($modal);

        // Creating the table
        $table = new Framework\Table();
        $table->setId('tbk-reservations-overview');
        $table->setClass('tbk-loading striped');
        $table->setSelectable(TRUE);

        // Preparing the table columns
        $columns_array = array(
            'service'        => esc_html__('Service', 'team-booking'),
            'who'            => esc_html__('Who', 'team-booking'),
            'when'           => esc_html__('Date of service', 'team-booking'),
            'date'           => esc_html__('Date of booking', 'team-booking'),
            'status'         => esc_html__('Status', 'team-booking'),
            'payment status' => esc_html__('Payment Status', 'team-booking')
        );
        if (current_user_can('manage_options')) {
            $columns_array['coworker'] = esc_html__('Coworker', 'team-booking');
        }
        $columns_array['actions'] = esc_html__('Actions', 'team-booking');
        $table->addColumns($columns_array);
        $table->addNoSortableColumn(count($columns_array) - 1);

        // Preparing the filter row: services
        $options_array = array();
        foreach ($this->settings->getServices() as $service) {
            $options_array[ $service->getName() ] = $service->getName();
        }
        $options_array[] = array('text' => '──────────', 'value' => '', 'separator' => TRUE);
        $options_array[] = array('text' => __('All', 'team-booking'), 'value' => '');
        $table->addToFilterRow('service', Framework\ElementFrom::content(Framework\Html::select(array(
            'id'      => 'tbk-overview-filter-service-select',
            'class'   => 'tbk-table-search',
            'data'    => array('column' => 1),
            'options' => $options_array
        ))));
        // Preparing the filter row: status
        $options_array = array(
            __('confirmed', 'team-booking')            => ucfirst(__('confirmed', 'team-booking')),
            __('cancelled', 'team-booking')            => ucfirst(__('cancelled', 'team-booking')),
            __('waiting for approval', 'team-booking') => ucfirst(__('waiting for approval', 'team-booking')),
            array('text' => '──────────', 'value' => '', 'separator' => TRUE),
            array('text' => __('All', 'team-booking'), 'value' => '')
        );
        $table->addToFilterRow('status', Framework\ElementFrom::content(Framework\Html::select(array(
            'id'      => 'tbk-overview-filter-status-select',
            'class'   => 'tbk-table-search',
            'data'    => array('column' => 5),
            'options' => $options_array
        ))));
        // Preparing the filter row: payment status
        $options_array = array(
            __('paid', 'team-booking') . ' OR !=' . __('not paid', 'team-booking') => ucfirst(__('paid', 'team-booking')),
            __('not paid', 'team-booking')                                         => ucfirst(__('not paid', 'team-booking')),
            __('unhandled', 'team-booking')                                        => ucfirst(__('unhandled', 'team-booking')),
            array('text' => '──────────', 'value' => '', 'separator' => TRUE),
            array('text' => __('All', 'team-booking'), 'value' => '')
        );
        $table->addToFilterRow('payment status', Framework\ElementFrom::content(Framework\Html::select(array(
            'id'      => 'tbk-overview-filter-payment-select',
            'class'   => 'tbk-table-search',
            'data'    => array('column' => 6),
            'options' => $options_array
        ))));

        // Preparing the filter row: coworker
        if (current_user_can('manage_options')) {
            $options_array = array();
            foreach (Functions\getAllCoworkersList() as $id => $coworker) {
                $options_array[ $coworker['name'] ] = $coworker['name'];
            }
            $options_array[] = array('text' => '──────────', 'value' => '', 'separator' => TRUE);
            $options_array[] = array('text' => __('All', 'team-booking'), 'value' => '');
            $table->addToFilterRow('coworker', Framework\ElementFrom::content(Framework\Html::select(array(
                'id'      => 'tbk-overview-filter-coworker-select',
                'class'   => 'tbk-table-search',
                'data'    => array('column' => 7),
                'options' => $options_array
            ))));
        }

        // Preparing the table rows
        foreach ($this->reservations as $id => $reservation) {
            if (!$reservation instanceof \TeamBooking_ReservationData) continue;
            if (!current_user_can('manage_options')
                && $reservation->getCoworker() != get_current_user_id()
            ) continue;

            $time = $reservation->getCreationInstant(); // UTC, seconds
            $date_time_object = \DateTime::createFromFormat('U', $time);
            $date_time_object->setTimezone($timezone);
            $date_time_of_reservation_value_raw = $date_time_object->getTimestamp();
            $date_time_of_reservation_value = Functions\date_i18n_tb(get_option('date_format'), $date_time_object->getTimestamp() + $date_time_object->getOffset())
                . Framework\Html::span(array(
                    'text'  => Functions\date_i18n_tb(get_option('time_format'), $date_time_object->getTimestamp() + $date_time_object->getOffset()),
                    'class' => 'tbk-description'
                ));
            if (NULL !== $reservation->getStart()) {
                $date_time_object = \DateTime::createFromFormat('U', $reservation->getStart());
                $date_time_object_end = \DateTime::createFromFormat('U', $reservation->getEnd());
                $date_time_object->setTimezone($timezone);
                $date_time_object_end->setTimezone($timezone);
                $when_value_raw = $date_time_object->getTimestamp();
                $when_value = Functions\date_i18n_tb(get_option('date_format'), $date_time_object->getTimestamp() + $date_time_object->getOffset())
                    . Framework\Html::span(array(
                        'text'  => Functions\date_i18n_tb(get_option('time_format'), $date_time_object->getTimestamp() + $date_time_object->getOffset())
                            . ' - '
                            . Functions\date_i18n_tb(get_option('time_format'), $date_time_object_end->getTimestamp() + $date_time_object_end->getOffset()),
                        'class' => 'tbk-description'
                    ));
            } else {
                $when_value = esc_html__('Unscheduled', 'team-booking');
                $when_value_raw = $date_time_of_reservation_value_raw;
            }
            $row = array(
                'service' => Functions\checkServiceIdExistance($reservation->getServiceId())
                    ? (Framework\Html::anchor(array(
                            'href'  => get_admin_url() . 'admin.php?page=team-booking&tab=events&event=' . $reservation->getServiceId(),
                            'text'  => $reservation->getServiceName(),
                            'class' => 'tbk-to-sort'
                        )) . Framework\Html::span(array(
                            'text'  => Functions\getSettings()->getService($reservation->getServiceId())->getClass(TRUE),
                            'class' => 'tbk-description'
                        )))
                    : $reservation->getServiceName()
                ,
                'who'     => Framework\Html::span(array(
                        'text'  => $reservation->getCustomerDisplayName(),
                        'class' => 'tbk-to-sort'
                    )) . Framework\Html::span(array(
                        'text'  => $reservation->getCustomerEmail(),
                        'class' => 'tbk-description'
                    )),
                'when'    => $when_value,
                'date'    => $date_time_of_reservation_value
            );
            // Row data (reservation id)
            $row['data-row'] = $id;
            // Cell data (sorting by date)
            $row['data-cell']['when'] = array('data' => 'text', 'value' => $when_value_raw);
            $row['data-cell']['date'] = array('data' => 'text', 'value' => $date_time_of_reservation_value_raw);
            // Status column
            try {
                $this->settings->getService($reservation->getServiceId());
                if (Functions\isReservationTimedOut($reservation)
                    && $reservation->isPending()
                ) {
                    $row['status'][] = Framework\Html::span(array(
                        'text'  => __('expired', 'team-booking'),
                        'class' => 'tbk-expired-reservation'
                    ));
                } else {
                    if ($reservation->isConfirmed() && $reservation->getServiceClass() !== 'service') {
                        $row['status'][] = Framework\Html::span(array(
                            'text'  => __('confirmed', 'team-booking'),
                            'class' => 'tbk-confirmed'
                        ));
                        switch ($this->whoIs($reservation, $reservation->getConfirmationWho())) {
                            case 'admin':
                                $row['status'][] = Framework\Html::span(array(
                                    'text'  => __('by Admin', 'team-booking'),
                                    'class' => 'tbk-description'
                                ));
                                break;
                            case 'coworker':
                                $row['status'][] = Framework\Html::span(array(
                                    'text'  => __('by Coworker', 'team-booking'),
                                    'class' => 'tbk-description'
                                ));
                                break;
                            case 'API':
                                $row['status'][] = Framework\Html::span(array(
                                    'text'  => __('via API', 'team-booking'),
                                    'class' => 'tbk-description'
                                ));
                                break;
                            default:
                                break;
                        }
                    }
                    if ($reservation->isConfirmed() && $reservation->getServiceClass() === 'service')
                        $row['status'][] = Framework\TextLabel::blue(__('todo', 'team-booking'));
                    if ($reservation->isDone())
                        $row['status'][] = Framework\Html::span(array(
                            'text'  => __('done', 'team-booking'),
                            'class' => 'tbk-confirmed'
                        ));
                    if ($reservation->isPending())
                        $row['status'][] = Framework\Html::span(array(
                            'text'  => __('on hold', 'team-booking'),
                            'class' => 'tbk-pending'
                        ));
                    if ($reservation->isCancelled()) {
                        $row['status'][] = Framework\Html::span(array(
                            'text'  => __('cancelled', 'team-booking'),
                            'class' => 'tbk-cancelled'
                        ));
                        switch ($this->whoIs($reservation, $reservation->getCancellationWho())) {
                            case 'admin':
                                $row['status'][] = Framework\Html::span(array(
                                    'text'  => __('by Admin', 'team-booking'),
                                    'class' => 'tbk-description'
                                ));
                                break;
                            case 'coworker':
                                $row['status'][] = Framework\Html::span(array(
                                    'text'  => __('by Coworker', 'team-booking'),
                                    'class' => 'tbk-description'
                                ));
                                break;
                            case 'customer':
                                $row['status'][] = Framework\Html::span(array(
                                    'text'  => __('by Customer', 'team-booking'),
                                    'class' => 'tbk-description'
                                ));
                                break;
                            case 'API':
                                $row['status'][] = Framework\Html::span(array(
                                    'text'  => __('via API', 'team-booking'),
                                    'class' => 'tbk-description'
                                ));
                                break;
                            default:
                                break;
                        }
                    }
                    if ($reservation->isWaitingApproval())
                        $row['status'][] = Framework\Html::span(array(
                            'text'  => __('waiting for approval', 'team-booking'),
                            'class' => 'tbk-pending'
                        ));
                }
            } catch (\Exception $ex) {
                $row['status'] = Framework\TextLabel::black(__('service removed', 'team-booking'));
            }
            // Payment status column
            if ($reservation->isPending() && !$reservation->isPaid()) {
                if (!Functions\isReservationTimedOut($reservation)) {
                    $row['payment status'][] = Framework\Html::span(array(
                        'text'  => __('pending', 'team-booking'),
                        'class' => 'tbk-pending tbk-to-sort'
                    ));
                } else {
                    $row['payment status'][] = Framework\Html::span(array(
                        'text'  => __('expired', 'team-booking'),
                        'class' => 'tbk-expired-reservation tbk-to-sort'
                    ));
                }
                $row['payment status'][] = Framework\Html::span(array(
                    'text'  => Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), $reservation->getPriceIncremented() * $reservation->getTickets()),
                    'class' => 'tbk-description'
                ));
            } else {
                if ($reservation->isPaid()) {
                    $row['payment status'][] = Framework\Html::span(array(
                        'text'  => __('paid', 'team-booking'),
                        'class' => 'tbk-paid tbk-to-sort'
                    ));
                    if ($reservation->getPaymentGateway()) {
                        $row['payment status'][] = Framework\Html::span(array(
                            'text'  => ' ' . $this->settings->getPaymentGatewaySettingObject($reservation->getPaymentGateway())->getLabel(),
                            'class' => 'tbk-paid-via'
                        ));
                    }
                    $row['payment status'][] = Framework\Html::span(array(
                        'text'  => Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), $reservation->getPriceIncremented() * $reservation->getTickets()),
                        'class' => 'tbk-description'
                    ));
                } elseif ($reservation->getPrice() > 0) {
                    if ($reservation->getPaymentMustBeManuallyConfirmedOnPayPal()) {
                        $row['payment status'][] = Framework\TextLabel::yellow(__('must be manually confirmed on PayPal', 'team-booking'));
                    } else {
                        $row['payment status'][] = Framework\Html::span(array(
                            'text'  => __('not paid', 'team-booking'),
                            'class' => 'tbk-not-paid tbk-to-sort'
                        ));
                        $row['payment status'][] = Framework\Html::span(array(
                            'text'  => Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), $reservation->getPriceIncremented() * $reservation->getTickets()),
                            'class' => 'tbk-description'
                        ));
                    }
                } else {
                    $row['payment status'] = NULL;
                }
            }

            if (current_user_can('manage_options')) {
                $row_content = Framework\Html::span(array(
                    'text'  => ucwords($this->settings->getCoworkerData($reservation->getCoworker())->getDisplayName()),
                    'class' => 'tbk-to-sort'
                ));
                if ($reservation->getCoworker() == get_current_user_id()) $row_content .= ' (' . esc_html__('you', 'team-booking') . ')';
                $row_content .= Framework\Html::span(array(
                    'text'  => $this->settings->getCoworkerData($reservation->getCoworker())->getEmail(),
                    'class' => 'tbk-description'
                ));
                $row['coworker'] = $row_content;
            }

            // Actions column
            $row['actions'][] = $this->getReservationActions($id, $reservation, $this->filter_object->getPendingReservations());
            if (!current_user_can('manage_options')
                && $reservation->getCoworker() != get_current_user_id()
            ) continue;
            if (NULL !== $reservation->getStart()) {
                $when_value = Functions\date_i18n_tb(get_option('date_format'), $date_time_object->getTimestamp() + $date_time_object->getOffset())
                    . ' '
                    . Functions\date_i18n_tb(get_option('time_format'), $date_time_object->getTimestamp() + $date_time_object->getOffset())
                    . ' - '
                    . Functions\date_i18n_tb(get_option('time_format'), $date_time_object_end->getTimestamp() + $date_time_object_end->getOffset());
            } else {
                $when_value = esc_html__('Unscheduled', 'team-booking');
            }
            ob_start();
            $this->getReservationDetailsModal($reservation, $when_value);
            if ($reservation->getServiceClass() === 'service'
                || ($reservation->getPrice() > 0 && !$this->filter_object->getPendingReservations() && current_user_can('manage_options'))
            ) {
                // modify reservation modal
                $modal = new Framework\Modal('tb-modal-modify-' . $time);
                $modal->setHeaderText(array('main' => __('Modify reservation', 'team-booking')));
                $modal->setButtonText(array('approve' => __('Save changes', 'team-booking'), 'deny' => __('Cancel', 'team-booking')));
                $modal->addContent('<form action="' . admin_url('admin-post.php') . '" method="POST" id="tb-modal-modify-form-' . $time . '">');
                $modal->addContent('<input type="hidden" name="action" value="option_submit">');
                $modal->addContent(wp_nonce_field('team_booking_options_verify', '_wpnonce', TRUE, FALSE));
                $modal->addContent('<input type="hidden" name="tb_modify_reservation_save" value="">');
                $modal->addContent('<input type="hidden" name="reservation_database_id" value="' . $id . '">');
                if (current_user_can('manage_options') && !$reservation->isPending()) {
                    $radios = new Framework\PanelSettingRadios();
                    $radios->addLegend(__('Payment status', 'team-booking'));
                    $radios->addFieldname('reservation_payment_status');
                    $radios->addOption(array(
                        'label'   => __('paid', 'team-booking'),
                        'checked' => $reservation->isPaid(),
                        'value'   => 1
                    ));
                    $radios->addOption(array(
                        'label'   => __('not paid', 'team-booking'),
                        'checked' => !$reservation->isPaid(),
                        'value'   => 0
                    ));
                    $modal->addContent($radios);
                }
                if ($reservation->getServiceClass() === 'service'
                    && ($reservation->isConfirmed() || $reservation->isDone())
                ) {
                    if (current_user_can('manage_options') && !$reservation->isPending()) {
                        $modal->addContent('<hr>');
                    }
                    $radios = new Framework\PanelSettingRadios();
                    $radios->addLegend(__('Status', 'team-booking'));
                    $radios->addFieldname('reservation_unscheduled_status');
                    $radios->addOption(array(
                        'label'   => __('done', 'team-booking'),
                        'checked' => $reservation->isDone(),
                        'value'   => 1
                    ));
                    $radios->addOption(array(
                        'label'   => __('todo', 'team-booking'),
                        'checked' => !$reservation->isDone(),
                        'value'   => 0
                    ));
                    $modal->addContent($radios);
                }
                $modal->addContent('</form>');
                $modal->render();
                ?>
                <script>
                    jQuery('#tb-modify-<?= $time ?>').click(function (event) {
                        event.preventDefault();
                    });
                    jQuery('#tb-modal-modify-<?= $time ?>')
                        .uiModal({
                            blurring  : true,
                            transition: 'fade up'
                        })
                        .uiModal('attach events', '#tb-modify-<?= $time ?>', 'show')
                        .uiModal('setting', 'onApprove', function ($clicked) {
                            $clicked.addClass('loading');
                            jQuery('#tb-modal-modify-form-<?= $time ?>').submit();
                            return false;
                        })
                    ;
                </script>
            <?php }
            $row['actions'][] = ob_get_clean();

            if (in_array($id, $this->new_reservation_ids)) $row['class'] = 'new-reservation';
            $table->addRow($row);
        }

        $panel->addElement($table);

        ob_start();
        ?>
        <script>
            jQuery(document).ready(function ($) {
                $('.tbk-table-search')
                    .on('keydown', function (event, config) {
                        if (event.which !== 8) {
                            $('.tbk-table-search-icon').removeClass('dashicons-search').addClass('dashicons-update').css('color', '#D54E21');
                            $('#tbk-reservations-overview').addClass('tbk-loading');
                        }
                    });
                $("#tbk-reservations-overview")
                    .on('filterEnd', function (event, config) {
                        $('.tbk-table-search-icon').removeClass('dashicons-update').addClass('dashicons-search').css('color', '');
                        $('#tbk-reservations-overview').removeClass('tbk-loading');
                    })
                    .tablesorter({
                        theme         : 'teambooking',
                        headers       : {
                            '.no-sort': {sorter: false}
                        },
                        textExtraction: {
                            1: function (node, table, cellIndex) {
                                return $(node).find('.tbk-to-sort').text();
                            },
                            2: function (node, table, cellIndex) {
                                return $(node).find('.tbk-to-sort').text();
                            },
                            3: function (node, table, cellIndex) {
                                return $(node).data('text');
                            },
                            4: function (node, table, cellIndex) {
                                return $(node).data('text');
                            },
                            6: function (node, table, cellIndex) {
                                return $(node).find('.tbk-to-sort').text();
                            },
                            7: function (node, table, cellIndex) {
                                return $(node).find('.tbk-to-sort').text();
                            }
                        },
                        initialized   : function (table) {
                            $(table).removeClass('tbk-loading');
                        },
                        cssIgnoreRow  : 'ignore-row',
                        sortList      : [[4, 1]],
                        widgets       : ['filter', 'pager', 'reflow'],
                        widgetOptions : {
                            filter_external     : '.tbk-table-search',
                            filter_columnFilters: false,
                            pager_output        : '{page} <?= esc_html__('of', 'team-booking') ?> {totalPages}',
                            pager_size          : 15,
                            pager_removeRows    : false,
                            pager_selectors     : {
                                container  : '.tablenav-pages',
                                first      : '.first-page',
                                prev       : '.prev-page',
                                next       : '.next-page',
                                last       : '.last-page',
                                pageDisplay: '.paging-input',
                                pageSize   : '.paging-size',
                                gotoPage   : '.paging-goto'
                            }
                        }
                    })
                ;
            });
        </script>

        <?php
        if (current_user_can('manage_options')) {
            // Delete Modal Markup
            $modal = new Framework\Modal('tb-reservation-delete-modal');
            $modal->setHeaderText(array('main' => __('Are you sure?', 'team-booking')));
            $modal->setButtonText(array('approve' => __('Yes', 'team-booking'), 'deny' => __('No', 'team-booking')));
            $modal->addContent(esc_html__('The reservation(s) will be removed from the database.', 'team-booking'));
            $modal->addContent(Framework\Html::paragraph(__('Please note: non-expired-nor-cancelled Appointments Classes will stay booked, non-expired-nor-cancelled Events Classes will regain the tickets, but customers and coworkers will not be notified of that.', 'team-booking')));
            $modal->render();
        }
        // Cancellation Modal Markup
        $modal = new Framework\Modal('tb-reservation-cancel-modal');
        $modal->setHeaderText(array('main' => __('Are you sure?', 'team-booking')));
        $modal->setButtonText(array('approve' => __('Yes', 'team-booking'), 'deny' => __('No', 'team-booking')));
        $modal->addContent(Framework\Html::span(array('class' => 'previously-confirmed', 'text' => __('The reservation will be revoked, and the slot will be freed.', 'team-booking'))));
        $modal->addContent('<div>');
        $modal->addContent(Framework\Html::paragraph(__('Add a reason (optional, will be added to the cancellation email):', 'team-booking')));
        $modal->addContent('<textarea style="width:100%;" id="reason"></textarea>');
        $modal->addContent('</div>');
        $modal->addContent(Framework\Html::paragraph(__('Please note: non-expired-nor-cancelled Appointments Classes will stay booked, non-expired-nor-cancelled Events Classes will regain the tickets, but customers and coworkers will not be notified of that.', 'team-booking')));
        $modal->render();
        ?>

        <!--Pagination Markup-->
        <div class="tablenav">
            <div id="selected-reservations-actions" style="float:left;display: none;" class="displaying-num">
                <?= Framework\Html::span(array('id' => 'selected-reservations-number')) ?>
                <?= strtolower(esc_html__('selected', 'team-booking')) ?>
                <?php if (current_user_can('manage_options')) { ?>
                    (<?= Framework\Html::anchor(array(
                        'id'   => 'remove-selected-reservations',
                        'text' => strtolower(__('delete', 'team-booking')),
                        'data' => $this->filter_object->getPendingReservations() ? array('pending' => 1) : array()
                    )) ?>)
                <?php } ?>
            </div>
            <!-- Select services scripts -->
            <script>
                jQuery(document).ready(function ($) {
                    $('#tbk-reservations-overview').on('change', '.tb-table-select-row', function () {
                        var checked = $('#tbk-reservations-overview').find('.tb-table-select-row:checked').length;
                        if (checked > 0) {
                            $('#selected-reservations-actions').show();
                            $('#selected-reservations-number').html(checked);
                        } else {
                            $('#selected-reservations-actions').hide();
                            $('#selected-reservations-number').html(checked);
                        }
                    });
                });
            </script>

            <div class="tablenav-pages">
                <?php
                echo Framework\Html::span(array('class' => 'displaying-num', 'text' => __('Items', 'team-booking')));
                echo Framework\Html::select(array(
                    'class'   => 'paging-size displaying-num',
                    'style'   => 'margin-top: auto',
                    'options' => array(
                        15    => 15,
                        30    => 30,
                        60    => 60,
                        'all' => __('All', 'team-booking')
                    )
                ));
                echo Framework\Html::span(array('class' => 'displaying-num', 'text' => __('Page', 'team-booking')));
                echo Framework\Html::select(array('class' => 'paging-goto displaying-num', 'style' => 'margin-top: auto'));
                echo Framework\Html::anchor(array('class' => 'first-page', 'text' => '«'));
                echo Framework\Html::anchor(array('class' => 'prev-page', 'text' => '‹'));
                echo Framework\Html::span(array('class' => 'paging-input'));
                echo Framework\Html::anchor(array('class' => 'next-page', 'text' => '›'));
                echo Framework\Html::anchor(array('class' => 'last-page', 'text' => '»'));
                ?>
            </div>
        </div>
        <?php
        $panel->addElement(Framework\ElementFrom::content(ob_get_clean()));

        return $panel;
    }

    //------------------------------------------------------------

    /**
     * @param                              $database_id
     * @param \TeamBooking_ReservationData $reservation
     * @param bool|FALSE                   $pending
     *
     * @return string
     * @throws \Exception
     */
    public function getReservationActions($database_id, \TeamBooking_ReservationData $reservation, $pending = FALSE)
    {
        if (!Functions\checkServiceIdExistance($reservation->getServiceId())) {
            /**
             * The service is not present anymore
             * so we'll use the pending reservations actions set
             */
            $pending = TRUE;
        }
        ob_start();

        $button = new Framework\ActionButton('dashicons-editor-alignleft');
        $button->addClass('tbk-reservations-action-details');
        $button->addData('modal', $reservation->getCreationInstant());
        $button->setTitle(__('Details', 'team-booking'));
        $button->render();

        if (current_user_can('manage_options')) {
            $button = new Framework\ActionButton('dashicons-trash');
            $button->addClass('tbk-reservations-action-delete');
            $button->addClass('team-booking-delete-reservation');
            $button->setTitle(__('Delete', 'team-booking'));
            $button->addData('reservationid', $database_id);
            if ($pending) $button->addData('pending', 1);
            $button->render();
        }

        if ($pending && current_user_can('manage_options')) {
            if (!Functions\isReservationPastInTime($reservation)) {
                // Pending reservation actions
                if (Functions\checkServiceIdExistance($reservation->getServiceId())) {
                    $button = new Framework\ActionButton('dashicons-yes');
                    $button->addClass('tbk-reservations-action-approve');
                    $button->addClass('team-booking-confirm-pending-reservation');
                    $button->setTitle(__('Confirm', 'team-booking'));
                    $button->addData('reservationid', $database_id);
                    $button->render();
                }
                // Confirm pending Modal
                $modal = new Framework\Modal('tb-reservation-confirm-pending-modal-' . $database_id);
                $modal->setButtonText(array('approve' => __('Yes', 'team-booking'), 'secondary' => __('Yes and set as paid', 'team-booking'), 'deny' => __('No', 'team-booking')));
                $modal->setHeaderText(array('main' => __('Are you sure you want to confirm this pending reservation?', 'team-booking')));
                $modal->addContent(esc_html__('Accordingly to the service settings, the confirmation and notification e-mail messages will be sent.', 'team-booking'));
                $modal->addContent(Framework\Html::paragraph(array(
                    'text'   => esc_html__('The pending reservation cannot be confirmed due the following error:', 'team-booking') . Framework\Html::span(),
                    'class'  => 'error',
                    'escape' => FALSE
                )));
                $modal->render();
            }
        } else {
            // Normal reservation actions
            if ($reservation->isWaitingApproval()) {
                if ((current_user_can('manage_options')
                        && !Functions\isReservationPastInTime($reservation))
                    || ($this->settings->getService($reservation->getServiceId())->getApproveRule() === 'coworker'
                        && !Functions\isReservationPastInTime($reservation))
                ) {
                    $button = new Framework\ActionButton('dashicons-thumbs-up');
                    $button->addClass('tbk-reservations-action-approve');
                    $button->addClass('team-booking-approve-reservation');
                    $button->setTitle(__('Approve', 'team-booking'));
                    $button->addData('reservationid', $database_id);
                    $button->render();

                    // Approval modal
                    $modal = new Framework\Modal('tb-reservation-approve-modal-' . $database_id);
                    $modal->setHeaderText(array('main' => __('Are you sure you want to confirm this reservation?', 'team-booking')));
                    $modal->setButtonText(array('approve' => __('Yes', 'team-booking'), 'deny' => __('No', 'team-booking')));
                    $modal->addContent(esc_html__('Accordingly to the service settings, the confirmation and notification e-mail messages will be sent.', 'team-booking'));
                    $modal->addContent(Framework\Html::paragraph(array(
                        'text'   => esc_html__('The reservation cannot be approved due the following error:', 'team-booking') . Framework\Html::span(),
                        'class'  => 'error',
                        'escape' => FALSE
                    )));
                    $modal->render();
                }
            }

            if (current_user_can('manage_options')
                || ($this->settings->getService($reservation->getServiceId())->getApproveRule() === 'coworker' && $reservation->isWaitingApproval())
            ) {
                if (!$reservation->isCancelled()
                    && !Functions\isReservationPastInTime($reservation)
                ) {
                    if ($reservation->getServiceClass() !== 'service'
                        || ($reservation->getServiceClass() === 'service' && $reservation->isWaitingApproval())
                    ) {
                        $button = new Framework\ActionButton($reservation->isWaitingApproval() ? 'dashicons-thumbs-down' : 'dashicons-no-alt');
                        $button->addClass('tbk-reservations-action-cancel');
                        $button->addClass('team-booking-cancel-reservation');
                        $button->setTitle($reservation->isWaitingApproval() ? __('Decline', 'team-booking') : __('Cancel', 'team-booking'));
                        $button->addData('reservationid', $database_id);
                        $button->addData('previouslyconfirmed', $reservation->isWaitingApproval() ? 'no' : 'yes');
                        $button->render();
                    }
                }
            }
        }
        if ($reservation->getServiceClass() === 'service'
            || ($reservation->getPrice() > 0 && current_user_can('manage_options'))
        ) {
            $button = new Framework\ActionButton('dashicons-edit');
            $button->setId('tb-modify-' . $reservation->getCreationInstant());
            $button->addClass('tbk-reservations-action-modify');
            $button->addClass('team-booking-modify-reservation');
            $button->setTitle(__('Modify', 'team-booking'));
            $button->render();
        }

        return ob_get_clean();
    }

    //------------------------------------------------------------

    // TODO: remove
    public function getLastErrors()
    {
        ob_start();
        ?>

        <div class="tbk-content">
            <table class="widefat">
                <tbody>
                <tr class="alternate">
                    <th style="font-weight:bold"><?= esc_html__('Message', 'team-booking') ?></th>
                    <th style="font-weight:bold"><?= esc_html__('When', 'team-booking') ?></th>

                    <?php if (current_user_can('manage_options')) { ?>
                        <th style="font-weight:bold"><?= esc_html__('Coworker', 'team-booking') ?></th>
                    <?php } ?>
                </tr>

                <?php
                $error_logs = $this->settings->getErrorLogs();

                if (!empty($error_logs)) {

                    $error_logs = array_reverse($this->settings->getErrorLogs());

                    foreach ($error_logs as $log) {
                        /* @var $log \TeamBooking_ErrorLog */
                        if (!current_user_can('manage_options') && get_current_user_id() != $log->getCoworkerId()) {
                            continue;
                        }
                        ?>
                        <tr>
                            <td>
                                <a href="#" title="<?= $log->getDescription() ?>"><?= $log->getMessage() ?></a>
                            </td>

                            <td>
                                <?= Functions\date_i18n_tb(get_option('date_format') . ' ' . get_option('time_format'), $log->getTimestamp()) ?>
                            </td>

                            <?php if (current_user_can('manage_options')) { ?>
                                <td>
                                    <?= $log->getCoworker() ?>
                                </td>
                            <?php } ?>

                        </tr>
                        <?php
                    }
                }
                ?>
                </tbody>
            </table>
        </div>

        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    public function getStats()
    {
        // retrieve some reservations count
        $non_expired_reservations_num = 0;
        $cancelled_reservations_num = 0;
        $waiting_for_approval_num = 0;
        $at_least_one_service_with_approval = FALSE;
        $total_amount = 0;
        $paid_amount = 0;
        foreach ($this->reservations as $id => $reservation) {
            if (!Functions\checkServiceIdExistance($reservation->getServiceId())) {
                continue;
            }
            if (!current_user_can('manage_options')
                && $reservation->getCoworker() != get_current_user_id()
            ) continue;
            if (!Functions\isReservationPastInTime($reservation)
                ||
                ($reservation->getServiceClass() === 'service' && $reservation->isConfirmed())
            ) {
                $non_expired_reservations_num++;
            }
            if ($reservation->isCancelled()) {
                $cancelled_reservations_num++;
            }
            if ($this->settings->getService($reservation->getServiceId())->getApproveRule() !== 'none') {
                $at_least_one_service_with_approval = TRUE;
            }
            if ($reservation->isWaitingApproval()) {
                $waiting_for_approval_num++;
            }
            if (!$reservation->isCancelled()
                || (
                    $reservation->isCancelled()
                    && $reservation->isPaid()
                )
            ) {
                if ($reservation->getDiscount()) {
                    $total_amount += $reservation->getTickets() * $reservation->getPriceDiscounted();
                } else {
                    $total_amount += $reservation->getTickets() * $reservation->getPrice();
                }
            }
            if ($reservation->isPaid()) {
                if ($reservation->getDiscount()) {
                    $paid_amount += $reservation->getTickets() * $reservation->getPriceDiscounted();
                } else {
                    $paid_amount += $reservation->getTickets() * $reservation->getPrice();
                }
            }
        }
        $currency_symbol = '$';
        foreach (Toolkit\getCurrencies() as $code => $currency) {
            if ($code == $this->settings->getCurrencyCode()) {
                $currency_symbol = $currency['symbol'];
            }
        }
        $string_1 = __('Upcoming', 'team-booking');
        $string_2 = __('Cancelled', 'team-booking');
        $string_3 = __('Total amount', 'team-booking');
        $string_4 = __('Paid amount', 'team-booking');
        $string_5 = __('Waiting for approval', 'team-booking');

        ob_start();
        // Non-expired reservations
        Framework\Stats::get($string_1, $non_expired_reservations_num)->render();
        // Waiting for approval
        if ($at_least_one_service_with_approval || $waiting_for_approval_num > 0) {
            Framework\Stats::get($string_5, $waiting_for_approval_num)->render();
        }
        // Cancelled reservations
        Framework\Stats::get($string_2, $cancelled_reservations_num)->render();
        // Total & paid amount
        if ($total_amount > 0) {
            Framework\Stats::get($string_3, $total_amount . Framework\Html::span(array('text' => $currency_symbol, 'class' => 'currency')))->render();
            Framework\Stats::get($string_4, $paid_amount . Framework\Html::span(array('text' => $currency_symbol, 'class' => 'currency')))->render();
        }

        return ob_get_clean();
    }

    //------------------------------------------------------------

    public
    function getDataExportForms()
    {
        ob_start();
        ?>
        <form id="tb-get-csv-form" method="POST" action="<?= admin_url() . 'admin-post.php' ?>">
            <input type="hidden" name="action" value="option_submit">
            <?php wp_nonce_field('team_booking_options_verify') ?>
            <input type="hidden" name="filter"
                   value="">
            <input type="hidden" name="get_csv_file_tb" value="1">
        </form>

        <form id="tb-get-xlsx-form" method="POST" action="<?= admin_url() . 'admin-post.php' ?>">
            <input type="hidden" name="action" value="option_submit">
            <?php wp_nonce_field('team_booking_options_verify') ?>
            <input type="hidden" name="filter"
                   value="">
            <input type="hidden" name="get_xlsx_file_tb" value="1">
        </form>

        <form id="tb-print-reservation-data-form" method="POST" action="<?= admin_url() . 'admin-post.php' ?>">
            <input type="hidden" name="action" value="option_submit">
            <input type="hidden" name="print-reservation-data_tb" value="1">
            <?php wp_nonce_field('team_booking_options_verify') ?>
        </form>

        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    private
    function getReservationDetailsModal(\TeamBooking_ReservationData $reservation, $when_value)
    {
        $files = $reservation->getFilesReferences();
        ?>
        <div class="ui small modal tbk-reservation-details-modal"
             id='tb-modal-<?= $reservation->getCreationInstant() ?>'
             data-id="<?= $reservation->getCreationInstant() ?>">
            <i class="close tb-icon"></i>

            <div class="header">
                <a class="blue ui label" data-show="tb-customer-details">
                    <?= esc_html__('Customer', 'team-booking') ?>
                </a>
                <?php if ($reservation->isPaid() && NULL !== $reservation->getPaymentDetails()) { ?>
                    |
                    <a class="ui label" data-show="tb-payment-details">
                        <?= esc_html__('Payment', 'team-booking') ?>
                    </a>
                <?php } ?>
                |
                <a class="ui label" data-show="tb-booking-details">
                    <?= esc_html__('Booking', 'team-booking') ?>
                </a>
                <?php if (!empty($files)) { ?>
                    |
                    <a class="ui label" data-show="tb-files-details">
                        <?= esc_html__('Files', 'team-booking') ?>
                    </a>
                <?php } ?>
                <?php if ($reservation->getCancellationReason() || ($reservation->getPendingReason() && $reservation->isPending())) { ?>
                    |
                    <a class="ui label" data-show="tb-other-details">
                        <?= esc_html__('Other', 'team-booking') ?>
                    </a>
                <?php } ?>

                <?php if (current_user_can('manage_options')) { ?>
                    <div class="ui mini circular label tbk-edit-reservation-data" style="float: right;"
                         title="<?= esc_html__('Edit', 'team-booking') ?>">
                        <span class="dashicons dashicons-edit"></span>
                    </div>
                <?php } ?>
                <div class="ui mini circular label tbk-print-reservation-data" style="float: right;"
                     title="<?= esc_html__('Print PDF', 'team-booking') ?>"
                     data-id="<?= $reservation->getDatabaseId() ?>">
                    <span class="dashicons dashicons-download"></span>
                </div>
            </div>

            <div class="content" style="width: calc(100% - 3rem);">
                <!-- Other details part -->
                <div class="tb-data tb-other-details" style="display:none;">
                    <?php if ($reservation->getCancellationReason()) { ?>
                        <h4><?= esc_html__('Cancellation reason', 'team-booking') ?></h4>
                        <div style="font-style: italic;font-weight: 300;">
                            <?= $reservation->getCancellationReason() ?>
                        </div>
                    <?php } ?>
                    <?php if ($reservation->getPendingReason() && $reservation->isPending()) { ?>
                        <h4><?= esc_html__('Pending reason', 'team-booking') ?></h4>
                        <div style="font-style: italic;font-weight: 300;">
                            <?= $reservation->getPendingReason() ?>
                        </div>
                    <?php } ?>
                </div>

                <!-- Customer details part -->
                <div class="tb-data tb-customer-details">
                    <h4><?= esc_html__('Customer details', 'team-booking') ?></h4>
                    <?php
                    foreach ($reservation->getFieldsArray() as $key => $field) {
                        if ($field) {
                            ?>
                            <div style="font-style: italic;font-weight: 300;">
                                <?php
                                try {
                                    $label_from_hook = $this->settings->getService($reservation->getServiceId())->getFormFields()->getLabelFromHook($key);
                                } catch (\Exception $ex) {
                                    $label_from_hook = FALSE;
                                }
                                if ($label_from_hook) {
                                    echo $label_from_hook;
                                } else {
                                    echo $key;
                                }
                                try {
                                    if ($key === 'email'
                                        && $reservation->isConfirmed()
                                        && !Functions\isReservationPastInTime($reservation)
                                        && $this->settings->getService($reservation->getServiceId())->isSendReminderEmail()
                                    ) {
                                        echo ' (' . esc_html__('reminder', 'team-booking');
                                        echo ' <span class="tbk-email-reminder-sent"' . ($reservation->isEmailReminderSent() ? '' : ' style="display:none;"') . '>' . esc_html__('sent', 'team-booking') . '</span>';
                                        echo '<span class="tbk-email-reminder-notsent"' . ($reservation->isEmailReminderSent() ? ' style="display:none;"' : '') . '>' . esc_html__('not sent yet', 'team-booking') . ' - </span>';
                                        if (!$reservation->isEmailReminderSent()) {
                                            echo '<a href="#" data-id="' . $reservation->getDatabaseId() . '" class="tbk-email-reminder-send-manually">' . esc_html__('send now', 'team-booking') . '</a>';
                                        }
                                        echo ')';
                                    }
                                } catch (\Exception $e) {
                                    // nothing
                                }
                                ?>
                            </div>
                            <input type="text" class="large-text" style='margin-bottom:10px;' value="<?= $field ?>"
                                   data-key="<?= $key ?>"
                                   readonly="">
                            <?php
                        }
                    }
                    ?>
                    <div style="font-style: italic;font-weight: 300;">
                        <?= esc_html__("Customer's timezone", 'team-booking') ?>
                    </div>
                    <input type="text" class="large-text noedit" style='margin-bottom:10px;'
                           value="<?= $reservation->getCustomerTimezone() ?>" readonly="">
                    <?php
                    if ($reservation->getCustomerUserId()) {
                        ?>
                        <div style="font-style: italic;font-weight: 300;">
                            <?= esc_html__('WordPress User', 'team-booking') ?>
                        </div>
                        <input type="text" class="large-text noedit" style='margin-bottom:10px;'
                               value="<?= get_userdata($reservation->getCustomerUserId())->user_nicename ?>"
                               readonly="">

                    <?php } ?>

                </div>

                <?php
                if ($reservation->isPaid() && NULL !== $reservation->getPaymentDetails()) {
                    ?>
                    <!-- Payment details part -->
                    <div class="tb-data tb-payment-details" style="display:none;">
                        <h4><?= esc_html__('Payment details', 'team-booking') ?></h4>

                        <div style="font-style: italic;font-weight: 300;">
                            <?= esc_html__('Payment gateway', 'team-booking') ?>
                        </div>
                        <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                               value="<?= $reservation->getPaymentGateway() ?>">
                        <?php foreach ($reservation->getPaymentDetails() as $key => $detail) { ?>
                            <div style="font-style: italic;font-weight: 300;">
                                <?= ucwords($key) ?>
                            </div>
                            <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                                   value="<?= $detail ?>">
                        <?php } ?>
                    </div>
                <?php } ?>

                <!-- Booking details part -->
                <div class="tb-data tb-booking-details" style="display:none;">
                    <h4><?= esc_html__('Booking details', 'team-booking') ?></h4>

                    <div style="font-style: italic;font-weight: 300;">
                        <?= esc_html__('Service', 'team-booking') ?>
                    </div>
                    <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                           value="<?= $reservation->getServiceName() ?>">
                    <?php if ($reservation->getServiceLocation() != '') { ?>
                        <div style="font-style: italic;font-weight: 300;">
                            <?= esc_html__('Location', 'team-booking') ?>
                        </div>
                        <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                               value="<?= $reservation->getServiceLocation() ?>">
                    <?php } ?>
                    <?php if (current_user_can('manage_options')) { ?>
                        <div style="font-style: italic;font-weight: 300;">
                            <?= esc_html__('Coworker', 'team-booking') ?>
                        </div>
                        <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                               value="<?= $this->settings->getCoworkerData($reservation->getCoworker())->getDisplayName() ?>">
                    <?php } ?>
                    <?php if ($reservation->getServiceClass() !== 'service') { ?>
                        <div style="font-style: italic;font-weight: 300;">
                            <?= esc_html__('Date of service', 'team-booking') ?>
                        </div>
                        <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                               value="<?= $when_value ?>">
                        <div style="font-style: italic;font-weight: 300;">
                            <?= esc_html__('Tickets', 'team-booking') ?>
                        </div>
                        <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                               value="<?= $reservation->getTickets() ?>">
                        <?php
                    }
                    ?>
                    <div style="font-style: italic;font-weight: 300;">
                        <?= esc_html__('Total price', 'team-booking') ?>
                    </div>
                    <input type="text" readonly="" class="large-text" style='margin-bottom:10px;'
                           value="<?= Functions\currencyCodeToSymbol($reservation->getCurrencyCode(), $reservation->getTickets() * $reservation->getPriceIncremented()) ?>">

                    <?php if ($reservation->getDiscount()) {
                        $discount_array = $reservation->getDiscount();
                        ?>
                        <div style="font-style: italic;font-weight: 300;">
                            <?= esc_html__('Discounts used', 'team-booking') ?>
                        </div>
                        <?php foreach ($discount_array as $discount) {
                            echo '<span class="ui mini label" style="margin-bottom:10px;">' . $discount['name'] . '</span>';
                        }
                        ?>
                    <?php } ?>
                </div>

                <?php
                if (!empty($files)) {
                    ?>
                    <!-- Files details part -->
                    <div class="tb-data tb-files-details" style="display:none;">
                        <h4><?= esc_html__('Attached files', 'team-booking') ?></h4>
                        <?php foreach ($files as $hook => $file_info_array) { ?>
                            <div style="font-style: italic;font-weight: 300;">
                                <?= ucwords(str_replace('_', ' ', $hook)) ?>
                            </div>
                            <a href="<?= $file_info_array['url'] ?>"><?= esc_html__('Open it', 'team-booking') ?></a>
                        <?php } ?>
                    </div>
                    <?php
                }
                ?>
            </div>

            <div class="actions">
                <div class="ui button green tbk-edit-reservation-data-save"
                     data-id="<?= $reservation->getDatabaseId() ?>"
                     style="height: auto;display: none;">
                    <?= esc_html__('Save', 'team-booking') ?>
                </div>
                <div class="ui button deny black" style="height: auto;">
                    <?= esc_html__('Close', 'team-booking') ?>
                </div>
            </div>
        </div>
        <?php
    }

    //------------------------------------------------------------

    /**
     * @param \TeamBooking_ReservationData $data
     * @param                              $who
     *
     * @return string
     */
    private function whoIs(\TeamBooking_ReservationData $data, $who)
    {
        if ($who === FALSE) {
            return FALSE;
        } elseif (!is_numeric($who)) {
            return 'API';
        } elseif (user_can($who, 'manage_options')) {
            return 'admin';
        } elseif ($who === $data->getCoworker()) {
            return 'coworker';
        } else {
            return 'customer';
        }
    }
}
