<?php

namespace TeamBooking\Admin;

// Blocks direct access to this file
defined('ABSPATH') or die("No script kiddies please!");

use TeamBooking\Functions,
    TeamBooking\Toolkit;

class Payments
{
    private $currency;
    private $gateways;

    public function __construct()
    {
        $this->currency = Functions\getSettings()->getCurrencyCode();
        $this->gateways = Functions\getSettings()->getPaymentGatewaySettingObjects();
    }

    //------------------------------------------------------------

    public function getPostBody()
    {
        ob_start();
        ?>
        <div class="tbk-wrapper">
            <?php
            $row = new Framework\Row();
            $column = Framework\Column::fullWidth();
            $column->addElement(Framework\ElementFrom::content($this->getCurrencyHeader()));
            $column->appendTo($row);
            $row->render();

            $row = new Framework\Row();
            foreach ($this->gateways as $gateway) {
                $column = Framework\Column::ofWidth(6);
                $column->addElement($gateway->getBackendSettingsTab());
                $column->appendTo($row);
            }
            $row->render();
            ?>
        </div>

        <?php
        return ob_get_clean();
    }

    private function getCurrencyHeader()
    {
        ob_start();
        ?>
        <form method="POST" action="<?= admin_url('admin-post.php') ?>">
            <input type="hidden" name="action" value="option_submit">
            <?php wp_nonce_field('team_booking_options_verify') ?>
            <div class="tbk-panel tbk-content">
                <div class="tbk-settings-title">
                    <?= Framework\Html::h3(array('text' => __('Currency', 'team-booking'), 'class' => 'tbk-heading')) ?>

                    <p class="tbk-excerpt">
                        <select name="currency_code">
                            <option value=""><?= esc_html__('Select currency...', 'team-booking') ?></option>
                            <?php foreach (Toolkit\getCurrencies() as $code => $data) { ?>
                                <option value="<?= $code ?>" <?php selected($this->currency, $code, TRUE); ?>>
                                    <?= $code ?> (<?= $data['label'] ?>)
                                </option>
                            <?php } ?>
                        </select>
                    </p>
                    <input type="submit" name="team_booking_payment_options_submit"
                           class="button button-hero button-primary"
                           value="<?= esc_html__('Save changes', 'team-booking') ?>">
                </div>
            </div>
        </form>
        <?php
        return ob_get_clean();
    }

}
