<?php

namespace TeamBooking\Admin;

// Blocks direct access to this file
defined('ABSPATH') or die("No script kiddies please!");

use TeamBooking\Functions,
    TeamBooking\Database;

class Promotion
{
    private $settings;
    private $promotions;
    private $services;
    private $strings = array();

    public function __construct()
    {
        $this->settings = Functions\getSettings();
        $this->promotions = Database\Promotions::getAll();
        $this->services = $this->settings->getServices();
        $this->strings = array(
            'wrong_date_format' => esc_html__('Wrong date or format. Please retry.', 'team-booking')
        );
    }

    //------------------------------------------------------------

    public function getPostBody()
    {
        ob_start();
        ?>
        <div class="tbk-wrapper">

            <div class="tbk-row">
                <div class="tbk-column tbk-span-12">
                    <?php $this->getPromotionsList()->render() ?>
                </div>
            </div>
        </div>

        <script>
            jQuery('.tbk-edit-promotion-modal').find('.tbk-promotion-services').on('click', 'input[type="checkbox"]', function () {
                var modal = jQuery(this).closest('.tbk-edit-promotion-modal');
                var promotion_to_edit = modal.data('promotion');
                if (this.checked) {
                    var base_price = jQuery('#' + promotion_to_edit + '-' + this.value + '-base').data('price');
                    var discount = modal.find('input[name="discount_value"]').val();
                    var discount_type = modal.find('input[name="' + promotion_to_edit + '-discount_type"]:checked').val();
                    if (discount_type == 'percentage') {
                        if (discount > 100) {
                            discount = 100;
                        }
                        var discounted_price = base_price - (base_price * discount / 100);
                    } else {
                        var discounted_price = base_price - discount;
                        if (discounted_price < 0) {
                            discounted_price = 0;
                        }
                    }
                    jQuery('#' + promotion_to_edit + '-' + this.value + '-discounted')
                        .show()
                        .html(discounted_price)
                        .autoNumeric('update')
                    ;
                } else {
                    jQuery('#' + promotion_to_edit + '-' + this.value + '-discounted').html('')
                    ;
                }
            })
        </script>
        <script>
            jQuery('.tbk-edit-promotion-modal')
                .find('input[name$="-discount_type"]')
                .click(function () {
                    var modal = jQuery(this).closest('.tbk-edit-promotion-modal');
                    var promotion_name = modal.data('promotion');
                    var discount = jQuery(this).closest('tr').find('input[name="discount_value"]');
                    if (this.value == 'percentage') {
                        discount.attr('max', 100);
                        if (discount.val() > 100) {
                            discount.val(100);
                        }
                    } else {
                        discount.removeAttr('max');
                    }
                    modal
                        .find('input[type="checkbox"]:checked')
                        .each(function () {
                            var base_price = jQuery('#' + promotion_name + '-' + this.value + '-base').data('price');
                            var discount = modal.find('input[name="discount_value"]').val();
                            var discount_type = modal.find('input[name="' + promotion_name + '-discount_type"]:checked').val();
                            if (discount_type == 'percentage') {
                                if (discount > 100) {
                                    discount = 100;
                                }
                                var discounted_price = base_price - (base_price * discount / 100);
                            } else {
                                var discounted_price = base_price - discount;
                                if (discounted_price < 0) {
                                    discounted_price = 0;
                                }
                            }
                            jQuery('#' + promotion_name + '-' + this.value + '-discounted')
                                .html(discounted_price)
                                .autoNumeric('update')
                            ;
                        })
                });
        </script>
        <script>
            jQuery('.tbk-edit-promotion-modal')
                .find('input[name="discount_value"]')
                .on('change', function () {
                    var modal = jQuery(this).closest('.tbk-edit-promotion-modal');
                    var promotion_name = modal.data('promotion');
                    modal
                        .find('input[type="checkbox"]:checked')
                        .each(function () {
                            var base_price = jQuery('#' + promotion_name + '-' + this.value + '-base').data('price');
                            var discount = modal.find('input[name="discount_value"]').val();
                            var discount_type = modal.find('input[name="' + promotion_name + '-discount_type"]:checked').val();
                            if (discount_type == 'percentage') {
                                if (discount > 100) {
                                    discount = 100;
                                }
                                var discounted_price = base_price - (base_price * discount / 100);
                            } else {
                                var discounted_price = base_price - discount;
                                if (discounted_price < 0) {
                                    discounted_price = 0;
                                }
                            }
                            jQuery('#' + promotion_name + '-' + this.value + '-discounted')
                                .html(discounted_price)
                                .autoNumeric('update')
                            ;
                        })
                    ;
                })
        </script>
        <?php
        echo $this->getAreYouSureModal();

        return ob_get_clean();
    }

    //------------------------------------------------------------

    private function getPromotionsList()
    {
        $panel = new Framework\PanelForList(__('Promotions', 'team-booking'));
        $button = new Framework\PanelTitleAddNewButton(__('New coupon', 'team-booking'));
        $button->setId('add-new-promotion-coupon');
        $panel->addTitleContent($button);
        $panel->addElement($this->getNewPromotionModal('coupon'));
        $button = new Framework\PanelTitleAddNewButton(__('New campaign', 'team-booking'));
        $button->setId('add-new-promotion-campaign');
        $panel->addTitleContent($button);
        $panel->addElement($this->getNewPromotionModal('campaign'));

        $table = new Framework\Table();
        $table->setId('tbk-promotions-list');
        $table->setSelectable(TRUE);
        $table->addColumns(array(
            'name'    => __('Name', 'team-booking'),
            'type'    => __('Type', 'team-booking'),
            'status'  => __('Status', 'team-booking'),
            'start'   => __('Start', 'team-booking'),
            'end'     => __('End', 'team-booking'),
            'actions' => __('Actions', 'team-booking')
        ));

        $now = current_time('timestamp', TRUE);
        foreach ($this->promotions as $id => $promotion) {
            $row = array();
            $row['name'] = $promotion->getName();
            if ($now > $promotion->getEndTime()) {
                $row['status'] = Framework\TextLabel::basic(__('Expired', 'team-booking'));
            } elseif (!$promotion->getStatus()) {
                $row['status'] = Framework\TextLabel::basic(__('Deactivated', 'team-booking'));
            } elseif ($now >= $promotion->getStartTime() && $now <= $promotion->getEndTime()) {
                $row['status'] = Framework\TextLabel::green(__('Running', 'team-booking'));
            } else {
                $row['status'] = Framework\TextLabel::basic(__('Not running', 'team-booking'));
            }
            $row['type'] = ($promotion instanceof \TeamBooking_Promotions_Coupon) ? __('Coupon', 'team-booking') : __('Campaign', 'team-booking');
            $row['start'] = \DateTime::createFromFormat('U', $promotion->getStartTime())->format(get_option('date_format'));
            $row['end'] = \DateTime::createFromFormat('U', $promotion->getEndTime())->format(get_option('date_format'));
            // Actions
            if ($now >= $promotion->getStartTime() && $now <= $promotion->getEndTime()) {
                if ($promotion->getStatus()) {
                    $button = new Framework\ActionButton('dashicons-controls-pause');
                    $button->addClass('tbk-promotion-action-pause');
                    $button->addData('id', $id);
                    $button->addData('promotion', $promotion->getName());
                    $button->setTitle(__('Pause', 'team-booking'));
                    $row['actions'][] = $button;
                } else {
                    $button = new Framework\ActionButton('dashicons-controls-play');
                    $button->addClass('tbk-promotion-action-run');
                    $button->addData('id', $id);
                    $button->addData('promotion', $promotion->getName());
                    $button->setTitle(__('Run', 'team-booking'));
                    $row['actions'][] = $button;
                }
            }
            $button = new Framework\ActionButton('dashicons-trash');
            $button->addClass('tbk-promotion-action-delete');
            $button->addData('id', $id);
            $button->addData('name', $promotion->getName());
            $button->setTitle(__('Delete', 'team-booking'));
            $row['actions'][] = $button;
            $button = new Framework\ActionButton('dashicons-edit');
            $button->addClass('tbk-promotion-action-edit');
            $button->addData('id', $id);
            $button->addData('promotion', $promotion->getName());
            $button->setTitle(__('Edit', 'team-booking'));
            $row['actions'][] = $button;
            $row['actions'][] = $this->getEditPromotionModal($promotion);
            $table->addRow($row);
        }
        $panel->addElement($table);

        return $panel;
    }

    //------------------------------------------------------------

    private function getNewPromotionModal($promotion_type)
    {
        $modal = new Framework\Modal('tb-new-promotion-modal-' . $promotion_type);
        $modal->setHeaderText(array('main' => ($promotion_type == 'coupon') ? __('New coupon', 'team-booking') : __('New campaign', 'team-booking')));
        $modal->setButtonText(array('approve' => __('Add', 'team-booking'), 'deny' => __('Cancel', 'team-booking')));
        ob_start();
        ?>
        <ul class="tbk-list">
            <li>
                <h4><?= esc_html__('Name', 'team-booking') ?></h4>
                <?= Framework\Html::textfield(array('name' => 'promotion_name', 'placeholder' => 'e.g. special-offer', 'required' => TRUE)) ?>
                <p class="error name" style="display:none;">
                    <?= esc_html__('This name is already in use, please provide a fresh one.', 'team-booking') ?>
                </p>
            </li>

            <li>
                <h4><?= esc_html__('Discount', 'team-booking') ?></h4>
                <table>
                    <tbody>
                    <tr>
                        <td>
                            <input type="number" name="discount_value" min="1" step="1" class="small-text"
                                   value="10">
                        </td>
                        <td>
                            <?= Framework\Html::radio(array(
                                'text'    => '%',
                                'name'    => $promotion_type . '-discount_type',
                                'value'   => 'percentage',
                                'checked' => TRUE
                            )) ?>
                            <br>
                            <?= Framework\Html::radio(array(
                                'text'  => Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), NULL),
                                'name'  => $promotion_type . '-discount_type',
                                'value' => 'direct'
                            )) ?>
                        </td>
                    </tr>
                    </tbody>
                </table>
            </li>
            <li>
                <h4><?= esc_html__('Start date', 'team-booking') ?></h4>
                <?= Framework\Html::textfield(array(
                        'name'        => 'start_date',
                        'placeholder' => 'MM/DD/YYYY',
                        'required'    => TRUE,
                        'class'       => 'tb-flatpickr',
                        'data'        => array(
                            'mindate'     => 'today',
                            'dateFormat'  => 'm/d/Y',
                            'altFormat'   => get_option('date_format'),
                            'altinput'    => '',
                            'defaultDate' => 'today',
                            'allowInput'  => ''
                        )
                    )
                ) ?>
                <p class="error start-date" style="display:none;">
                    <?= $this->strings['wrong_date_format'] ?>
                </p>
            </li>
            <li>
                <h4><?= esc_html__('End date', 'team-booking') ?></h4>
                <?= Framework\Html::textfield(array(
                        'name'        => 'end_date',
                        'placeholder' => 'MM/DD/YYYY',
                        'required'    => TRUE,
                        'class'       => 'tb-flatpickr',
                        'data'        => array(
                            'mindate'     => 'today',
                            'dateFormat'  => 'm/d/Y',
                            'altFormat'   => get_option('date_format'),
                            'altinput'    => '',
                            'defaultDate' => 'today',
                            'allowInput'  => ''
                        )
                    )
                ) ?>
                <p class="error end-date" style="display:none;">
                    <?= $this->strings['wrong_date_format'] ?>
                </p>
            </li>
            <li>
                <table class="tbk-promotion-services" style="width: 100%">
                    <tbody>
                    <tr>
                        <th>
                            <h4><?= esc_html__('Services', 'team-booking') ?></h4>
                        </th>
                        <th>
                            <h4><?= esc_html__('Base price', 'team-booking') ?></h4>
                        </th>
                        <th>
                            <h4><?= esc_html__('Discounted price', 'team-booking') ?></h4>
                        </th>
                    </tr>
                    <?php foreach ($this->services as $service) {
                        if ($service->getPrice() <= 0) {
                            continue;
                        }
                        ?>
                        <tr>
                            <td style="border-bottom: 1px dashed lightgrey;">
                                <?= Framework\Html::checkbox(array(
                                    'text'  => $service->getName(),
                                    'name'  => $service->getId(),
                                    'value' => $service->getId()
                                )) ?>
                            </td>
                            <td style="border-bottom: 1px dashed lightgrey;">
                                        <span id="<?= $promotion_type . '-' . $service->getId() ?>-base"
                                              data-price="<?= $service->getPrice() ?>"
                                              class='tbk-promotion-service-base-price'>
                                            <?= Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), $service->getPrice()) ?>
                                        </span>
                            </td>
                            <td style="border-bottom: 1px dashed lightgrey;">
                                        <span id="<?= $promotion_type . '-' . $service->getId() ?>-discounted"
                                              data-a-sign="<?= Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), NULL) ?>"
                                              class='tbk-promotion-service-discounted-price'>
                                        </span>
                            </td>
                        </tr>
                    <?php } ?>
                    </tbody>
                </table>
                <script>
                    jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>').find('.tbk-promotion-services').on('click', 'input[type="checkbox"]', function () {
                        if (this.checked) {
                            var base_price = jQuery('#<?= $promotion_type ?>-' + this.value + '-base').data('price');
                            var discount = jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>').find('input[name="discount_value"]').val();
                            var discount_type = jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>').find('input[name="<?= $promotion_type ?>-discount_type"]:checked').val();
                            if (discount_type == 'percentage') {
                                if (discount > 100) {
                                    discount = 100;
                                }
                                var discounted_price = base_price - (base_price * discount / 100);
                            } else {
                                var discounted_price = base_price - discount;
                                if (discounted_price < 0) {
                                    discounted_price = 0;
                                }
                            }
                            jQuery('#<?= $promotion_type ?>-' + this.value + '-discounted')
                                .html(discounted_price)
                                .autoNumeric('init')
                            ;
                        } else {
                            jQuery('#<?= $promotion_type ?>-' + this.value + '-discounted').html('')
                                .autoNumeric('destroy')
                            ;
                        }
                    })
                </script>
                <script>
                    jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>')
                        .find('input[name="<?= $promotion_type ?>-discount_type"]')
                        .click(function () {
                            var discount = jQuery(this).closest('tr').find('input[name="discount_value"]');
                            if (this.value == 'percentage') {
                                discount.attr('max', 100);
                                if (discount.val() > 100) {
                                    discount.val(100);
                                }
                            } else {
                                discount.removeAttr('max');
                            }
                            jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>')
                                .find('input[type="checkbox"]:checked')
                                .each(function () {
                                    var base_price = jQuery('#<?= $promotion_type ?>-' + this.value + '-base').data('price');
                                    var discount = jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>').find('input[name="discount_value"]').val();
                                    var discount_type = jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>').find('input[name="<?= $promotion_type ?>-discount_type"]:checked').val();
                                    if (discount_type == 'percentage') {
                                        if (discount > 100) {
                                            discount = 100;
                                        }
                                        var discounted_price = base_price - (base_price * discount / 100);
                                    } else {
                                        var discounted_price = base_price - discount;
                                        if (discounted_price < 0) {
                                            discounted_price = 0;
                                        }
                                    }
                                    jQuery('#<?= $promotion_type ?>-' + this.value + '-discounted')
                                        .html(discounted_price)
                                        .autoNumeric('update')
                                    ;
                                })
                        });
                </script>
                <script>
                    jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>')
                        .find('input[name="discount_value"]')
                        .on('change', function () {
                            jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>')
                                .find('input[type="checkbox"]:checked')
                                .each(function () {
                                    var base_price = jQuery('#<?= $promotion_type ?>-' + this.value + '-base').data('price');
                                    var discount = jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>').find('input[name="discount_value"]').val();
                                    var discount_type = jQuery('#tb-new-promotion-modal-<?= $promotion_type ?>').find('input[name="<?= $promotion_type ?>-discount_type"]:checked').val();
                                    if (discount_type == 'percentage') {
                                        if (discount > 100) {
                                            discount = 100;
                                        }
                                        var discounted_price = base_price - (base_price * discount / 100);
                                    } else {
                                        var discounted_price = base_price - discount;
                                        if (discounted_price < 0) {
                                            discounted_price = 0;
                                        }
                                    }
                                    jQuery('#<?= $promotion_type ?>-' + this.value + '-discounted')
                                        .html(discounted_price)
                                        .autoNumeric('update')
                                    ;
                                })
                            ;
                        })
                </script>
            </li>
        </ul>
        <?php
        $modal->addContent(ob_get_clean());

        return $modal;
    }

    //------------------------------------------------------------

    private function getEditPromotionModal(\TeamBooking\Promotions\Promotion $promotion)
    {
        $start_time = \DateTime::createFromFormat('U', $promotion->getStartTime());
        $end_time = \DateTime::createFromFormat('U', $promotion->getEndTime());
        ob_start();
        ?>
        <div class="ui small modal tbk-edit-promotion-modal"
             id="<?= $promotion->getName() ?>-edit-modal"
             data-promotion="<?= $promotion->getName() ?>">
            <i class="close tb-icon"></i>

            <div class="header">
                <?= $promotion->getName() ?>
            </div>

            <div class="content" style="width:calc(100% - 4rem)">
                <ul class="tbk-list">
                    <li>
                        <h4><?= esc_html__('Name', 'team-booking') ?></h4>
                        <?= Framework\Html::textfield(array('name' => 'promotion_name', 'value' => $promotion->getName(), 'required' => TRUE)) ?>
                        <p class="error name" style="display:none;">
                            <?= esc_html__('This name is already in use, please provide a fresh one.', 'team-booking') ?>
                        </p>
                    </li>

                    <li>
                        <h4><?= esc_html__('Discount', 'team-booking') ?></h4>
                        <table>
                            <tbody>
                            <tr>
                                <td>
                                    <input type="number" name="discount_value" min="1" step="1" class="small-text"
                                           value="<?= $promotion->getDiscount() ?>">
                                </td>
                                <td>
                                    <?= Framework\Html::radio(array(
                                        'text'    => '%',
                                        'name'    => $promotion->getName() . '-discount_type',
                                        'value'   => 'percentage',
                                        'checked' => ('percentage' == $promotion->getDiscountType())
                                    )) ?>
                                    <br>
                                    <?= Framework\Html::radio(array(
                                        'text'    => Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), NULL),
                                        'name'    => $promotion->getName() . '-discount_type',
                                        'value'   => 'direct',
                                        'checked' => ('direct' == $promotion->getDiscountType())
                                    )) ?>
                                </td>
                            </tr>
                            </tbody>
                        </table>
                    </li>

                    <li>
                        <h4><?= esc_html__('Start date', 'team-booking') ?></h4>
                        <?= Framework\Html::textfield(array(
                            'name'        => 'start_date',
                            'value'       => $start_time->format('m/d/Y'),
                            'required'    => TRUE,
                            'placeholder' => 'MM/DD/YYYY',
                            'class'       => 'tb-flatpickr',
                            'data'        => array(
                                'mindate'    => $start_time->format('m/d/Y'),
                                'dateFormat' => 'm/d/Y',
                                'altFormat'  => get_option('date_format'),
                                'altinput'   => '',
                                'allowInput' => ''
                            )
                        )) ?>
                        <p class="error start-date" style="display:none;">
                            <?= $this->strings['wrong_date_format'] ?>
                        </p>
                    </li>
                    <li>
                        <h4><?= esc_html__('End date', 'team-booking') ?></h4>
                        <?= Framework\Html::textfield(array(
                            'name'        => 'end_date',
                            'value'       => $end_time->format('m/d/Y'),
                            'required'    => TRUE,
                            'placeholder' => 'MM/DD/YYYY',
                            'class'       => 'tb-flatpickr',
                            'data'        => array(
                                'mindate'    => $start_time->format('m/d/Y'),
                                'dateFormat' => 'm/d/Y',
                                'altFormat'  => get_option('date_format'),
                                'altinput'   => '',
                                'allowInput' => ''
                            )
                        )) ?>
                        <p class="error end-date" style="display:none;">
                            <?= $this->strings['wrong_date_format'] ?>
                        </p>
                    </li>

                    <li>
                        <table class="tbk-promotion-services" style="width: 100%">
                            <tbody>
                            <tr>
                                <th>
                                    <h4><?= esc_html__('Services', 'team-booking') ?></h4>
                                </th>
                                <th>
                                    <h4><?= esc_html__('Base price', 'team-booking') ?></h4>
                                </th>
                                <th>
                                    <h4><?= esc_html__('Discounted price', 'team-booking') ?></h4>
                                </th>
                            </tr>
                            <?php foreach ($this->services as $service) {
                                if ($service->getPrice() <= 0) {
                                    continue;
                                }
                                ?>
                                <tr>
                                    <td style="border-bottom: 1px dashed lightgrey;">
                                        <?= Framework\Html::checkbox(array(
                                            'text'    => $service->getName(),
                                            'name'    => $promotion->getName() . $service->getId(),
                                            'value'   => $service->getId(),
                                            'checked' => $promotion->checkService($service->getId())
                                        )) ?>
                                    </td>
                                    <td style="border-bottom: 1px dashed lightgrey;">
                                        <span id="<?= $promotion->getName() . '-' . $service->getId() ?>-base"
                                              data-price="<?= $service->getPrice() ?>"
                                              class='tbk-promotion-service-base-price'>
                                            <?= Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), $service->getPrice()) ?>
                                        </span>
                                    </td>
                                    <td style="border-bottom: 1px dashed lightgrey;">
                                        <span id="<?= $promotion->getName() . '-' . $service->getId() ?>-discounted"
                                              data-a-sign="<?= Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), NULL) ?>"
                                              class='tbk-promotion-service-discounted-price'
                                            <?php if (!$promotion->checkService($service->getId())) { ?>
                                                style="display: none"
                                            <?php } ?>
                                        >
                                            <?php if ($promotion->checkService($service->getId())) { ?>
                                                <?php if ($promotion->getDiscountType() == 'percentage') { ?>
                                                    <?= $service->getPrice() - $service->getPrice() * $promotion->getDiscount() / 100 ?>
                                                <?php } else {
                                                    if ($service->getPrice() - $promotion->getDiscount() < 0) {
                                                        ?>
                                                        0
                                                    <?php } else { ?>
                                                        <?= $service->getPrice() - $promotion->getDiscount() ?>
                                                    <?php }
                                                }
                                            } ?>
                                        </span>
                                    </td>
                                </tr>
                            <?php } ?>
                            </tbody>
                        </table>
                    </li>
                </ul>
            </div>

            <div class="actions">
                <div class="ui positive button" style="height:auto">
                    <?= esc_html__('Save', 'team-booking') ?>
                </div>
                <div class="ui deny black button" style="height:auto">
                    <?= esc_html__('Cancel', 'team-booking') ?>
                </div>
            </div>

        </div>

        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    private function getAreYouSureModal()
    {
        ob_start();
        // Confirmation Modal Markup
        $promotion_name_span = "<span class='promotion-name'></span>";
        $modal_content = sprintf(esc_html__('You are going to permanently delete %s', 'team-booking'), $promotion_name_span);
        $modal = new Framework\Modal('tbk-promotion-delete-modal');
        $modal->setButtonText(array('approve' => __('Yes', 'team-booking'), 'deny' => __('No', 'team-booking')));
        $modal->setHeaderText(array('main' => __('Are you sure?', 'team-booking')));
        $modal->addContent($modal_content);
        $modal->render();

        return ob_get_clean();
    }

}