<?php

namespace TeamBooking\Admin;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Mappers;

class Service
{
    private $action;

    /** @var \TeamBookingCoworker */
    private $coworker;

    /** @var \TeamBookingType */
    private $service;

    public function __construct($service)
    {
        $this->coworker = Functions\getSettings()->getCoworkerData(get_current_user_id());
        /* @var $service \TeamBookingType */
        $this->service = $service;
        $this->action = admin_url('admin-post.php');
    }

    /**
     * The service's general settings page
     *
     * @return string
     */
    public function getPostBody()
    {
        $string_1 = strtoupper(__('Class', 'team-booking'))
            . ' '
            . ucfirst($this->service->getClass())
            . ' - ' .
            strtoupper(__('Id', 'team-booking'))
            . ' '
            . $this->service->getId();
        ob_start();
        ?>
        <div class="tbk-wrapper">
            <?php
            // Header (service)
            $row = new Framework\Row();
            $column = Framework\Column::fullWidth();
            $column->addElement(Framework\ElementFrom::content($this->getHeaderBlock($string_1)));
            $column->appendTo($row);
            $row->render();

            // Setting panels
            ?>
            <form method="POST" action="<?= $this->action ?>">
                <input type="hidden" name="action" value="option_submit">
                <?php wp_nonce_field('team_booking_options_verify') ?>
                <input type="hidden" name="service_id" value="<?= $this->service->getId() ?>">
                <input type="hidden" name="service_settings" value="general">
                <?php
                $row = new Framework\Row();
                $column = Framework\Column::ofWidth(4);
                $column->addElement($this->getGeneralEdit());
                $column->addElement($this->getGeneralAccess());
                $column->appendTo($row);
                $column = Framework\Column::ofWidth(4);
                $column->addElement($this->getGeneralSlotsAppearance());
                $column->addElement($this->getGeneralPayments());
                $column->addElement($this->getGeneralRedirect());
                $column->appendTo($row);
                $column = Framework\Column::ofWidth(4);
                $column->addElement($this->getGeneralApproveDeny());
                if (!$this->service->isClass('service')) {
                    $column->addElement($this->getCancellationSettings());
                }
                $column->appendTo($row);
                $row->render();
                ?>
            </form>
        </div>

        <?php
        return ob_get_clean();
    }

    /**
     * The service's e-mail settings page
     *
     * @return string
     */
    public function getPostBodyEmail()
    {
        $string_1 = esc_html__('E-mail settings', 'team-booking');
        $string_2 = esc_html__('active', 'team-booking');
        $string_3 = esc_html__('inactive', 'team-booking');
        $string_4 = esc_html__('confirmation e-mail', 'team-booking');
        $string_5 = esc_html__('reminder e-mail', 'team-booking');
        $hooks = $this->service->getFormFields()->getHookList();
        if (!$this->service->isClass('service')) {
            $hooks[] = 'start_datetime';
            $hooks[] = 'start_date';
            $hooks[] = 'start_time';
            $hooks[] = 'end_datetime';
            $hooks[] = 'end_date';
            $hooks[] = 'end_time';
            $hooks[] = 'timezone';
            if (!$this->service->isClass('appointment')) {
                $hooks[] = 'tickets_quantity';
                $hooks[] = 'total_price';
            }
        }
        $hooks[] = 'unit_price';
        $hooks[] = 'post_id';
        $hooks[] = 'post_title';
        $hooks[] = 'coworker_name';
        $hooks[] = 'coworker_url';
        $hooks[] = 'hangout_link';
        if ($this->service->getLocationSetting() !== 'none') {
            $hooks[] = 'service_location';
        }
        // The summary
        $summary = '';
        if (current_user_can('manage_options')) {
            $summary .= $string_4;
            if ($this->service->getConfirm()) {
                $summary .= ' ' . Framework\Html::span(array('text' => $string_2, 'class' => 'active'));
            } else {
                $summary .= ' ' . Framework\Html::span(array('text' => $string_3, 'class' => 'inactive'));
            }
            $summary .= '<br>' . $string_5;
            if ($this->service->isSendReminderEmail()) {
                $summary .= ' ' . Framework\Html::span(array('text' => $string_2, 'class' => 'active'));
            } else {
                $summary .= ' ' . Framework\Html::span(array('text' => $string_3, 'class' => 'inactive'));
            }
        }
        ob_start();
        ?>
        <div class="tbk-wrapper">
            <?php
            // Header (service)
            $row = new Framework\Row();
            $column = Framework\Column::fullWidth();
            $column->addElement(Framework\ElementFrom::content($this->getHeaderBlock($string_1, $summary)));
            $column->appendTo($row);
            $row->render();

            // Setting panels
            ?>
            <form method="POST" action="<?= $this->action ?>">
                <input type="hidden" name="action" value="option_submit">
                <?php wp_nonce_field('team_booking_options_verify') ?>
                <input type="hidden" name="service_id" value="<?= $this->service->getId() ?>">
                <input type="hidden" name="service_settings" value="email">
                <?php
                $row = new Framework\Row();
                if (current_user_can('manage_options')) {
                    $column = Framework\Column::ofWidth(4);
                    $column->addElement($this->getEmailAdminSettings($hooks));
                    $column->appendTo($row);
                }
                $column = Framework\Column::ofWidth(4);
                $column->addElement($this->getEmailCoworkerSettings($hooks));
                $column->appendTo($row);
                if (current_user_can('manage_options')) {
                    $column = Framework\Column::ofWidth(4);
                    $column->addElement($this->getEmailCancellationSettings($hooks));
                    $column->appendTo($row);
                }
                $row->render();
                ?>
            </form>
        </div>
        <?php
        return ob_get_clean();
    }


    /**
     * The service's Your Google Calendar settings page
     *
     * @return string
     */
    public function getPostBodyGcalSettings()
    {
        ob_start();
        ?>
        <div class="tbk-wrapper">
            <?php
            $row = new Framework\Row();
            $column = Framework\Column::fullWidth();
            $column->addElement(Framework\ElementFrom::content($this->getHeaderBlock(__('Your Google Calendar settings', 'team-booking'))));
            $column->appendTo($row);
            $row->render();

            $row = new Framework\Row();
            $column = Framework\Column::ofWidth(8);
            ob_start();
            ?>
            <form method="POST" action="<?= $this->action ?>">
                <input type="hidden" name="action" value="option_submit">
                <input type="hidden" name="service_id" value="<?= $this->service->getId() ?>">
                <?php wp_nonce_field('team_booking_options_verify') ?>
                <?php $this->getGCalSettingsContent()->render() ?>
            </form>
            <?php
            $column->addElement(Framework\ElementFrom::content(ob_get_clean()));
            $column->appendTo($row);
            $column = Framework\Column::ofWidth(4);
            $column->addElement($this->getAvailabilityModes());
            $column->appendTo($row);

            $row->render();
            ?>
        </div>
        <?php
        return ob_get_clean();
    }


    /**
     * The service's reservation form builder page
     *
     * @return string
     */
    public function getPostBodyReservationForm()
    {
        ob_start();
        ?>
        <div class="tbk-wrapper">
            <?php
            $row = new Framework\Row();
            $column = Framework\Column::fullWidth();
            $column->addElement(Framework\ElementFrom::content($this->getHeaderBlock(ucfirst(__('Reservation form', 'team-booking')))));
            $column->appendTo($row);
            $row->render();

            $row = new Framework\Row();
            $column = Framework\Column::ofWidth(4);
            $column->addElement(Framework\ElementFrom::content($this->getBuiltInFieldsSettings()));
            $column->appendTo($row);
            $column = Framework\Column::ofWidth(8);
            $column->addElement($this->getCustomFieldsSettings());
            $column->appendTo($row);
            $row->render();
            ?>
        </div>
        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    /**
     * The admin e-mail and confirmation e-mail part
     *
     * @param array $hooks
     *
     * @return Framework\Panel
     */
    private function getEmailAdminSettings(array $hooks)
    {
        $string_18 = esc_html__("E-mail reminders are sent via WordPress Cron system, which is triggered by site's visits. If your site has low traffic, then reminders can have delays, or not be sent at all.", 'team-booking');
        $panel = new Framework\Panel(ucfirst(__('Admin notification and customer confirmation', 'team-booking')));

        // Admin address for receiving notifications
        $element = new Framework\PanelSettingText(__('Admin address for receiving notifications', 'team-booking'));
        $element->addDescription(__('This address will be the sender of confirmation e-mail to customers', 'team-booking'));
        $element->addDefaultValue($this->service->getEmailForNotifications());
        $element->addFieldname('event[email_for_notifications]');
        $element->setRequired(TRUE);
        $panel->addElement($element);

        // Include uploaded files as attachment
        $element = new Framework\PanelSettingYesOrNo(__('Include uploaded files as attachment', 'team-booking'));
        $element->addFieldname('event[include_files_as_attachment]');
        $element->setState($this->service->getIncludeFilesAsAttachment());
        $panel->addElement($element);

        // Notification e-mail message to admin
        $element = new Framework\PanelSettingEmailEditor(__('Notification e-mail message to admin', 'team-booking'));
        $element->setShowSend(TRUE);
        $element->setSendFieldname('event[email_to_email]');
        $element->setSendState($this->service->getNotify());
        $element->addFieldname('event[back_end_email]');
        $element->setSubject($this->service->getBackendEmail()->getSubject());
        $element->setBody($this->service->getBackendEmail()->getBody());
        foreach ($hooks as $hook) {
            $element->addPlaceholder($hook);
        }
        if ($this->service->getApproveRule() !== 'none') {
            $element->addPlaceholder('approve_link', TRUE);
            $element->addPlaceholder('decline_link', TRUE);
        }
        $panel->addElement($element);

        // Confirmation e-mail message to customer
        $element = new Framework\PanelSettingEmailEditor(__('Confirmation e-mail message to customer', 'team-booking'));
        $element->setShowSend(TRUE);
        $element->setSendFieldname('event[email_to_customer]');
        $element->setSendState($this->service->getConfirm());
        $element->addFieldname('event[front_end_email]');
        $element->setSubject($this->service->getFrontendEmail()->getSubject());
        $element->setBody($this->service->getFrontendEmail()->getBody());
        foreach ($hooks as $hook) {
            $element->addPlaceholder($hook);
        }
        if ($this->service->getAllowCustomerCancellation()) {
            $element->addPlaceholder('cancellation_link', TRUE);
        }
        if ($this->service->getPrice() > 0 && $this->service->getPaymentMustBeDone() !== 'later') {
            $element->addPlaceholder('pay_link', TRUE);
        }
        $element->addPlaceholder('ics_link', TRUE);
        $panel->addElement($element);

        // Reminder e-mail message to customer
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingEmailEditor(__('Reminder e-mail message to customer', 'team-booking'));
            $element->setShowSend(TRUE);
            $element->setSendFieldname('event[email_for_reminder]');
            $element->setSendState($this->service->isSendReminderEmail());
            $element->addFieldname('event[reminder_email]');
            $element->setSubject($this->service->getReminderEmail()->getSubject());
            $element->setBody($this->service->getReminderEmail()->getBody());
            foreach ($hooks as $hook) {
                $element->addPlaceholder($hook);
            }
            $element->addDescription(Framework\Html::paragraph($string_18), FALSE);
            ob_start();
            ?>
            <select name="event[email_for_reminder_timeframe]"
                    id="tb-reminder-email-timeframe" <?= $this->service->isSendReminderEmail() ? '' : 'style="display:none;"' ?>>
                <?php $reminder_email_timeframes = $this->service->getSendReminderEmail() ?>
                <?php for ($i = 1; $i < 4; $i++) {
                    echo '<option value="' . $i . '"';
                    if ((isset ($reminder_email_timeframes['timeframe']) ? $reminder_email_timeframes['timeframe'] : 1) == $i) {
                        echo ' selected="selected"';
                    }
                    echo '>' . sprintf(esc_html(_n('%d day before', '%d days before', $i, 'team-booking')), $i) . '</option>';
                }
                ?>
            </select>
            <script>
                jQuery("input[name='event[email_for_reminder]']").on('click', function () {
                    jQuery('#tb-reminder-email-timeframe').fadeToggle('fast');
                })
            </script>
            <?php
            $element->addToDescription(ob_get_clean(), FALSE);
            $panel->addElement($element);
        }

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $element->appendTo($panel);

        return $panel;
    }


    /**
     * The coworker notification e-mail part
     *
     * @param array $hooks
     *
     * @return Framework\Panel
     */
    private function getEmailCoworkerSettings(array $hooks)
    {
        $custom_booking_type_settings = $this->coworker->getCustomEventSettings($this->service->getId());
        $panel = new Framework\Panel(ucfirst(__('Your personal notification', 'team-booking')));

        // Get reservation details by e-mail
        $element = new Framework\PanelSettingYesOrNo(__('Get reservation details by e-mail', 'team-booking'));
        $element->addDescription(__('Send to:', 'team-booking'));
        $element->addToDescription(' ' . $this->coworker->getEmail());
        $element->addToDescription(' (<a href="' . get_edit_user_link() . '">', FALSE);
        $element->addToDescription(__('where can I change this?', 'team-booking'));
        $element->addToDescription('</a>)', FALSE);
        $element->addFieldname('data[get_details_by_email]');
        $element->setState($custom_booking_type_settings->getGetDetailsByEmail());
        $panel->addElement($element);

        // Include uploaded files as attachment
        $element = new Framework\PanelSettingYesOrNo(__('Include uploaded files as attachment', 'team-booking'));
        $element->addFieldname('data[include_uploaded_files_as_attachment]');
        $element->setState($custom_booking_type_settings->getIncludeFilesAsAttachment());
        $panel->addElement($element);

        // Notification e-mail message
        $element = new Framework\PanelSettingEmailEditor(__('Notification e-mail message', 'team-booking'));
        $element->addFieldname('data[email][email_text]');
        $element->setSubject($custom_booking_type_settings->getNotificationEmailSubject());
        $element->setBody($custom_booking_type_settings->getNotificationEmailBody());
        foreach ($hooks as $hook) {
            if ($hook === 'coworker_name') continue;
            $element->addPlaceholder($hook);
        }
        if ($this->service->getApproveRule() === 'coworker') {
            $element->addPlaceholder('approve_link', TRUE);
            $element->addPlaceholder('decline_link', TRUE);
        }
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The cancellation e-mail part
     *
     * @param array $hooks
     *
     * @return Framework\Panel
     */
    private function getEmailCancellationSettings(array $hooks)
    {
        $panel = new Framework\Panel(ucfirst(__('Cancellation notification', 'team-booking')));

        // Send an e-mail message back to customer when a reservation is cancelled
        $element = new Framework\PanelSettingYesOrNo(__('Send an e-mail message back to customer when a reservation is cancelled', 'team-booking'));
        $element->addFieldname('event[send_cancellation_email]');
        $element->setState($this->service->getSendCancellationEmail());
        $panel->addElement($element);

        // Cancellation e-mail message
        $element = new Framework\PanelSettingEmailEditor(__('Cancellation e-mail message', 'team-booking'));
        $element->addFieldname('event[cancellation_email]');
        $element->setSubject($this->service->getCancellationEmail()->getSubject());
        $element->setBody($this->service->getCancellationEmail()->getBody());
        foreach ($hooks as $hook) {
            $element->addPlaceholder($hook);
        }
        $element->addPlaceholder('reason');
        $panel->addElement($element);

        // Send an e-mail message back to admin and coworker when a reservation is cancelled
        $element = new Framework\PanelSettingYesOrNo(__('Send an e-mail message back to admin and coworker when a reservation is cancelled', 'team-booking'));
        $element->addFieldname('event[send_cancellation_email_backend]');
        $element->setState($this->service->getSendCancellationEmailBackend());
        $panel->addElement($element);

        // Admin/Coworker cancellation e-mail message
        $element = new Framework\PanelSettingEmailEditor(__('Cancellation e-mail message to admin and coworker', 'team-booking'));
        $element->addFieldname('event[cancellation_email_backend]');
        $element->setSubject($this->service->getCancellationEmailBackend()->getSubject());
        $element->setBody($this->service->getCancellationEmailBackend()->getBody());
        foreach ($hooks as $hook) {
            $element->addPlaceholder($hook);
        }
        $element->addPlaceholder('reason');
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The header block
     *
     * @param      $description
     * @param bool $summary
     *
     * @return string
     */
    private function getHeaderBlock($description, $summary = FALSE)
    {
        ob_start();
        ?>
        <div class="tbk-panel tbk-content">
            <div class="tbk-settings-title">
                <?= Framework\Html::anchor(array(
                    'text'   => Framework\Html::span(array('class' => 'dashicons dashicons-arrow-left-alt')) . esc_html__('Back to list', 'team-booking'),
                    'escape' => FALSE,
                    'class'  => 'button button-hero button-primary',
                    'href'   => admin_url('admin.php?page=team-booking&tab=events')
                )) ?>
                <div style="display: inline-block;">
                    <?= Framework\Html::h3(array('text' => $this->service->getName(), 'class' => 'tbk-heading')) ?>
                    <?= Framework\Html::paragraph(array('text' => $description, 'class' => 'tbk-excerpt')) ?>
                </div>
                <?php if ($summary) { ?>
                    <div class="tb-panel-summary" style="display: inline-block;">
                        <?= $summary ?>
                    </div>
                <?php } ?>
            </div>
        </div>

        <?php
        return ob_get_clean();
    }

    /**
     * The general service's settings part
     *
     * @return Framework\Panel
     */
    private function getGeneralEdit()
    {
        $panel = new Framework\Panel(ucfirst(__('General settings', 'team-booking')));

        // Service name
        $element = new Framework\PanelSettingText(__('Name', 'team-booking'));
        $element->addDefaultValue($this->service->getName());
        $element->addPlaceholder('e.g. House Surveys');
        $element->addFieldname('event[booking_type_name]');
        $element->setRequired(TRUE);
        $panel->addElement($element);

        // Service description
        $element = new Framework\PanelSettingRichTextarea(__('Service description', 'team-booking'));
        $element->addDescription(__('This description will be shown on reservation modal. You can use html formatting.', 'team-booking'));
        $element->setText($this->service->getServiceInfo());
        $element->addFieldname('event[info]');
        $panel->addElement($element);

        // Service color
        $element = new Framework\PanelSettingColorPicker(__('Service color', 'team-booking'));
        $element->addFieldname('event[service_color]');
        $element->setValue($this->service->getServiceColor());
        $panel->addElement($element);

        // Max total tickets per slot & max tickets per reservation
        if ($this->service->getAttendees() != NULL) {
            $element = new Framework\PanelSettingNumber(__('Max total tickets per slot', 'team-booking'));
            $element->addFieldname('event[max_attendees]');
            $element->addDescription(__('Max value = 200', 'team-booking'));
            $element->setValue($this->service->getAttendees());
            $element->setMin(1);
            $element->setMax(200);
            $panel->addElement($element);

            $element = new Framework\PanelSettingNumber(__('Max tickets per reservation', 'team-booking'));
            $element->addFieldname('event[max_tickets_per_reservation]');
            $element->addDescription(__('For "event" services, a customer (identified by email address) can place one reservation per slot. If you want to allow the customer to book multiple tickets, provide the maximum number here.', 'team-booking'));
            $element->setValue($this->service->getMaxReservationsPerUser());
            $element->setMin(1);
            $element->setMax($this->service->getAttendees());
            $panel->addElement($element);
        }

        // Slot duration
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Slot duration rule', 'team-booking'));
            $element->addFieldname('event[duration_rule]');
            $element->addOption(array(
                'label'       => __('Let Coworkers decide', 'team-booking'),
                'label_title' => __('Let Coworkers decide', 'team-booking'),
                'value'       => 'coworker',
                'checked'     => (int)('coworker' === $this->service->getDurationRule())
            ));
            $element->addOption(array(
                'label'       => __('Inherited from Google Calendar event', 'team-booking'),
                'label_title' => __('Inherited from Google Calendar event', 'team-booking'),
                'value'       => 'inherited',
                'checked'     => (int)('inherited' === $this->service->getDurationRule())
            ));
            $label_element = new Framework\PanelSettingTimespanSelector(__('Fixed', 'team-booking'));
            $label_element->addFieldname('event[default_duration]');
            $label_element->isNested(TRUE);
            $label_element->setHoursLabel(__('hours', 'team-booking'));
            $label_element->setMinsLabel(__('minutes', 'team-booking'));
            $label_element->setSelectedHours(floor($this->service->getDefaultDuration() / HOUR_IN_SECONDS));
            $label_element->setSelectedMins(ceil($this->service->getDefaultDuration() - floor($this->service->getDefaultDuration() / HOUR_IN_SECONDS) * HOUR_IN_SECONDS) / MINUTE_IN_SECONDS);
            $element->addOption(array(
                'label'       => $label_element,
                'label_title' => __('Fixed', 'team-booking'),
                'value'       => 'fixed',
                'checked'     => (int)('fixed' === $this->service->getDurationRule())
            ));
            $panel->addElement($element);
        }

        // Service assignment rule
        if ($this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Assignment rule', 'team-booking'));
            $element->addDescription(__('In case of multiple coworkers giving the availability to this service, it needs a rule to decide which of them will be assigned to a new reservation, time after time.', 'team-booking'));
            $element->addFieldname('event[assignment_rule]');
            $element->addOption(array(
                'label'       => __('Equal', 'team-booking'),
                'label_title' => __('Equal', 'team-booking'),
                'value'       => 'equal',
                'checked'     => (int)('equal' === $this->service->getAssignmentRule()),
                'description' => __('A new reservation will be assigned to the coworker with less of them assigned (for this service), based on reservation history in the database.', 'team-booking')
            ));
            $description_element = new Framework\PanelSettingDescriptionWithSelector(NULL);
            $description_element->addFieldname('event[direct_coworker]');
            $description_element->addDescription(__('A new reservation will be always assigned to the coworker specified below.', 'team-booking'));
            $description_element->setSelectedOption($this->service->getCoworkerForDirectAssignment());
            $description_element->setDisabled($this->service->getAssignmentRule() !== 'direct');
            foreach (Functions\getCoworkersIdList() as $id) {
                $user_data = get_userdata($id);
                $description_element->addOption($id, $user_data->user_firstname . ' ' . $user_data->user_lastname . ' (' . $user_data->user_email . ')');
            }
            $element->addOption(array(
                'label'       => __('Direct', 'team-booking'),
                'label_title' => __('Direct', 'team-booking'),
                'value'       => 'direct',
                'checked'     => (int)('direct' === $this->service->getAssignmentRule()),
                'description' => $description_element
            ));
            $element->addOption(array(
                'label'       => __('Random', 'team-booking'),
                'label_title' => __('Random', 'team-booking'),
                'value'       => 'random',
                'checked'     => (int)('random' === $this->service->getAssignmentRule()),
                'description' => __('Picks a random coworker.', 'team-booking')
            ));
            $element->setDividers(TRUE);
            $panel->addElement($element);
        }

        // location
        $element = new Framework\PanelSettingRadios(__('Location', 'team-booking'));
        $element->addDescription(__("If a location is set, a map will appear on the reservation form. If inherited from Google Calendar slot, but the slot has no location, the map won't show.", 'team-booking'));
        $element->addFieldname('event[location_setting]');
        $element->addOption(array(
            'label'       => __('No location', 'team-booking'),
            'label_title' => __('No location', 'team-booking'),
            'value'       => 'none',
            'checked'     => (int)('none' === $this->service->getLocationSetting())
        ));
        if (!$this->service->isClass('service')) {
            $element->addOption(array(
                'label'       => __('Inherited from Google Calendar slot', 'team-booking'),
                'label_title' => __('Inherited from Google Calendar slot', 'team-booking'),
                'value'       => 'inherited',
                'checked'     => (int)('inherited' === $this->service->getLocationSetting())
            ));
        }
        $description_element = new Framework\PanelSettingDescriptionWithTextfield(NULL);
        $description_element->addFieldname('event[location_address]');
        $description_element->setDefaultText($this->service->getLocationAddress());
        $description_element->setPlaceholder('e.g. 1234, Main Street, Anytown, USA');
        $description_element->setDisabled($this->service->getLocationSetting() !== 'fixed');
        $element->addOption(array(
            'label'       => __('Fixed location', 'team-booking'),
            'label_title' => __('Fixed location', 'team-booking'),
            'value'       => 'fixed',
            'checked'     => (int)('fixed' === $this->service->getLocationSetting()),
            'description' => $description_element
        ));
        if ($this->service->isClass('appointment')) {
            $element->addAlertDropcap(__('Note', 'team-booking'));
            $element->addAlert(__('If set to "no location" and the reservation form has the built-in address field active, then the Google Calendar event will be updated with the address of the customer, if given. Only for Appointment Class.', 'team-booking'));
        }
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The service's payment settings part
     *
     * @return Framework\Panel
     */
    private function getGeneralPayments()
    {
        $panel = new Framework\Panel(ucfirst(__('Payments settings', 'team-booking')));

        // Price for reservation
        $element = new Framework\PanelSettingNumber(__('Price for reservation', 'team-booking'));
        $element->addFieldname('event[service_price]');
        $element->addDescription(__("If zero, won't appears. To change the currency, go to Payments Gateway tab.", 'team-booking'));
        $element->setMin(0);
        $element->setStep(0.01);
        $element->setValue($this->service->getPrice());
        $element->setFieldDescription(Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), NULL));
        $panel->addElement($element);

        // How the payment must be done
        $element = new Framework\PanelSettingRadios(__('How the payment must be done', 'team-booking'));
        $element->addFieldname('event[payment_must_be_done]');
        $element->addOption(array(
            'label'       => __('Immediately', 'team-booking'),
            'label_title' => __('Immediately', 'team-booking'),
            'value'       => 'immediately',
            'checked'     => (int)('immediately' === $this->service->getPaymentMustBeDone()),
            'description' => __(
                'The customer is redirect to the payment gateway after the reservation. If no payment will be done within the Max Pending Time specified in Payments gateway section, the reservation will be released.',
                'team-booking'
            )
        ));
        $element->addOption(array(
            'label'       => __('Later', 'team-booking'),
            'label_title' => __('Later', 'team-booking'),
            'value'       => 'later',
            'checked'     => (int)('later' === $this->service->getPaymentMustBeDone()),
            'description' => __(
                'The customer will pay on a later time. Team Booking will not handle the payment, and you should manually set the reservation as "Paid", eventually.',
                'team-booking'
            )
        ));
        $element->addOption(array(
            'label'       => __("At the customer's discretion", 'team-booking'),
            'label_title' => __("At the customer's discretion", 'team-booking'),
            'value'       => 'discretional',
            'checked'     => (int)('discretional' === $this->service->getPaymentMustBeDone()),
            'description' => __(
                'The customer choose whether redirect to the payment gateway after the reservation, or pay locally/later. If he wants to pay on a later time and/or cancel the payment, the reservation will still be in place, as "Not paid". You should manually set the reservation as "Paid", eventually.',
                'team-booking'
            )
        ));
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The service's redirect panel
     *
     * @return Framework\Panel
     */
    private function getGeneralRedirect()
    {
        $panel = new Framework\Panel(ucfirst(__('Redirect and conversion tracking', 'team-booking')));
        // Redirect
        $element = new Framework\PanelSettingRadios(__('Redirect', 'team-booking'));
        $element->addFieldname('event[redirect]');
        $element->addDescription(__('If active, the customer will be redirected to the specified URL after a successful reservation for this service. This is meant for tracking conversions. If an offsite payment gateway like PayPal is chosen, then this URL will override the one specified in the gateway settings and will be called after the payment (but if the customer decides not to be redirected, then an eventual conversion code will not be triggered).', 'team-booking'));
        $element->addOption(array(
            'label'       => __('No redirect', 'team-booking'),
            'label_title' => __('No redirect', 'team-booking'),
            'value'       => 'no_redirect',
            'checked'     => !$this->service->getRedirect()
        ));
        $description_element = new Framework\PanelSettingDescriptionWithTextfield(NULL);
        $description_element->addFieldname('event[redirect_url]');
        $description_element->setDefaultText($this->service->getRedirectUrl());
        $description_element->setPlaceholder('http://, https://');
        $description_element->setDisabled(!$this->service->getRedirect());
        $element->addOption(array(
            'label'       => __('Redirect to URL', 'team-booking'),
            'label_title' => __('Redirect to URL', 'team-booking'),
            'value'       => 'redirect_to_url',
            'checked'     => $this->service->getRedirect(),
            'description' => $description_element
        ));
        $element->addAlertDropcap(__('Note', 'team-booking'));
        $element->addAlert(__(
            "About payments: when redirect is active, while the immediate and later payment settings will work just fine, the customer's discretional payment setting otherwise won't work if selected, so no payment option will be presented to the customer after the reservation.",
            'team-booking'
        ));
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The service's slot appearance panel
     *
     * @return Framework\Panel
     */
    private function getGeneralSlotsAppearance()
    {
        $panel = new Framework\Panel(ucfirst(__('Frontend slot settings', 'team-booking')));

        // Show reservations left
        if ($this->service->isClass('event')) {
            $element = new Framework\PanelSettingYesOrNo(__('Show reservations left', 'team-booking'));
            $element->addFieldname('event[show_reservations_left]');
            $element->setState($this->service->getShowReservationsLeft());
            $panel->addElement($element);
        }

        // Show sold-out slots
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingYesOrNo(__('Show sold-out slots', 'team-booking'));
            $element->addFieldname('event[show_soldout]');
            $element->setState($this->service->getShowSoldout());
            $panel->addElement($element);
        }

        // Show start/end times
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Show start/end times', 'team-booking'));
            $element->addFieldname('event[show_times]');
            foreach ($this->service->getShowTimes()->getOptionsList() as $var => $value) {
                $element->addOption(array(
                    'label'       => $this->service->getShowTimes()->getOptionDescription($var),
                    'label_title' => $this->service->getShowTimes()->getOptionDescription($var),
                    'value'       => $var,
                    'checked'     => $value == TRUE
                ));
            }
            $panel->addElement($element);
        }

        // Show coworker's name
        $element = new Framework\PanelSettingYesOrNo(__("Show coworker's name", 'team-booking'));
        $element->addFieldname('event[show_coworker]');
        $element->setState($this->service->getShowCoworker());
        $panel->addElement($element);

        // Show coworker's profile page link
        $element = new Framework\PanelSettingYesOrNo(__("Show coworker's profile page link", 'team-booking'));
        $element->addDescription(__("If yes, and if Coworker's name is shown, then it becomes a link pointing to Coworker's profile page. You can set the profile page for each of your Coworkers in the Manage Coworkers tab.", 'team-booking'));
        $element->addFieldname('event[show_coworker_url]');
        $element->setState($this->service->getShowCoworkerUrl());
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }

    /**
     * The service's approval system panel
     *
     * @return Framework\Panel
     */
    private function getGeneralApproveDeny()
    {
        $panel = new Framework\Panel(ucfirst(__('Approval settings', 'team-booking')));

        // Approval requirement
        $element = new Framework\PanelSettingRadios(__('Approval requirement', 'team-booking'));
        $element->addFieldname('event[approve_rule]');
        $element->addDescription(__(
            'If approval is required, then the reservation will be shown in the overview panel in "approval required" status. No actions like sending confirmation e-mail, or updating Google Calendar will be taken, until the reservation is confirmed.',
            'team-booking'
        ));
        $element->addOption(array(
            'label'       => __('Do not require approval', 'team-booking'),
            'label_title' => __('Do not require approval', 'team-booking'),
            'value'       => 'none',
            'checked'     => (int)('none' === $this->service->getApproveRule())
        ));
        if ($this->service->getPaymentMustBeDone() !== 'later') {
            $element->addAlertDropcap(__('Note', 'team-booking'));
            $element->addAlert(__('you can activate approval only if the "payment must be done" setting for this service is set to "later"!', 'team-booking'));
        } else {
            $element->addOption(array(
                'label'       => __('Require Admin approval', 'team-booking'),
                'label_title' => __('Require Admin approval', 'team-booking'),
                'value'       => 'admin',
                'checked'     => (int)('admin' === $this->service->getApproveRule())
            ));
            $element->addOption(array(
                'label'       => __('Require Coworker approval', 'team-booking'),
                'label_title' => __('Require Coworker approval', 'team-booking'),
                'value'       => 'coworker',
                'checked'     => (int)('coworker' === $this->service->getApproveRule())
            ));
        }
        $panel->addElement($element);

        // Until approval
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Until approval:', 'team-booking'));
            $element->addFieldname('event[approve_until]');
            $element->addOption(array(
                'label'       => __('keep the slot/tickets free (could lead to overbooking)', 'team-booking'),
                'label_title' => __('keep the slot/tickets free (could lead to overbooking)', 'team-booking'),
                'value'       => 'yes',
                'checked'     => $this->service->getFreeUntilApproval()
            ));
            $element->addOption(array(
                'label'       => __('set the slot/tickets in a booked status', 'team-booking'),
                'label_title' => __('set the slot/tickets in a booked status', 'team-booking'),
                'value'       => 'no',
                'checked'     => !$this->service->getFreeUntilApproval()
            ));
            $panel->addElement($element);
        }

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The service's cancellation settings panel
     *
     * @return Framework\Panel
     */
    private function getCancellationSettings()
    {
        $panel = new Framework\Panel(ucfirst(__('Cancellation settings', 'team-booking')));

        // Allow customer's cancellation
        $element = new Framework\PanelSettingYesOrNo(__('Allow cancellation by customer', 'team-booking'));
        $element->addDescription(__('Allow the customer to cancel a reservation for this service. If yes, customers will be able to cancel their reservations for this service either by the e-mail cancellation link (if provided) or by the page where [tb-reservations] shortcode is placed (in which case they must be logged in)', 'team-booking'));
        $element->addFieldname('event[allow_customer_cancellation]');
        $element->setState($this->service->getAllowCustomerCancellation());
        $panel->addElement($element);

        // Allow customer to left a reason for the cancellation
        $element = new Framework\PanelSettingYesOrNo(__('Allow cancellation reason', 'team-booking'));
        $element->addDescription(__('Allow the customer to left a reason for the cancellation.', 'team-booking'));
        $element->addFieldname('event[allow_customer_cancellation_reason]');
        $element->setState($this->service->getAllowCustomerCancellationReason());
        $panel->addElement($element);

        // Cancellation time span
        $element = new Framework\PanelSettingTimespanSelector(__('Cancellation time span', 'team-booking'));
        $element->addFieldname('event[allow_customer_cancellation_timespan]');
        $element->addDescription(__("Choose a time limit, relative to the reservation's start time, after which the cancellation will be not possible anymore by the customer.", 'team-booking'));
        $element->setShowDays(TRUE);
        $element->setDaysLabel(__('days', 'team-booking'));
        $element->setHoursLabel(__('hours', 'team-booking'));
        $element->setMinsLabel(__('minutes', 'team-booking'));
        $selected_days = floor($this->service->getAllowCustomerCancellationUntil() / DAY_IN_SECONDS);
        $element->setSelectedDays($selected_days);
        $selected_hours = floor(($this->service->getAllowCustomerCancellationUntil() - $selected_days * DAY_IN_SECONDS) / HOUR_IN_SECONDS);
        $element->setSelectedHours($selected_hours);
        $element->setSelectedMins(ceil($this->service->getAllowCustomerCancellationUntil() - $selected_hours * HOUR_IN_SECONDS - $selected_days * DAY_IN_SECONDS) / MINUTE_IN_SECONDS);
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The service's access settings panel
     *
     * @return Framework\Panel
     */
    private function getGeneralAccess()
    {
        $panel = new Framework\Panel(ucfirst(__('Access settings', 'team-booking')));

        // Who can make a reservation
        $element = new Framework\PanelSettingRadios(__('Who can make a reservation', 'team-booking'));
        $element->addFieldname('event[allow_reservation]');
        $element->addDescription(__(
            'If "Logged user only" is selected, then an advice with link to registration page will be shown.
            The default link can be changed in the "Core settings" tab.',
            'team-booking'
        ));
        $element->addOption(array(
            'label'       => __('Everyone', 'team-booking'),
            'label_title' => __('Everyone', 'team-booking'),
            'value'       => 0,
            'checked'     => $this->service->getLoggedOnly() == FALSE
        ));
        $element->addOption(array(
            'label'       => __('Logged users only', 'team-booking'),
            'label_title' => __('Logged users only', 'team-booking'),
            'value'       => 1,
            'checked'     => $this->service->getLoggedOnly() == TRUE
        ));
        $panel->addElement($element);

        // Max reservations per logged user
        $element = new Framework\PanelSettingNumber(__('Max reservations per logged user', 'team-booking'));
        $element->addFieldname('event[max_reservations_logged_user]');
        $element->addDescription(__(
            "If the reservations are allowed to logged users only, then you can also allow a limited number of them. The book now button will show how many are left. If the limit is reached, the logged user can't proceed. The counter decreases if the reservations are deleted.",
            'team-booking'
        ));
        $element->setMin(0);
        $element->setValue($this->service->getMaxReservationsLoggedUser());
        $element->setFieldDescription('0 = ' . __('endless', 'team-booking'));
        $panel->addElement($element);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $panel->addElement($element);

        return $panel;
    }


    /**
     * The custom fields panel
     *
     * @return Framework\Panel
     */
    private function getCustomFieldsSettings()
    {
        $panel = new Framework\Panel(ucfirst(__('Custom fields', 'team-booking')));
        $element = new Framework\PanelSettingWildcard(NULL);
        ob_start();
        ?>
        <div class="tbk-custom-fields-dock">
            <a class="tb-add-custom-field" data-field="text"
               data-serviceid="<?= $this->service->getId() ?>">
                <span class="dashicons dashicons-plus"></span>
                <?= esc_html__('Text Field', 'team-booking') ?>
                <span class="sprite"
                      style="background-image: url('<?= TEAMBOOKING_URL ?>images/form-textfield-sprite.png');"></span>
            </a>
            <a class="tb-add-custom-field" data-field="select"
               data-serviceid="<?= $this->service->getId() ?>" href="#">
                <span class="dashicons dashicons-plus"></span>
                <?= esc_html__('Select', 'team-booking') ?>
                <span class="sprite"
                      style="background-image: url('<?= TEAMBOOKING_URL ?>images/form-select-sprite.png');"></span>
            </a>
            <a class="tb-add-custom-field" data-field="textarea"
               data-serviceid="<?= $this->service->getId() ?>" href="#">
                <span class="dashicons dashicons-plus"></span>
                <?= esc_html__('Textarea', 'team-booking') ?>
                <span class="sprite"
                      style="background-image: url('<?= TEAMBOOKING_URL ?>images/form-textarea-sprite.png');"></span>
            </a>
            <a class="tb-add-custom-field" data-field="checkbox"
               data-serviceid="<?= $this->service->getId() ?>" href="#">
                <span class="dashicons dashicons-plus"></span>
                <?= esc_html__('Checkbox', 'team-booking') ?>
                <span class="sprite"
                      style="background-image: url('<?= TEAMBOOKING_URL ?>images/form-checkbox-sprite.png');"></span>
            </a>
            <a class="tb-add-custom-field" data-field="radio"
               data-serviceid="<?= $this->service->getId() ?>" href="#">
                <span class="dashicons dashicons-plus"></span>
                <?= esc_html__('Radio Group', 'team-booking') ?>
                <span class="sprite"
                      style="background-image: url('<?= TEAMBOOKING_URL ?>images/form-radio-sprite.png');"></span>
            </a>
            <a class="tb-add-custom-field" data-field="file"
               data-serviceid="<?= $this->service->getId() ?>" href="#">
                <span class="dashicons dashicons-plus"></span>
                <?= esc_html__('File Upload', 'team-booking') ?>
                <span class="sprite"
                      style="background-image: url('<?= TEAMBOOKING_URL ?>images/form-file-sprite.png');"></span>
            </a>
        </div>
        <hr>
        <?php
        $custom_fields = $this->service->getFormFields()->getCustomFields();
        if (empty($custom_fields)) {
            ?>
            <div class='tb-no-custom-fields'>
                <?= esc_html__('No custom fields for this service yet.', 'team-booking') ?>
            </div>
            <?php
        }
        ?>
        <ul class="tb-custom-fields-list sortable">
            <?php
            foreach ($custom_fields as $field) {
                if (is_object($field)) {
                    ?>
                    <li>
                        <?= Mappers\adminFormCustomFieldsMapper($field, $this->service->getId()) ?>
                    </li>
                    <?php
                }
            }
            ?>
        </ul>
        <?php
        $element->addContent(ob_get_clean());
        $element->appendTo($panel);

        return $panel;
    }


    /**
     * The built-in fields panel
     *
     * @return string
     */
    private function getBuiltInFieldsSettings()
    {
        $panel = new Framework\Panel(ucfirst(__('Built-in fields', 'team-booking')));

        // Built-in fields
        $fields = $this->service->getFormFields()->getBuiltInFields();
        $element = new Framework\PanelSettingWildcard(ucfirst(__('Built-in fields', 'team-booking')));
        $element->addDescription(__('White checkbox = show', 'team-booking') . '<br>' . __('Red checkbox = required', 'team-booking'), FALSE);
        ob_start();
        ?>
        <fieldset>
            <?php
            echo $this->getMozCheckboxCSSTarget('EF6060');

            foreach ($fields as $field) {
                /* @var $field \TeamBookingFormTextField */
                ?>
                <div class="tb-builtin-field-wrapper">
                    <label for="form_fields[<?= $field->getHook() ?>]"
                           style="cursor:auto;">
                        <input name="form_fields[<?= $field->getHook() ?>]"
                               style="vertical-align: middle;" type="checkbox"
                               value="<?= $field->getHook() ?>" <?php checked(1, $field->getIsActive()) ?> />
                        <input name="form_fields[<?= $field->getHook() ?>_required]"
                               style="vertical-align: middle;" type="checkbox"
                               value="<?= $field->getHook() ?>" <?php checked(1, $field->getIsRequired()) ?>
                               class="tb_required _EF6060"/>
                        <span><?= $field->getLabel() ?></span><span
                            class="tbk-builtin-field-hook"> [<?= $field->getHook() ?>]</span>
                    </label>

                    <a title="<?= esc_html__('Field validation rule', 'team-booking') ?>"
                       id="tb-field-regex-open-<?= $field->getHook() ?>"
                       class="ui <?= $field->getValidationRegex() ? 'active' : 'inverted' ?> circular mini label"
                       style="float: right">
                        <span class="dashicons dashicons-editor-spellcheck"></span>
                    </a>
                    <?= Functions\getValidationModal($field, 'form_fields') ?>
                </div>
                <?php
            } ?>
        </fieldset>
        <?php
        $element->addContent(ob_get_clean());
        $element->appendTo($panel);

        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_update_booking_type');
        $element->appendTo($panel);
        ob_start();
        ?>
        <form method="POST" action="<?= $this->action ?>">
            <input type="hidden" name="action" value="option_submit">
            <?php wp_nonce_field('team_booking_options_verify') ?>
            <input type="hidden" name="service_id" value="<?= $this->service->getId() ?>">
            <input type="hidden" name="service_settings" value="form">
            <?php $panel->render() ?>
        </form>

        <?php
        return ob_get_clean();
    }


    /**
     * The informative panel about availability modes
     *
     * @return Framework\Panel
     */
    private function getAvailabilityModes()
    {
        $colors = array(
            0  => '56b378',
            1  => 'a4bdfc',
            2  => '7ae7bf',
            3  => 'dbadff',
            4  => 'ff887c',
            5  => 'fbd75b',
            6  => 'ffb878',
            7  => '46d6db',
            8  => 'e1e1e1',
            9  => '5484ed',
            10 => '51b749',
            11 => 'dc2127'
        );

        $panel = new Framework\Panel(ucfirst(__('Availability modes', 'team-booking')));

        $element = new Framework\PanelSettingWildcard(NULL);
        $element->addDescription(__('The three modes of placing availability in Google Calendar are described below. You can use all of them, but please note that the Container Mode is NOT available if the slot duration is inherited from Google Calendar event.', 'team-booking'));
        $element->appendTo($panel);

        $element = new Framework\PanelSettingWildcard(__('Slot mode', 'team-booking'));
        $element->addDescription(
            esc_html__('Event name', 'team-booking') . ' = ' . '<strong>' . $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle() . '</strong>', FALSE
        );
        $row = new Framework\Row();
        $column = Framework\Column::ofWidth(5);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding:10px;">
                <div style="background: #928686;
                            min-height: 35px;
                            border: 1px solid #5a5a5a;
                            color: white;
                            font-size: 0.6vw;
                            line-height: 0.7vw;
                            padding: 4px;">
                    11:00 - 12:00<br>' . $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle()
            . '</div></div>'
        ));
        $column->appendTo($row);
        $column = Framework\Column::ofWidth(2);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding-right: 0;padding-left: 0;text-align: center;">
                    <span class="dashicons dashicons-arrow-right-alt"></span>
             </div>'
        ));
        $column->appendTo($row);
        $column = Framework\Column::ofWidth(5);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding:10px;">
                <div style="background: #' . $colors[ $this->coworker->getCustomEventSettings($this->service->getId())->getBookedEventColor() ] . ';
                            min-height: 35px;
                            border: 1px solid #5a5a5a;
                            color: white;
                            font-size: 0.6vw;
                            line-height: 0.7vw;
                            padding: 4px;">
                    11:00 - 12:00<br>'
            . ($this->service->isClass('event')
                ? $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle()
                : $this->coworker->getCustomEventSettings($this->service->getId())->getAfterBookedTitle())
            . '</div></div>'
        ));
        $column->appendTo($row);
        $element->addContent($row);
        $element->appendTo($panel);

        $element = new Framework\PanelSettingWildcard(__('Container mode', 'team-booking'));
        $element->addDescription(
            esc_html__('Event name', 'team-booking') . ' = <strong>' . $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle()
            . '</strong> + "container"', FALSE
        );
        $row = new Framework\Row();
        $column = Framework\Column::ofWidth(5);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding:10px;">
                <div style="background: #928686;
                            min-height: 180px;
                            border: 1px solid #5a5a5a;
                            color: white;
                            font-size: 0.6vw;
                            line-height: 0.7vw;
                            padding: 4px;">
                    14:00 - 18:00<br>'
            . $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle() . ' container'
            . '</div></div>'
        ));
        $column->appendTo($row);
        $column = Framework\Column::ofWidth(2);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding-right: 0;padding-left: 0;text-align: center;">
                    <span class="dashicons dashicons-arrow-right-alt"></span>
             </div>'
        ));
        $column->appendTo($row);
        $column = Framework\Column::ofWidth(5);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding:10px;">
                <div style="background: #928686;
                            min-height: 180px;
                            border: 1px solid #5a5a5a;
                            color: white;
                            font-size: 0.6vw;
                            line-height: 0.7vw;
                            padding: 4px;">
                    14:00 - 18:00<br>'
            . $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle() . ' container'
            . '<div class="tbk-column tbk-span-12" style="position:absolute;">
                        <div class="tbk-content" style="padding:10px;">
                            <div style="background: #' . $colors[ $this->coworker->getCustomEventSettings($this->service->getId())->getBookedEventColor() ] . ';
                                        min-height: 35px;
                                        border: 1px solid #5a5a5a;
                                        color: white;
                                        font-size: 0.6vw;
                                        line-height: 0.7vw;
                                        padding: 4px;">
                                15:00 - 16:00<br>'
            . ($this->service->isClass('event')
                ? $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle()
                : $this->coworker->getCustomEventSettings($this->service->getId())->getAfterBookedTitle())
            . '</div></div>
                    </div>
                    <div class="tbk-column tbk-span-12" style="position:absolute;top: 100px;">
                        <div class="tbk-content" style="padding:10px;">
                            <div style="background: #' . $colors[ $this->coworker->getCustomEventSettings($this->service->getId())->getBookedEventColor() ] . ';
                                        min-height: 35px;
                                        border: 1px solid #5a5a5a;
                                        color: white;
                                        font-size: 0.6vw;
                                        line-height: 0.7vw;
                                        padding: 4px;">
                                16:15 - 17:15<br>'
            . ($this->service->isClass('event')
                ? $this->coworker->getCustomEventSettings($this->service->getId())->getLinkedEventTitle()
                : $this->coworker->getCustomEventSettings($this->service->getId())->getAfterBookedTitle())
            . '</div></div>
                    </div>
                </div>
            </div>'
        ));
        $column->appendTo($row);
        $element->addContent($row);
        $element->appendTo($panel);

        $element = new Framework\PanelSettingWildcard(__('Container mode (mutually exclusive multiple services)', 'team-booking'));
        $element->addDescription(
            __('Event name', 'team-booking') . ' = ' . __('Event title (free slot)', 'team-booking') . ' 1 + "+" + '
            . __('Event title (free slot)', 'team-booking') . ' 2 + "+" ... + "container"'
        );
        $row = new Framework\Row();
        $column = Framework\Column::ofWidth(5);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding:10px;">
                <div style="background: #928686;
                            min-height: 180px;
                            border: 1px solid #5a5a5a;
                            color: white;
                            font-size: 0.6vw;
                            line-height: 0.7vw;
                            padding: 4px;">
                    14:00 - 18:00<br>
                    Service 1 + Service 2 + Service 3 container
                </div>
            </div>'
        ));
        $column->appendTo($row);
        $column = Framework\Column::ofWidth(2);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding-right: 0;padding-left: 0;text-align: center;">
                    <span class="dashicons dashicons-arrow-right-alt"></span>
             </div>'
        ));
        $column->appendTo($row);
        $column = Framework\Column::ofWidth(5);
        $column->addElement(Framework\ElementFrom::content(
            '<div class="tbk-content" style="padding:10px;">
                <div style="background: #928686;
                            min-height: 180px;
                            border: 1px solid #5a5a5a;
                            color: white;
                            font-size: 0.6vw;
                            line-height: 0.7vw;
                            padding: 4px;">
                    14:00 - 18:00<br>
                    Service 1 + Service 2 + Service 3 container
                    <div class="tbk-column tbk-span-12" style="position:absolute;">
                        <div class="tbk-content" style="padding:10px;">
                            <div style="background: #' . $colors[ $this->coworker->getCustomEventSettings($this->service->getId())->getBookedEventColor() ] . ';
                                        min-height: 35px;
                                        border: 1px solid #5a5a5a;
                                        color: white;
                                        font-size: 0.6vw;
                                        line-height: 0.7vw;
                                        padding: 4px;">
                                15:00 - 16:00<br>
                                New reservation for Service 1
                            </div>
                        </div>
                    </div>
                    <div class="tbk-column tbk-span-12" style="position:absolute;top: 100px;">
                        <div class="tbk-content" style="padding:10px;">
                            <div style="background: #' . $colors[ $this->coworker->getCustomEventSettings($this->service->getId())->getBookedEventColor() ] . ';
                                        min-height: 55px;
                                        border: 1px solid #5a5a5a;
                                        color: white;
                                        font-size: 0.6vw;
                                        line-height: 0.7vw;
                                        padding: 4px;">
                                16:15 - 17:15<br>
                                New reservation for Service 3
                            </div>
                        </div>
                    </div>
                </div>
            </div>'
        ));
        $column->appendTo($row);
        $element->addContent($row);
        $element->appendTo($panel);

        return $panel;
    }


    /**
     * The Google Calendar settings panel
     *
     * @return Framework\Panel
     */
    private function getGCalSettingsContent()
    {
        $custom_booking_type_settings = $this->coworker->getCustomEventSettings($this->service->getId());
        $panel = new Framework\Panel(ucfirst(__('Your Google Calendar settings', 'team-booking')));

        // Event title (free slot)
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingText(__('Event title (free slot)', 'team-booking'));
            $element->addFieldname('data[linked_event_title]');
            $element->addDescription(__('Case insensitive. This is the name that events in your Google Calendar must have, in order to become free slots for this service.', 'team-booking'));
            $element->addDefaultValue($custom_booking_type_settings->getLinkedEventTitle());
            $element->appendTo($panel);
        }

        // Event title (booked slot)
        if ($this->service->isClass('appointment')) {
            $element = new Framework\PanelSettingText(__('Event title (booked slot)', 'team-booking'));
            $element->addFieldname('data[booked_title]');
            $element->addDescription(__('Case insensitive. This is the name that a free slot in your Google Calendar will get, once booked. If you manually give this name to an event, it will appear as a booked slot.', 'team-booking'));
            $element->addDefaultValue($custom_booking_type_settings->getAfterBookedTitle());
            $element->appendTo($panel);
        }

        // When reservations should be closed?
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingDoubleSelector(__('When reservations should be closed?', 'team-booking'));
            $element->addFieldname2('data[min_time]', 'data[min_time_reference]');
            $element->addDescription(__('Choose the minimum time to book prior to the slot start or end time', 'team-booking'));
            $element->setSelected2($custom_booking_type_settings->getMinTime(), $custom_booking_type_settings->getMinTimeReference());
            $element->addOption2('PT10M', __('10 min before', 'team-booking'), 'data[min_time]');
            $element->addOption2('PT30M', __('30 min before', 'team-booking'), 'data[min_time]');
            $element->addOption2('PT1H', __('1 hour before', 'team-booking'), 'data[min_time]');
            $element->addOption2('PT2H', __('2 hours before', 'team-booking'), 'data[min_time]');
            $element->addOption2('PT3H', __('3 hours before', 'team-booking'), 'data[min_time]');
            $element->addOption2('PT6H', __('6 hours before', 'team-booking'), 'data[min_time]');
            $element->addOption2('PT12H', __('12 hours before', 'team-booking'), 'data[min_time]');
            $element->addOption2('P1D', __('24 hours before', 'team-booking'), 'data[min_time]');
            $element->addOption2('P1Dmid', __('1 day before (until midnight)', 'team-booking'), 'data[min_time]');
            $element->addOption2('P2Dmid', __('2 days before (until midnight)', 'team-booking'), 'data[min_time]');
            $element->addOption2('P3Dmid', __('3 days before (until midnight)', 'team-booking'), 'data[min_time]');
            $element->addOption2('P7Dmid', __('7 days before (until midnight)', 'team-booking'), 'data[min_time]');
            $element->addOption2('P10Dmid', __('10 days before (until midnight)', 'team-booking'), 'data[min_time]');
            $element->addOption2('P15Dmid', __('15 days before (until midnight)', 'team-booking'), 'data[min_time]');
            $element->addOption2('P30Dmid', __('30 days before (until midnight)', 'team-booking'), 'data[min_time]');
            $element->addOption2('start', __('slot start time', 'team-booking'), 'data[min_time_reference]');
            $element->addOption2('end', __('slot end time', 'team-booking'), 'data[min_time_reference]');
            $element->appendTo($panel);
        }

        // When reservations should be opened?
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingSelector(__('When reservations should be opened?', 'team-booking'));
            $element->addFieldname('data[open_time]');
            $element->addDescription(__('When we are closer than the following time span to the start time of the event, the slots begin to appear. If "Immediately", they appear right after their creation. If not "Immediately", the value must be greater than the previous one!', 'team-booking'));
            $element->setSelected($custom_booking_type_settings->getOpenTime());
            $element->addOption('0', __('Immediately', 'team-booking'));
            $element->addOption('P1D', __('24 hours before', 'team-booking'));
            $element->addOption('P1Dmid', __('1 day before (until midnight)', 'team-booking'));
            $element->addOption('P2Dmid', __('2 days before (until midnight)', 'team-booking'));
            $element->addOption('P3Dmid', __('3 days before (until midnight)', 'team-booking'));
            $element->addOption('P7Dmid', __('7 days before (until midnight)', 'team-booking'));
            $element->addOption('P10Dmid', __('10 days before (until midnight)', 'team-booking'));
            $element->addOption('P15Dmid', __('15 days before (until midnight)', 'team-booking'));
            $element->addOption('P30Dmid', __('30 days before (until midnight)', 'team-booking'));
            $element->appendTo($panel);
        }

        // Slot duration rule
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Slot duration rule', 'team-booking'));
            $element->addDescription(__('If inherited, the container mode will not be available', 'team-booking'));
            $element->addFieldname('data[duration_rule]');
            if ($this->service->getDurationRule() !== 'coworker') {
                $service_hours = floor($this->service->getDefaultDuration() / HOUR_IN_SECONDS);
                $service_minutes = ceil($this->service->getDefaultDuration() - $service_hours * HOUR_IN_SECONDS) / MINUTE_IN_SECONDS;
                if ($this->service->getDurationRule() === 'fixed') {
                    $rule_string = esc_html__('Fixed', 'team-booking') . ' ' . $service_hours . 'h ' . $service_minutes . 'm';
                }
                if ($this->service->getDurationRule() === 'inherited') {
                    $rule_string = esc_html__('Inherited from Google Calendar event', 'team-booking');
                }
                ob_start();
                ?>
                <div class="ui mini blue label"><?= $rule_string ?>
                    <div class="detail"><?= '(' . esc_html__('Set by Admin', 'team-booking') . ')' ?></div>
                </div>
                <?php
                $element->addToDescription(ob_get_clean(), FALSE);
            } else {
                $element->addOption(array(
                    'label'       => __('Inherited from Google Calendar event', 'team-booking'),
                    'label_title' => __('Inherited from Google Calendar event', 'team-booking'),
                    'value'       => 'inherited',
                    'checked'     => $custom_booking_type_settings->getDurationRule() === 'inherited'
                ));
                $label_element = new Framework\PanelSettingTimespanSelector(__('Fixed', 'team-booking'));
                $label_element->addFieldname('data[default_duration]');
                $label_element->isNested(TRUE);
                $label_element->setHoursLabel(__('hours', 'team-booking'));
                $label_element->setMinsLabel(__('minutes', 'team-booking'));
                $label_element->setSelectedHours(floor($custom_booking_type_settings->getFixedDuration() / HOUR_IN_SECONDS));
                $label_element->setSelectedMins(ceil($custom_booking_type_settings->getFixedDuration() - floor($custom_booking_type_settings->getFixedDuration() / HOUR_IN_SECONDS) * HOUR_IN_SECONDS) / MINUTE_IN_SECONDS);
                $element->addOption(array(
                    'label'       => $label_element,
                    'label_title' => __('Fixed', 'team-booking'),
                    'value'       => 'fixed',
                    'checked'     => (int)('fixed' === $custom_booking_type_settings->getDurationRule())
                ));
            }
            $element->appendTo($panel);
        }

        // Buffer between consecutive slots
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingTimespanSelector(__('Buffer between consecutive slots', 'team-booking'));
            $element->addFieldname('data[buffer]');
            $element->addDescription(__('This applies only inside container slots', 'team-booking'));
            $element->setHoursLabel(__('hours', 'team-booking'));
            $element->setMinsLabel(__('minutes', 'team-booking'));
            $element->setSelectedHours(floor($custom_booking_type_settings->getBufferDuration() / HOUR_IN_SECONDS));
            $element->setSelectedMins(ceil($custom_booking_type_settings->getBufferDuration() - floor($custom_booking_type_settings->getBufferDuration() / HOUR_IN_SECONDS) * HOUR_IN_SECONDS) / MINUTE_IN_SECONDS);
            $element->appendTo($panel);
        }

        // Event color after reservation
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingColoredRadios(__('Event color after reservation', 'team-booking'));
            $element->addFieldname('data[booked_color]');
            $element->addDescription(__('Visual aid to distinguish between free and reserved slots in your Google Calendar', 'team-booking'));
            $element->setDefaultRadio(0, __('Calendar default color', 'team-booking'));
            $element->setSelected($custom_booking_type_settings->getBookedEventColor());
            $element->addRadio(1, 'a4bdfc');
            $element->addRadio(2, '7ae7bf');
            $element->addRadio(3, 'dbadff');
            $element->addRadio(4, 'ff887c');
            $element->addRadio(5, 'fbd75b');
            $element->addRadio(6, 'ffb878');
            $element->addRadio(7, '46d6db');
            $element->addRadio(8, 'e1e1e1');
            $element->addRadio(9, '5484ed');
            $element->addRadio(10, '51b749');
            $element->addRadio(11, 'dc2127');
            $element->appendTo($panel);
        }

        // Reminder
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Set a reminder for reserved slot', 'team-booking'));
            $element->addFieldname('data[reminder]');
            $element->addDescription(__('If active, you will receive a reminder by email for any reserved slot for this service.', 'team-booking'));
            $element->addOption(array(
                'label'       => __('No reminder', 'team-booking'),
                'label_title' => __('No reminder', 'team-booking'),
                'value'       => 0,
                'checked'     => $custom_booking_type_settings->getReminder() == 0
            ));
            $element->addOption(array(
                'label'       => __('10 min before', 'team-booking'),
                'label_title' => __('10 min before', 'team-booking'),
                'value'       => 10,
                'checked'     => $custom_booking_type_settings->getReminder() == 10
            ));
            $element->addOption(array(
                'label'       => __('30 min before', 'team-booking'),
                'label_title' => __('30 min before', 'team-booking'),
                'value'       => 30,
                'checked'     => $custom_booking_type_settings->getReminder() == 30
            ));
            $element->addOption(array(
                'label'       => __('1 hour before', 'team-booking'),
                'label_title' => __('1 hour before', 'team-booking'),
                'value'       => 60,
                'checked'     => $custom_booking_type_settings->getReminder() == 60
            ));
            $element->appendTo($panel);
        }

        // Event description content
        if ($this->service->isClass('event')) {
            $element = new Framework\PanelSettingRadios(__('Google Calendar event description content', 'team-booking'));
            $element->addFieldname('data[event_description_content]');
            $element->addDescription(__(
                'Specify the content of the Google Calendar event description. If privacy is your concern, then keep in mind: this is visible by customers.',
                'team-booking'
            ));
            $element->addOption(array(
                'label'       => __('Leave it blank', 'team-booking'),
                'label_title' => __('Leave it blank', 'team-booking'),
                'value'       => 0,
                'checked'     => $custom_booking_type_settings->getEventDescriptionContent() == 0
            ));
            $element->addOption(array(
                'label'       => __("Customer's names, tickets and email addresses", 'team-booking'),
                'label_title' => __("Customer's names, tickets and email addresses", 'team-booking'),
                'value'       => 1,
                'checked'     => $custom_booking_type_settings->getEventDescriptionContent() == 1
            ));
            $element->appendTo($panel);
        }

        // Deal with unrelated events
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Overlapping with personal events', 'team-booking'));
            $element->addFieldname('data[deal_with_unrelated_events]');
            $element->addDescription(__(
                'If you have personal events in your Google Calendar(s), choose what to do with a free slot of this service, in case one of them overlaps.',
                'team-booking'
            ));
            $element->addOption(array(
                'label'       => __('Keep the free slot', 'team-booking'),
                'label_title' => __('Keep the free slot', 'team-booking'),
                'value'       => 0,
                'checked'     => $custom_booking_type_settings->dealWithUnrelatedEvents() == 0
            ));
            $element->addOption(array(
                'label'       => __('Discard the free slot', 'team-booking'),
                'label_title' => __('Discard the free slot', 'team-booking'),
                'value'       => 1,
                'checked'     => $custom_booking_type_settings->dealWithUnrelatedEvents() == 1
            ));
            $element->appendTo($panel);
        }

        // Deal with booked slots of the same service
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Overlapping with booked slots of the same service', 'team-booking'));
            $element->addFieldname('data[deal_with_booked_same]');
            $element->addDescription(__(
                'If your availability plan envisages overlapping availabilities, choose what to do with a free slot of this service when a booked slot of the same service overlaps.',
                'team-booking'
            ));
            $element->addOption(array(
                'label'       => __('Keep the free slot', 'team-booking'),
                'label_title' => __('Keep the free slot', 'team-booking'),
                'value'       => 0,
                'checked'     => $custom_booking_type_settings->dealWithBookedOfSameService() == 0
            ));
            $element->addOption(array(
                'label'       => __('Discard the free slot', 'team-booking'),
                'label_title' => __('Discard the free slot', 'team-booking'),
                'value'       => 1,
                'checked'     => $custom_booking_type_settings->dealWithBookedOfSameService() == 1
            ));
            $element->appendTo($panel);
        }

        // Deal with booked slots of the other services
        if (!$this->service->isClass('service')) {
            $element = new Framework\PanelSettingRadios(__('Overlapping with booked slots of other services', 'team-booking'));
            $element->addFieldname('data[deal_with_booked_others]');
            $element->addDescription(__(
                'If your availability plan envisages overlapping availabilities, choose what to do with a free slot of this service when a booked slot of any other service overlaps.',
                'team-booking'
            ));
            $element->addOption(array(
                'label'       => __('Keep the free slot', 'team-booking'),
                'label_title' => __('Keep the free slot', 'team-booking'),
                'value'       => 0,
                'checked'     => $custom_booking_type_settings->dealWithBookedOfOtherServices() == 0
            ));
            $element->addOption(array(
                'label'       => __('Discard the free slot', 'team-booking'),
                'label_title' => __('Discard the free slot', 'team-booking'),
                'value'       => 1,
                'checked'     => $custom_booking_type_settings->dealWithBookedOfOtherServices() == 1
            ));
            $element->appendTo($panel);
        }


        // Save changes
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_save_personal_event_settings');
        $element->appendTo($panel);

        return $panel;
    }

    //------------------------------------------------------------

    private function getMozCheckboxCSSTarget($hex_color)
    {
        ob_start();
        ?>
        <style type="text/css">
            @-moz-document url-prefix() {
            <?= "input[type='checkbox']._" . $hex_color ?> {
                outline: 2px solid <?= '#'.$hex_color ?>;
            }
            }
        </style>
        <?php
        return ob_get_clean();
    }

}
