<?php

// Blocks direct access to this file
defined('ABSPATH') or die("No script kiddies please!");

use TeamBooking\Toolkit;

/**
 * This class defines the specific coworker settings
 * for a specific service. Mostly, those settings are
 * about the relationship between the service and the
 * coworker's Google Calendar(s)
 */
class TeamBookingCustomBTSettings
{
    ///////////////////////////////
    // Service-GCal relationship //
    ///////////////////////////////
    private $linked_event_title;
    private $after_booked_title;
    private $min_time;
    private $min_time_reference;
    private $open_time;
    private $booked_color;
    private $fixed_duration;
    private $duration_rule;
    private $buffer_duration;
    private $reminder;
    private $event_description_content;
    // overlapping slots:
    private $deal_with_unrelated_events;
    private $deal_with_same_service_booked_slots;
    private $deal_with_other_service_booked_slots;
    ////////////////////
    // Email settings //
    ////////////////////
    private $email;
    private $get_details_by_email;
    private $include_uploaded_files_as_attachment;
    //////////////////////
    // Service settings //
    //////////////////////
    private $participate;

    public function __construct()
    {
        $this->email = array(
            'email_text' => array(
                'subject' => "",
                'body'    => "",
            ),
        );
    }

    //------------------------------------------------------------

    public function getFixedDuration()
    {
        return (!$this->fixed_duration) ? 3600 : $this->fixed_duration;
    }

    public function setFixedDuration($seconds)
    {
        $this->fixed_duration = (int)$seconds;
    }

    //------------------------------------------------------------

    public function setMinTimeReferenceStart()
    {
        $this->min_time_reference = 'start';
    }

    public function setMinTimeReferenceEnd()
    {
        $this->min_time_reference = 'end';
    }

    public function getMinTimeReference()
    {
        return (!$this->min_time_reference) ? 'start' : $this->min_time_reference;
    }

    //------------------------------------------------------------

    public function getBufferDuration()
    {
        return (!$this->buffer_duration) ? 0 : $this->buffer_duration;
    }

    public function setBufferDuration($seconds)
    {
        $this->buffer_duration = (int)$seconds;
    }

    //------------------------------------------------------------

    public function getDurationRule()
    {
        return (!$this->duration_rule) ? 'inherited' : $this->duration_rule;
    }

    public function setDurationRule($rule)
    {
        $this->duration_rule = $rule;
    }

    //------------------------------------------------------------

    public function getLinkedEventTitle()
    {
        return $this->linked_event_title;
    }

    public function setLinkedEventTitle($title)
    {
        $this->linked_event_title = $title;
    }

    //------------------------------------------------------------

    public function getAfterBookedTitle()
    {
        return $this->after_booked_title;
    }

    public function setAfterBookedTitle($title)
    {
        $this->after_booked_title = $title;
    }

    //------------------------------------------------------------

    public function getEventDescriptionContent()
    {
        /**
         * 0 = leave it blank
         * 1 = name, tickets, email
         */
        return isset($this->event_description_content) ? $this->event_description_content : 1; // default is 1 (leave blank) due to legacy
    }

    public function setEventDescriptionContent($content)
    {
        $this->event_description_content = $content;
    }

    //------------------------------------------------------------

    public function getMinTime()
    {
        return $this->min_time;
    }

    public function setMinTime($interval)
    {
        $this->min_time = $interval;
    }

    public function getOpenTime()
    {
        return (!isset($this->open_time)) ? 0 : $this->open_time;
    }

    public function setOpenTime($interval)
    {
        $this->open_time = $interval;
    }

    //------------------------------------------------------------

    public function setGetDetailsByEmail($bool)
    {
        $this->get_details_by_email = (bool)$bool;
    }

    public function getGetDetailsByEmail()
    {
        return $this->get_details_by_email;
    }

    //------------------------------------------------------------

    public function setParticipate($bool)
    {
        $this->participate = (bool)$bool;
    }

    public function isParticipate()
    {
        return isset($this->participate) ? $this->participate : TRUE;
    }

    //------------------------------------------------------------

    public function setIncludeFilesAsAttachment($bool)
    {
        $this->include_uploaded_files_as_attachment = (bool)$bool;
    }

    public function getIncludeFilesAsAttachment()
    {
        return isset($this->include_uploaded_files_as_attachment) ? $this->include_uploaded_files_as_attachment : FALSE;
    }

    //------------------------------------------------------------

    public function setBookedEventColor($color)
    {
        $this->booked_color = (int)$color;
    }

    public function getBookedEventColor()
    {
        return $this->booked_color;
    }

    //------------------------------------------------------------

    public function getReminder()
    {
        return $this->reminder;
    }

    public function setReminder($seconds)
    {
        $this->reminder = (int)$seconds;
    }

    //------------------------------------------------------------

    public function dealWithUnrelatedEvents()
    {
        return isset($this->deal_with_unrelated_events) ? (bool)$this->deal_with_unrelated_events : FALSE;
    }

    public function setDealWithUnrelatedEvents($bool)
    {
        $this->deal_with_unrelated_events = (bool)$bool;
    }

    public function dealWithBookedOfSameService()
    {
        return isset($this->deal_with_same_service_booked_slots) ? (bool)$this->deal_with_same_service_booked_slots : FALSE;
    }

    public function setDealWithBookedOfSameService($bool)
    {
        $this->deal_with_same_service_booked_slots = (bool)$bool;
    }

    public function dealWithBookedOfOtherServices()
    {
        return isset($this->deal_with_other_service_booked_slots) ? (bool)$this->deal_with_other_service_booked_slots : FALSE;
    }

    public function setDealWithBookedOfOtherServices($bool)
    {
        $this->deal_with_other_service_booked_slots = (bool)$bool;
    }

    //------------------------------------------------------------

    public function setNotificationEmailSubject($text)
    {
        $this->email['email_text']['subject'] = Toolkit\filterInput($text);
    }

    public function getNotificationEmailSubject()
    {
        return Toolkit\unfilterInput($this->email['email_text']['subject']);
    }

    //------------------------------------------------------------

    public function setNotificationEmailBody($text)
    {
        $this->email['email_text']['body'] = Toolkit\filterInput($text);
    }

    public function getNotificationEmailBody()
    {
        return Toolkit\unfilterInput($this->email['email_text']['body']);
    }

}
