<?php

namespace TeamBooking;

// Blocks direct access to this file
defined('ABSPATH') or die("No script kiddies please!");

class Customer
{
    private $email;
    private $name;
    private $wp_user_id;
    private $reservations;
    private $reservations_obj;

    public function __construct(\WP_User $user, array $reservations)
    {
        /** @var $reservations \TeamBooking_ReservationData[] */
        $this->email = $user->user_email;
        $this->name = $user->display_name;
        if (isset($user->ID)) {
            $this->wp_user_id = $user->ID;
        }
        $this->reservations = array();
        foreach ($reservations as $reservation) {
            if (isset($this->reservations[ $reservation->getServiceId() ])) {
                $this->reservations[ $reservation->getServiceId() ]++;
            } else {
                $this->reservations[ $reservation->getServiceId() ] = 1;
            }
            $this->reservations_obj[ $reservation->getServiceId() ][] = $reservation;
        }
    }

    public function getEmail()
    {
        return $this->email;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getID()
    {
        return $this->wp_user_id;
    }

    public function getReservations($service_id = NULL)
    {
        if (!is_null($service_id)) {
            if (isset($this->reservations[ $service_id ])) {
                return $this->reservations[ $service_id ];
            } else {
                return 0;
            }
        } else {
            return $this->reservations;
        }
    }

    public function getEnumerableReservations($service_id)
    {
        $num = 0;
        if (isset($this->reservations_obj[ $service_id ])) {
            foreach ($this->reservations_obj[ $service_id ] as $reservation) {
                /** @var \TeamBooking_ReservationData $reservation */
                if ($reservation->isEnumerableForCustomerLimits()) $num++;
            }
        }

        return $num;
    }

    public function getTotalReservations()
    {
        return array_sum($this->reservations);
    }


}