<?php

namespace TeamBooking\Database;

// Blocks direct access to this file
defined('ABSPATH') or die("No script kiddies please!");

use TeamBooking\Functions,
    TeamBooking\Toolkit;

class Promotions
{
    /**
     * @param      $param
     * @param      $value
     * @param bool $running
     *
     * @return \TeamBooking\Promotions\Promotion[]
     */
    private static function getBy($param, $value, $running = FALSE)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . "teambooking_promotions";
        if ($running) {
            $results = $wpdb->get_results($wpdb->prepare("
                SELECT id, data_object
                FROM $table_name WHERE $param = %s AND start_time < NOW() AND end_time > NOW()
                ", array($value)));
        } else {
            $results = $wpdb->get_results($wpdb->prepare("
                SELECT id, data_object
                FROM $table_name WHERE $param = %s
                ", array($value)));
        }
        $return = array();
        foreach ($results as $result) {
            $return[ $result->id ] = unserialize($result->data_object);
        }

        return $return;
    }

    /**
     * @param $id
     *
     * @return \TeamBooking\Promotions\Promotion
     */
    public static function getById($id)
    {
        return reset(Promotions::getBy('id', $id));
    }

    /**
     * @param      $name
     * @param bool $running
     *
     * @return \TeamBooking\Promotions\Promotion[]
     */
    public static function getByName($name, $running = FALSE)
    {
        $promotions = Promotions::getAll($running);
        $return = array();
        foreach ($promotions as $id => $promotion) {
            if ($promotion->getName() == $name) {
                $return[ $id ] = $promotion;
                break;
            }
        }

        return $return;
    }

    /**
     * @param      $class
     * @param bool $running
     *
     * @return \TeamBooking\Promotions\Promotion[]
     */
    public static function getByClass($class, $running = FALSE)
    {
        return Promotions::getBy('class', $class, $running);
    }

    /**
     * @param      $id
     * @param bool $active
     * @param bool $running
     *
     * @return \TeamBooking\Promotions\Promotion[]
     */
    public static function getByService($id, $active = FALSE, $running = FALSE)
    {
        $promotions = Promotions::getAll($running);
        foreach ($promotions as $promotion_id => $promotion) {
            if (!$promotion->checkService($id)) {
                unset($promotions[ $promotion_id ]);
            } elseif ($active && !$promotion->getStatus()) {
                unset($promotions[ $promotion_id ]);
            }
        }

        return $promotions;
    }

    /**
     * @param      $class
     * @param      $service_id
     * @param bool $active
     * @param bool $running
     *
     * @return \TeamBooking\Promotions\Promotion[]
     */
    public static function getByClassAndService($class, $service_id, $active = FALSE, $running = FALSE)
    {
        $promotions = Promotions::getByService($service_id, $active, $running);
        foreach ($promotions as $promotion_id => $promotion) {
            if (!$promotion->checkClass($class)) {
                unset($promotions[ $promotion_id ]);
            }
        }

        return $promotions;
    }

    /**
     * @param bool $running
     *
     * @return \TeamBooking\Promotions\Promotion[]
     */
    public static function getAll($running = FALSE)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . "teambooking_promotions";
        if ($running) {
            $results = $wpdb->get_results("SELECT id, data_object FROM $table_name WHERE start_time < NOW() AND end_time > NOW()");
        } else {
            $results = $wpdb->get_results("SELECT id, data_object FROM $table_name");
        }
        $return = array();
        foreach ($results as $result) {
            $return[ $result->id ] = unserialize($result->data_object);
        }

        return $return;
    }

    /**
     * @param \TeamBooking\Promotions\Promotion $data
     *
     * @return mixed
     */
    public static function insert(\TeamBooking\Promotions\Promotion $data)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . "teambooking_promotions";
        if ($data instanceof \TeamBooking_Promotions_Campaign) {
            $class = 'campaign';
        } elseif ($data instanceof \TeamBooking_Promotions_Coupon) {
            $class = 'coupon';
        }
        $created = current_time('mysql');
        $data_object = serialize($data);
        $wpdb->insert($table_name, array(
            'created'     => $created,
            'class'       => $class,
            'start_time'  => date("Y-m-d H:i:s", $data->getStartTime()),
            'end_time'    => date("Y-m-d H:i:s", $data->getEndTime()),
            'data_object' => $data_object,
        ));
        $inserted_id = $wpdb->insert_id;

        return $inserted_id;
    }

    /**
     * @param                                   $id
     * @param \TeamBooking\Promotions\Promotion $data
     *
     * @return mixed
     */
    public static function update($id, \TeamBooking\Promotions\Promotion $data)
    {
        global $wpdb;
        $data_object = serialize($data);
        $table_name = $wpdb->prefix . "teambooking_promotions";
        $result = $wpdb->update($table_name,
            array(
                'data_object' => $data_object,
                'start_time'  => date("Y-m-d H:i:s", $data->getStartTime()),
                'end_time'    => date("Y-m-d H:i:s", $data->getEndTime()),
            ),
            array('id' => $id)
        );

        return $result;
    }

    /**
     * @param $id
     *
     * @return mixed
     */
    public static function delete($id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . "teambooking_promotions";
        $result = $wpdb->delete($table_name, array('id' => $id));

        return $result;
    }
}