<?php

namespace TeamBooking\Database;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Toolkit;

class Reservations
{
    /**
     * @param bool $pending
     *
     * @return \TeamBooking_ReservationData[]
     */
    public static function getAll($pending = FALSE)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        if ($pending) {
            $results = $wpdb->get_results($wpdb->prepare("SELECT id, data_object FROM $table_name WHERE status = %s", 'pending'));
        } else {
            $results = $wpdb->get_results("SELECT id, data_object FROM $table_name");
        }
        $return = array();
        foreach ($results as $result) {
            $obj = self::decode_object($result->data_object);
            if (!$obj) continue;
            /** @var $obj \TeamBooking_ReservationData */
            $obj->setDatabaseId($result->id);
            $return[ $result->id ] = $obj;
        }

        return $return;
    }

    /**
     * @param \TeamBooking_ReservationData $data
     *
     * @return mixed
     */
    public static function insert(\TeamBooking_ReservationData $data)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $created = current_time('mysql');
        $wpdb->insert($table_name, array(
            'created'     => $created,
            'service_id'  => $data->getServiceId(),
            'coworker_id' => $data->getCoworker(),
            'data_object' => self::encode_object($data),
            'calendar_id' => $data->getGoogleCalendarId(),
            'event_id'    => $data->getGoogleCalendarEvent(),
            'status'      => $data->getStatus(),
            'token'       => $data->getToken(),
            'start'       => $data->getStart(),
            'end'         => $data->getEnd()
        ));

        return $wpdb->insert_id;
    }

    /**
     * @param \TeamBooking_ReservationData $data
     *
     * @return mixed
     */
    public static function update(\TeamBooking_ReservationData $data)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $result = $wpdb->update($table_name, array(
            'data_object' => self::encode_object($data),
            'coworker_id' => $data->getCoworker(),
            'calendar_id' => $data->getGoogleCalendarId(),
            'event_id'    => $data->getGoogleCalendarEvent(),
            'service_id'  => $data->getServiceId(),
            'status'      => $data->getStatus(),
            'token'       => $data->getToken(),
            'start'       => $data->getStart(),
            'end'         => $data->getEnd()
        ), array('id' => $data->getDatabaseId()));

        return $result;
    }

    /**
     * @param                              $calendar_id
     * @param                              $event_id
     * @param \TeamBooking_ReservationData $data
     *
     * @return mixed
     */
    public static function updateByCalendarEventId($calendar_id, $event_id, \TeamBooking_ReservationData $data)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';

        return $wpdb->update(
            $table_name,
            array(
                'data_object' => self::encode_object($data),
                'coworker_id' => $data->getCoworker(),
                'event_id'    => $data->getGoogleCalendarEvent(),
                'calendar_id' => $data->getGoogleCalendarId(),
                'status'      => $data->getStatus(),
                'service_id'  => $data->getServiceId(),
                'start'       => $data->getStart(),
                'end'         => $data->getEnd(),
                'token'       => $data->getToken()
            ),
            array(
                'calendar_id' => $calendar_id,
                'event_id'    => $event_id
            )
        );
    }

    /**
     * @param $id
     *
     * @return mixed
     */
    public static function delete($id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        if (is_array($id)) {
            $how_many = count($id);
            $placeholders = array_fill(0, $how_many, '%d');
            $format = implode(', ', $placeholders);
            $result = $wpdb->get_results($wpdb->prepare("DELETE FROM $table_name WHERE id IN ($format)", $id));
        } else {
            $result = $wpdb->delete($table_name, array('id' => $id));
        }

        return $result;
    }

    /**
     * @param $id
     *
     * @return bool|\TeamBooking_ReservationData
     */
    public static function getById($id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $result = $wpdb->get_row($wpdb->prepare("SELECT data_object FROM $table_name WHERE id = %d", $id));
        if (!$result) return FALSE;
        $res = self::decode_object($result->data_object);
        if (!$res) return FALSE;
        /* @var $res \TeamBooking_ReservationData */
        $res->setDatabaseId($id);

        return $res;
    }

    /**
     * @param $calendar_id
     * @param $event_id
     *
     * @return \TeamBooking_ReservationData[]
     */
    public static function getByCalendarEventId($calendar_id, $event_id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $results = $wpdb->get_results($wpdb->prepare("SELECT id, data_object FROM $table_name WHERE calendar_id = %s AND event_id = %s", array($calendar_id, $event_id)));

        $return = array();
        foreach ($results as $result) {
            $res = self::decode_object($result->data_object);
            if (!$res) continue;
            /* @var $res \TeamBooking_ReservationData */
            $res->setDatabaseId($result->id);
            $return[ $result->id ] = $res;
        }

        return $return;
    }

    /**
     * @param $id
     *
     * @return \TeamBooking_ReservationData[]
     */
    public static function getByCoworker($id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $results = $wpdb->get_results($wpdb->prepare("SELECT id, data_object FROM $table_name WHERE coworker_id = %d", $id));
        $return = array();
        foreach ($results as $result) {
            $res = self::decode_object($result->data_object);
            if (!$res) continue;
            /* @var $res \TeamBooking_ReservationData */
            $res->setDatabaseId($result->id);
            $return[ $result->id ] = $res;
        }

        return $return;
    }

    /**
     * @param $token
     *
     * @return \TeamBooking_ReservationData
     */
    public static function getByToken($token)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $result = $wpdb->get_row($wpdb->prepare("SELECT id, data_object FROM $table_name WHERE token = %s", $token));
        $return = FALSE;
        if (NULL !== $result) {
            $res = self::decode_object($result->data_object);
            if (!$res) return FALSE;
            /* @var $res \TeamBooking_ReservationData */
            $res->setDatabaseId($result->id);
            $return = $res;
        }

        return $return;
    }

    /**
     * @param $id
     *
     * @return \TeamBooking_ReservationData[]
     */
    public static function getByService($id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $results = $wpdb->get_results($wpdb->prepare("SELECT id, data_object FROM $table_name WHERE service_id = %s", $id));
        $return = array();
        foreach ($results as $result) {
            $res = self::decode_object($result->data_object);
            if (!$res) continue;
            /* @var $res \TeamBooking_ReservationData */
            $res->setDatabaseId($result->id);
            $return[ $result->id ] = $res;
        }

        return $return;
    }

    /**
     * @param array $services
     *
     * @return \TeamBooking_ReservationData[]
     */
    public static function getByServices(array $services)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $how_many = count($services);
        $placeholders = array_fill(0, $how_many, '%d');
        $format = implode(', ', $placeholders);
        $results = $wpdb->get_results($wpdb->prepare("SELECT id, data_object FROM $table_name WHERE service_id IN ($format)", $services));
        $return = array();
        foreach ($results as $result) {
            $res = self::decode_object($result->data_object);
            if (!$res) continue;
            /* @var $res \TeamBooking_ReservationData */
            $res->setDatabaseId($result->id);
            $return[ $result->id ] = $res;
        }

        return $return;
    }

    public static function getByServiceAndCoworker($service_id, $coworker_id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $results = $wpdb->get_results($wpdb->prepare("SELECT id, data_object FROM $table_name WHERE service_id = %s AND coworker_id = %s", array($service_id, $coworker_id)));
        $return = array();
        foreach ($results as $result) {
            $res = self::decode_object($result->data_object);
            if (!$res) continue;
            /* @var $res \TeamBooking_ReservationData */
            $res->setDatabaseId($result->id);
            $return[ $result->id ] = $res;
        }

        return $return;
    }

    /**
     * @return array
     */
    public static function getIDs()
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $results = $wpdb->get_results("SELECT id FROM $table_name");
        $return = array();
        foreach ($results as $result) {
            $return[] = $result->id;
        }

        return $return;
    }

    /**
     * @param $coworker_id
     *
     * @return array
     */
    public static function getIDsByCoworker($coworker_id)
    {
        global $wpdb;
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $results = $wpdb->get_results($wpdb->prepare("SELECT id FROM $table_name WHERE coworker_id = %d", $coworker_id));
        $return = array();
        foreach ($results as $result) {
            $return[] = $result->id;
        }

        return $return;
    }

    /**
     * @param      $min_time
     * @param      $max_time
     * @param bool $pending
     *
     * @return \TeamBooking_ReservationData[]
     */
    public static function getByTime($min_time, $max_time, $pending = FALSE)
    {
        global $wpdb;
        if (is_string($min_time)) $min_time = strtotime($min_time);
        if (is_string($max_time)) $max_time = strtotime($max_time);
        $table_name = $wpdb->prefix . 'teambooking_reservations';
        $query = "SELECT id, data_object FROM $table_name WHERE `end` >= %d";
        $params = array($min_time);
        if ($max_time != NULL) {
            $query .= " AND start <= %d";
            $params[] = $max_time;
        }
        if ($pending) {
            $query .= " AND status = %s";
            $params[] = 'pending';
        }
        $results = $wpdb->get_results($wpdb->prepare($query, $params));
        $return = array();
        foreach ($results as $result) {
            $res = self::decode_object($result->data_object);
            if (!$res) continue;
            /** @var $res \TeamBooking_ReservationData */
            $res->setDatabaseId($result->id);
            $return[ $result->id ] = $res;
        }

        return $return;
    }

    private static function decode_object($obj)
    {
        $obj_base = base64_decode($obj, TRUE);
        if (!$obj_base) {
            $obj = unserialize($obj);
        } else {
            $obj = unserialize(gzinflate($obj_base));
        }

        return $obj;
    }

    private static function encode_object($obj)
    {
        return base64_encode(gzdeflate(serialize($obj)));
    }

}