<?php

namespace TeamBooking\Frontend;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Toolkit,
    TeamBooking\Functions;

class Calendar
{
    /** @var \TeamBooking\SlotsResults */
    public $slots_obj;
    private $settings;
    private $params;
    private $timezones;
    private $now;

    public function __construct(\TeamBooking\RenderParameters $params)
    {
        // Day number of today
        $this->now = \DateTime::createFromFormat('U', current_time('timestamp', TRUE));
        $this->now->setTimezone($params->getTimezone());
        $this->params = $params;
        $this->settings = Functions\getSettings();
        $this->timezones = \DateTimeZone::listIdentifiers(\DateTimeZone::ALL);
    }

    //------------------------------------------------------------

    public function getContent()
    {
        // Start of week (day), 0 = Sunday
        $week_starting_day = get_option('start_of_week');
        // This should be plain English, not localized
        $week_starting_day_textual = date('D', strtotime("Sunday +{$week_starting_day} days"));
        // Week day of month's first day
        $first_month_day_position_from_week_start = date_i18n('N', mktime(0, 0, 0, $this->params->getMonth(), 1, $this->params->getYear())) - $week_starting_day;
        // If negative, add 7
        if ($first_month_day_position_from_week_start < 0) {
            $first_month_day_position_from_week_start += 7;
        }
        // Number of days in month
        $number_of_month_days = date_i18n('t', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01'));
        // Number of graphic calendar rows needed
        $rows = ceil(($number_of_month_days + $first_month_day_position_from_week_start) / 7);
        // Matrix index init
        $j = 0;
        ob_start();
        // Let's avoid double styling and filtering renders due to ajax calls
        if (!$this->params->getIsAjaxCall()) {
            echo $this->getCalendarStyle();
            ?>
            <!-- calendar list settings row -->
            <div class="tbk-main-calendar-settings tbk-noselection">

                <div class="tbk-filters">
                    <?php if (!$this->params->getNoFilter()) { ?>
                        <?php if (count($this->params->getServiceIds()) > 1) {
                            $this->getServicesFilterButton();
                        }
                        if (count($this->params->getCoworkerIds()) !== 1) {
                            $this->getCoworkersFilterButton();
                        } ?>
                    <?php } ?>
                </div>

                <div>
                    <?php if (!$this->params->getNoTimezone()) { ?>
                        <div class="tbk-setting-button tbk-timezones" tabindex="0" style="margin: 0"
                             title="<?= esc_html__('Timezone', 'team-booking') ?>"
                             aria-label="<?= esc_html__('Timezone', 'team-booking') ?>">
                            <i class="world tb-icon"></i>
                            <span class="tbk-text"><?= $this->params->getTimezone()->getName() ?></span>
                            <div class="<?= $this->params->getIsWidget() ? 'mini' : 'tiny' ?> tbk-menu">
                                <?php foreach ($this->timezones as $timezone) { ?>
                                    <div
                                        class="tbk-menu-item <?= ($timezone == Toolkit\getTimezone()->getName()) ? 'tbk-selected' : '' ?>"
                                        data-timezone="<?= $timezone ?>">
                                        <?= $timezone ?>
                                    </div>
                                <?php } ?>
                            </div>
                        </div>
                    <?php } ?>
                </div>
            </div>
            <?php
        }

        ?>
        <div
            class="<?= ($this->params->getIsWidget() === TRUE) ? 'tb-widget' : '' ?> tb-frontend-calendar tbk-slide-container"
            data-params="<?= $this->params->encode() ?>" data-instance='<?= $this->params->getInstance() ?>'>
            <div class="tbk-dimmer"></div>
            <div class="tbk-slide-canvas tbk-slide-0">
                <div class="tbk-slide">
                    <div class="tbk-calendar-view-header">
                        <div class="ui grid" style="margin: 0">
                            <!-- calendar main navigation row -->
                            <div class="tbk-row tbk-main-calendar-navigation tbk-noselection" role="navigation"
                                 aria-label="<?= esc_html__('calendar controls', 'team-booking') ?>">
                                <div class="three wide tbk-column tb-change-month" tabindex="0"
                                     aria-label="<?= esc_html__('previous month', 'team-booking') ?>"
                                     data-month="<?= date_i18n('n', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01 - 1 month')) ?>"
                                     data-year="<?= date_i18n('Y', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01 - 1 month')) ?>">
                                    <span class="dashicons dashicons-arrow-left-alt2"></span>
                                </div>
                                <div class="ten wide tbk-column">
                                    <?php
                                    echo '<span class="tbk-calendar-month-selector" tabindex="0">';
                                    if ($this->params->getIsWidget() === TRUE) {
                                        echo strtoupper(date_i18n('M', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01')));
                                    } else {
                                        echo strtoupper(date_i18n('F', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01')));
                                    }
                                    echo '</span><span class="tbk-calendar-year-selector" tabindex="0">';
                                    if ($this->params->getIsWidget() === TRUE) {
                                        echo strtoupper(date_i18n('y', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01')));
                                    } else {
                                        echo strtoupper(date_i18n('Y', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01')));
                                    }
                                    echo '</span>';
                                    ?>
                                </div>
                                <div class="three wide tbk-column tb-change-month" tabindex="0"
                                     aria-label="<?= esc_html__('next month', 'team-booking') ?>"
                                     data-month="<?= date_i18n('n', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01 + 1 month')) ?>"
                                     data-year="<?= date_i18n('Y', strtotime($this->params->getYear() . '-' . $this->params->getMonth() . '-01 + 1 month')) ?>">
                                    <span class="dashicons dashicons-arrow-right-alt2"></span>
                                </div>
                            </div>
                            <!-- month selector row -->
                            <div class="tb-fast-selector-month-panel lifted equal width centered tbk-row">
                                <?php for ($i = 1; $i <= 12; $i++) { ?>
                                    <div
                                        class="four wide tbk-column aligned tbk-month-selector
                                <?= $this->now->format('n') == $i ? 'current' : '' ?>
                                <?= $this->params->getMonth() == $i ? 'selected' : '' ?>"
                                        tabindex="0"
                                        data-month="<?= date_i18n('m', mktime(0, 0, 0, $i, 1, date('Y'))) ?>"
                                        data-instance="<?= $this->params->getInstance() ?>">
                                        <?php
                                        $value = date_i18n('M', mktime(0, 0, 0, $i, 1, date('Y')));
                                        echo $value;
                                        ?>
                                    </div>
                                    <?php
                                }
                                ?>
                            </div>
                            <!-- year selector row -->
                            <div class="tb-fast-selector-year-panel lifted equal width centered tbk-row">
                                <?php for ($i = 1; $i <= 4; $i++) {
                                    $value = date_i18n('Y', mktime(0, 0, 0, 1, 1, date('Y') + ($i - 1)));
                                    ?>
                                    <div
                                        class="four wide tbk-column aligned tbk-year-selector
                                <?= $this->now->format('Y') == $value ? 'current' : '' ?>
                                <?= $this->params->getYear() == $value ? 'selected' : '' ?>"
                                        data-year="<?= $value ?>"
                                        data-instance="<?= $this->params->getInstance() ?>">
                                        <?= $value ?>
                                    </div>
                                    <?php
                                }
                                ?>
                            </div>
                        </div>
                    </div>
                    <div class="tbk-calendar-view-content">
                        <div class="tbk-monthly-view ui grid tbk-slide-out" style="margin: 0;">
                            <div class="tb-calendar-line equal width tbk-column tbk-row">
                                <?php
                                $i = 0;
                                while ($i < 7) {
                                    $date_string = date_i18n('D', strtotime("last $week_starting_day_textual + $i day"));
                                    ?>
                                    <div class="tbk-column">
                                        <div class="tb-weekline-day">
                                            <?php
                                            if ($this->params->getIsWidget() === TRUE) {
                                                if (extension_loaded('mbstring')) {
                                                    echo mb_substr($date_string, 0, 1);
                                                } else {
                                                    echo $date_string[0];
                                                }
                                            } else {
                                                echo $date_string;
                                            }
                                            ?>
                                        </div>
                                    </div>
                                    <?php
                                    $i++;
                                }
                                ?>

                            </div>

                            <?php
                            while ($j < $rows) {
                                $i = 0;
                                ?>
                                <div class="equal width tbk-column tbk-row tb-days">
                                    <?php
                                    while ($i < 7) {
                                        // Grid index
                                        $absolute_counter = $i + 1 + $j * 7;
                                        // Day index
                                        $relative_counter = $absolute_counter - $first_month_day_position_from_week_start;
                                        $css_classes = $this->getDayCssClass($relative_counter);
                                        // Get slots to include (to serve the slots list)
                                        $slots_to_include = $this->slots_obj->getSlotsByDate(str_pad($relative_counter, 2, '0', STR_PAD_LEFT), str_pad($this->params->getMonth(), 2, '0', STR_PAD_LEFT), $this->params->getYear());
                                        $slots_to_include = $this->params->encode($slots_to_include);
                                        ?>
                                        <div class="tbk-column"><?php
                                            if ($relative_counter >= 1 && $relative_counter <= $number_of_month_days) {
                                                ?>
                                                <div class="ui tb-day <?= $css_classes ?>"
                                                    <?= (strpos($css_classes, 'slots') !== FALSE) ? (' tabindex="0" aria-label="' . esc_html('available', 'team-booking') . '"') : '' ?>
                                                     data-day="<?= $relative_counter ?>"
                                                     data-slots="<?= $slots_to_include ?>">
                                                    <div>
                                                        <?= $relative_counter ?>
                                                    </div>
                                                    <?php
                                                    if ($this->params->getIsWidget() != TRUE) {
                                                        $this->getReservationsLeftLabel($relative_counter);
                                                    }
                                                    ?>
                                                </div>
                                            <?php } ?>
                                        </div>
                                        <?php
                                        $i++;
                                    }
                                    ?></div>
                                <?php
                                $j++;
                            }
                            ?>
                        </div>
                    </div>
                </div>
            </div>
        </div>

        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    /**
     * Renders the small numbered dots label
     *
     * @param $relative_counter
     *
     * @throws \Exception
     */
    private function getReservationsLeftLabel($relative_counter)
    {
        if ($this->settings->getNumberedDotsLogic() === 'hide') {
            return;
        }
        $slots = $this->slots_obj->getSlotsByDate(str_pad($relative_counter, 2, '0', STR_PAD_LEFT), str_pad($this->params->getMonth(), 2, '0', STR_PAD_LEFT), $this->params->getYear());
        if (!empty($slots)) {
            $reservations = '';
            $sorted_slots = array();
            foreach ($slots as $slot) {
                /* @var $slot \TeamBooking\Slot */
                if ($slot->isSoldout()) {
                    continue;
                }
                try {
                    if (isset($sorted_slots[ $slot->getServiceId() ])) {
                        if ($this->settings->getNumberedDotsLogic() === 'slots' ||
                            $this->settings->getService($slot->getServiceId())->isClass('appointment')
                        ) {
                            $sorted_slots[ $slot->getServiceId() ]++;
                        } elseif ($this->settings->getNumberedDotsLogic() === 'tickets') {
                            $sorted_slots[ $slot->getServiceId() ] += $this->settings->getService($slot->getServiceId())->getAttendees() - $slot->getAttendeesNumber();
                        }
                    } else {
                        if ($this->settings->getNumberedDotsLogic() === 'slots' ||
                            $this->settings->getService($slot->getServiceId())->isClass('appointment')
                        ) {
                            $sorted_slots[ $slot->getServiceId() ] = 1;
                        } elseif ($this->settings->getNumberedDotsLogic() === 'tickets') {
                            $sorted_slots[ $slot->getServiceId() ] = $this->settings->getService($slot->getServiceId())->getAttendees() - $slot->getAttendeesNumber();
                        }
                    }
                } catch (\Exception $e) {
                    continue;
                }
            }
            foreach ($sorted_slots as $service_id => $number) {
                $reservations .= $this->getReservationsLeftDot($number, $service_id);
            }
            if (!empty($reservations)) {
                ?>
                <div class="ui small pointing above label computer only tbk-column">
                    <?= $reservations ?>
                </div>
                <?php
            }
        }
    }

    //------------------------------------------------------------

    /**
     * Renders a single numbered slot
     *
     * @param $number
     * @param $service_id
     *
     * @return string
     * @throws \Exception
     */
    private function getReservationsLeftDot($number, $service_id)
    {
        try {
            $color = $this->settings->getService($service_id)->getServiceColor();
            $name = $this->settings->getService($service_id)->getName();
        } catch (\Exception $e) {
            return NULL;
        }
        if ($number < $this->settings->getNumberedDotsLowerBound()) {
            $number = NULL;
        }
        ob_start();
        ?>
        <span class='ui mini circular label tb-pointing-label-dots'
              style='background-color:<?= $color ?>;color:<?= Functions\getRightTextColor($color, TRUE) ?>;'
              title='<?= $name ?>'>
            <?= $number ?>
        </span>

        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    /**
     * @param $relative_counter
     *
     * @return string
     */
    private function getDayCssClass($relative_counter)
    {
        $slots = $this->slots_obj->getSlotsByDate(str_pad($relative_counter, 2, '0', STR_PAD_LEFT), str_pad($this->params->getMonth(), 2, '0', STR_PAD_LEFT), $this->params->getYear());
        $css_classes = '';
        if ($relative_counter == $this->now->format('j')
            && $this->params->getMonth() == $this->now->format('m')
            && $this->params->getYear() == $this->now->format('Y')
        ) {
            $css_classes = 'today';
        }
        if ((
                $this->params->getYear() > $this->now->format('Y')
            ) || (
                $this->params->getMonth() > $this->now->format('m')
                && $this->params->getYear() >= $this->now->format('Y')
            ) || ($relative_counter >= $this->now->format('j')
                && $this->params->getMonth() == $this->now->format('m')
                && $this->params->getYear() == $this->now->format('Y')
            )
        ) {
            if (!empty($slots)) {
                $free_places_detected = FALSE;
                foreach ($slots as $slot) {
                    /* @var $slot \TeamBooking\Slot */
                    if (!$slot->isSoldout()) {
                        $free_places_detected = TRUE;
                        break;
                    }
                }
                if ($free_places_detected) {
                    $css_classes .= ' slots';
                } else {
                    $css_classes .= ' slots soldout';
                }
            }
        } else {
            $css_classes .= ' pastday';
        }

        return $css_classes;
    }

    //------------------------------------------------------------

    private function getServicesFilterButton()
    {
        ?>
        <div class="tbk-setting-button tbk-services" tabindex="0" title="<?= esc_html__('Services', 'team-booking') ?>"
             aria-label="<?= esc_html__('service selector', 'team-booking') ?>">
            <i class="unordered list tb-icon"></i>
            <?php if (!$this->params->getIsWidget()) { ?>
                <span class="tbk-text"><?= esc_html__('What', 'team-booking') ?></span>
            <?php } ?>
            <div class="<?= $this->params->getIsWidget() ? 'mini' : 'tiny' ?> tbk-menu">
                <?php
                if (is_array($this->params->getRequestedServiceIds())) {
                    $services = $this->params->getRequestedServiceIds();
                } else {
                    $services = $this->params->getServiceIds();
                }
                ?>
                <?php foreach ($services as $service_id) { ?>
                    <div class="tbk-menu-item" data-service="<?= $service_id ?>"
                         data-instance="<?= $this->params->getInstance() ?>">
                        <div class="ui empty circular label"
                             style="background-color:<?= $this->settings->getService($service_id)->getServiceColor() ?>"></div>
                        <?php
                        echo $this->settings->getService($service_id)->getName();
                        if ($this->settings->getService($service_id)->getPrice() > 0) {
                            $price = $this->settings->getService($service_id)->getPrice();
                            $discounted_price = Functions\getDiscountedPrice($this->settings->getService($service_id));
                            if ($price != $discounted_price) {
                                $price_string = Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), NULL)
                                    . '<del>' . Functions\priceFormat($price) . '</del> <span class="tbk-discounted-price">' . Functions\priceFormat($discounted_price) . '</span>';
                            } else {
                                $price_string = Functions\currencyCodeToSymbol(
                                    $this->settings->getCurrencyCode(),
                                    $this->settings->getService($service_id)->getPrice()
                                );
                            }
                            ?>
                            <span class="tbk-item-detail"><?= $price_string ?></span>
                        <?php } ?>
                    </div>
                    <?php
                }
                ?>
                <div class="tbk-divider"></div>
                <div class="tbk-menu-item tbk-reset-filter tbk-selected"
                     data-services="<?= base64_encode(serialize($services)) ?>">
                    <?= esc_html__('All', 'team-booking') ?>
                </div>
            </div>
        </div>
        <?php
    }

    //------------------------------------------------------------

    private function getCoworkersFilterButton()
    {
        $coworkers_requested = $this->params->getRequestedCoworkerIds();
        if (!empty($coworkers_requested)) {
            $coworkers = $coworkers_requested;
        } else {
            $coworkers = Functions\getAuthCoworkersIdList();
            if (count($coworkers) < 2) return;
        }
        ?>
        <div class="tbk-setting-button tbk-coworkers" tabindex="0"
             title="<?= esc_html__('Coworkers', 'team-booking') ?>"
             aria-label="<?= esc_html__('coworker selector', 'team-booking') ?>">
            <i class="user tb-icon"></i>
            <?php if (!$this->params->getIsWidget()) { ?>
                <span class="tbk-text"><?= esc_html__('Who', 'team-booking') ?></span>
            <?php } ?>
            <div class="<?= $this->params->getIsWidget() ? 'mini' : 'tiny' ?> tbk-menu">
                <?php foreach ($coworkers as $coworker) { ?>
                    <div class="tbk-menu-item" data-coworker="<?= $coworker ?>"
                         data-instance="<?= $this->params->getInstance() ?>">
                        <?= $this->settings->getCoworkerData($coworker)->getFullName() ?>
                    </div>
                    <?php
                }
                ?>
                <div class="tbk-menu-item tbk-reset-filter tbk-selected"
                     data-coworkers="<?= base64_encode(serialize($coworkers)) ?>">
                    <?= esc_html__('All', 'team-booking') ?>
                </div>
            </div>
        </div>
        <?php
    }

    //------------------------------------------------------------

    public function getFormDirectly($content)
    {
        ?>
        <?= $this->getCalendarStyle() ?>
        <div
            class="<?= ($this->params->getIsWidget() === TRUE) ? 'tb-widget' : '' ?> tb-frontend-calendar tbk-slide-container"
            data-params="<?= $this->params->encode() ?>" data-instance='<?= $this->params->getInstance() ?>'>
            <div class="tbk-calendar-view-content">
                <div class="tbk-dimmer"></div>
                <div class="tbk-slide-canvas tbk-slide-0">
                    <div class="tbk-slide tbk-active"><?= $content ?></div>
                </div>
            </div>
        </div>
        <?php
    }

    //------------------------------------------------------------

    public function getCalendarStyle()
    {
        $border = $this->settings->getBorder();
        $pattern = $this->settings->getPattern();
        ob_start();
        ?>
        <style>
            .tb-frontend-calendar {
                background: <?= $this->settings->getColorBackground() ?>;
                background-image: url(<?= Toolkit\getPattern($pattern['calendar'], $this->settings->getColorBackground())?>);
                border: <?= $border['size'] ?>px solid <?= $border['color'] ?>;
                border-radius: <?= $border['radius'] ?>px;
            }

            .tb-frontend-calendar {
                color: <?= Functions\getRightTextColor($this->settings->getColorBackground()) ?>;
            }

            .tb-frontend-calendar .ui.tbk-divider {
                color: <?= Functions\getRightTextColor($this->settings->getColorBackground()) ?>;
            }

            .tb-frontend-calendar .ui.tb-day.pastday {
                color: <?= Functions\getRightHoverColor($this->settings->getColorBackground()) ?>;
            }

            .tbk-calendar-year-selector {
                color: <?= Functions\getRightHoverColor($this->settings->getColorBackground()) ?>;
            }

            .tbk-schedule-filter-item,
            .tbk-schedule-filter-icon {
                background: <?= Functions\getRightBackgroundColor($this->settings->getColorBackground()) ?>;
            }

            .tbk-schedule-filter-item:hover,
            .tbk-schedule-filter-item:focus,
            .tbk-schedule-filter-icon:hover,
            .tbk-schedule-filter-icon:focus {
                background: <?= Functions\getRightHoverColor($this->settings->getColorBackground()) ?>;
                outline: 0;
            }

            .tb-frontend-calendar .ui.tb-day.today,
            .tb-change-month:hover,
            .tb-change-month:focus,
            .tbk-back-to:hover,
            .tbk-back-to:focus,
            .tbk-column .tb-day:not(.pastday):hover,
            .tbk-column .tb-day:not(.pastday):focus,
            div.tb-fast-selector-month-panel .tbk-month-selector:hover,
            div.tb-fast-selector-month-panel .tbk-month-selector:focus,
            div.tb-fast-selector-year-panel .tbk-year-selector:hover,
            div.tb-fast-selector-year-panel .tbk-year-selector:focus {
                background-color: <?= Functions\getRightHoverColor($this->settings->getColorBackground()) ?>;
                outline: 0;
            }

            .tb-frontend-calendar .ui.tb-day.slots {
                background-color: <?= $this->settings->getColorFreeSlot() ?>;
                color: <?= Functions\getRightTextColor($this->settings->getColorFreeSlot()) ?>;
            }

            .tb-frontend-calendar .ui.tb-day.slots.soldout {
                background-color: <?= $this->settings->getColorSoldoutSlot() ?>;
                color: <?= Functions\getRightTextColor($this->settings->getColorSoldoutSlot()) ?>;
            }

            .tb-frontend-calendar .ui.tb-day.slots:hover,
            .tb-frontend-calendar .ui.tb-day.slots:focus {
                background: <?= $this->settings->getColorFreeSlot() ?>;
                background: linear-gradient(to bottom, rgba(0, 0, 0, 0.15) 0%, rgba(0, 0, 0, 0.15) 100%),
                linear-gradient(to bottom,  <?= $this->settings->getColorFreeSlot() ?> 0%,<?= $this->settings->getColorFreeSlot() ?> 100%); /* W3C */
                outline: 0;
            }

            .tb-frontend-calendar .ui.tb-day.slots.soldout:hover,
            .tb-frontend-calendar .ui.tb-day.slots.soldout:focus {
                background: <?= $this->settings->getColorSoldoutSlot() ?>;
                background: linear-gradient(to bottom, rgba(0, 0, 0, 0.15) 0%, rgba(0, 0, 0, 0.15) 100%),
                linear-gradient(to bottom,  <?= $this->settings->getColorSoldoutSlot() ?> 0%,<?= $this->settings->getColorSoldoutSlot() ?> 100%); /* W3C */
                outline: 0;
            }

            .tb-calendar-line.equal.width.tbk-column.tbk-row {
                background: <?= $this->settings->getColorWeekLine() ?>;
                background-image: url(<?= Toolkit\getPattern($pattern['weekline'], $this->settings->getColorWeekLine())?>);
                color: <?= Functions\getRightTextColor($this->settings->getColorWeekLine()) ?>;
            }
        </style>

        <?php
        return ob_get_clean();
    }

}
