<?php

namespace TeamBooking\Frontend;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Database,
    TeamBooking\Toolkit,
    TeamBooking\Frontend\Components;

class Form
{
    public $instance;
    public $start_end_times;
    public $coworker_string;
    public $service_location;
    /** @var \TeamBookingType */
    public $service;
    private $coworker_id;
    private $calendar_id;
    private $event_id;
    private $service_id;
    private $tickets_left;
    private $event_id_parent;
    private $slot_start;
    private $slot_end;
    /** @var \TeamBookingSettings */
    private $settings;

    private function scheduled(\TeamBooking\Slot $slot)
    {
        $this->settings = Functions\getSettings();
        try {
            if (!$this->settings->getService($slot->getServiceId())->isClass('service')) {
                $this->event_id = $slot->getEventId();
                $this->service_id = $slot->getServiceId();
                $this->coworker_id = $slot->getCoworkerId();
                $this->calendar_id = $slot->getCalendarId();
                $this->event_id_parent = $slot->getEventIdParent();
                $this->slot_start = $slot->getStartTime();
                $this->slot_end = $slot->getEndTime();
                $this->service = $this->settings->getService($slot->getServiceId());
                $this->tickets_left = $this->settings->getService($slot->getServiceId())->getAttendees() - $slot->getAttendeesNumber();
                $this->service_location = $slot->getLocation();
                $this->start_end_times = $slot->getTimesString();
                $this->coworker_string = $slot->getCoworkerDisplayString();
            }
        } catch (\Exception $e) {
            // do nothing
        }
    }

    private function unscheduled($service_id)
    {
        $this->settings = Functions\getSettings();
        $this->service_id = $service_id;
        /* @var $service \TeamBookingType */
        try {
            $this->service = $this->settings->getService($service_id);
            $this->service_location = $this->service->getLocationAddress();
        } catch (\Exception $e) {
            // do nothing
        }
    }

    public static function fromService($service_id)
    {
        $form = new Form();
        $form->unscheduled($service_id);

        return $form;
    }

    public static function fromSlot(\TeamBooking\Slot $slot)
    {
        $form = new Form();
        $form->scheduled($slot);

        return $form;
    }

    //------------------------------------------------------------

    public function getContent()
    {
        $string_total_tickets = __('Tickets', 'team-booking') . ':';
        $string_total_price = __('Total price', 'team-booking') . ':';
        $string_book_now = __('Book now', 'team-booking');
        $string_book_and_pay = __('Book and pay', 'team-booking');

        if (did_action('tbk_reservation_form_header') < 1) {
            add_action('tbk_reservation_form_header', array(
                $this,
                'form_header',
            ));
        }
        if (did_action('tbk_reservation_form_map') < 1) {
            add_action('tbk_reservation_form_map', array(
                $this,
                'form_map',
            ));
        }
        if (did_action('tbk_reservation_form_description') < 1) {
            add_action('tbk_reservation_form_description', array(
                $this,
                'form_description',
            ));
        }

        if (is_user_logged_in()) {
            // TODO: use only wp methods
            $user = $this->settings->getCoworkerData(get_current_user_id());
            // TODO: better logic
            if ($this->service->getMaxReservationsLoggedUser()) {
                $customer = new \TeamBooking\Customer(get_user_by('id', get_current_user_id()), Database\Reservations::getAll());
                $customer_reservations_left = $this->service->getMaxReservationsLoggedUser() - $customer->getEnumerableReservations($this->service_id);
                if ($customer_reservations_left <= 0) {
                    return $this->getContentReservationsFinished();

                }
            }
        }
        $form_fields = $this->service->getFormFields()->getAllFields(TRUE);
        $there_are_files = FALSE;
        $form_fields_markup = array();
        $form_fields_hidden_markup = array();

        foreach ($form_fields as $field_markup) {
            // Check for file upload fields
            if (get_class($field_markup) === 'TeamBookingFormFileUpload') {
                $there_are_files = TRUE;
            }
            $is_hidden = FALSE;
            /* @var $field_markup \TeamBookingFormTextField */
            switch ($field_markup->getHook()) {
                case 'first_name':
                    $default_value = (isset($user) && $this->settings->getAutofillReservationForm() ? $user->getFirstName() : '');
                    if (!empty($default_value) && $this->settings->getAutofillReservationForm() === 'hide') {
                        $is_hidden = TRUE;
                    }
                    break;
                case 'second_name':
                    $default_value = (isset($user) && $this->settings->getAutofillReservationForm() ? $user->getLastName() : '');
                    if (!empty($default_value) && $this->settings->getAutofillReservationForm() === 'hide') {
                        $is_hidden = TRUE;
                    }
                    break;
                case 'email':
                    $default_value = (isset($user) && $this->settings->getAutofillReservationForm() ? $user->getEmail() : '');
                    if (!empty($default_value) && $this->settings->getAutofillReservationForm() === 'hide') {
                        $is_hidden = TRUE;
                    }
                    break;
                case 'url':
                    $default_value = (isset($user) && $this->settings->getAutofillReservationForm() ? $user->getUrl() : '');
                    if (!empty($default_value) && $this->settings->getAutofillReservationForm() === 'hide') {
                        $is_hidden = TRUE;
                    }
                    break;
                default:
                    $default_value = $field_markup->getValue();
            }
            if (get_class($field_markup) === 'TeamBookingFormCheckbox') {
                $field_markup->setValue(__('Selected', 'team-booking'));
            } else {
                $field_markup->setValue($default_value);
            }

            if ($is_hidden) {
                ob_start();
                $field_markup->getHiddenMarkup();
                $result = ob_get_clean();
                $form_fields_hidden_markup[] = $result;
            } else {
                ob_start();
                $field_markup->getMarkup();
                // Address autocomplete by Google
                if ($field_markup->getHook() === 'address') {
                    ?>
                    <style>
                        .pac-container {
                            z-index: 99999999;
                            -webkit-filter: none !important;
                            filter: none !important;
                        }
                    </style>
                    <?php
                }
                $result = ob_get_clean();
                $form_fields_markup[] = $result;
            }
        }

        $timezone = Toolkit\getTimezone();
        $start_date_time_object = new \DateTime($this->slot_start);
        $start_date_time_object->setTimezone($timezone);

        ob_start();
        ?>
        <?php
        if (isset($this->event_id)) {
            echo Components\NavigationHeader::InReservationForm($this->slot_start);
        }
        ?>
        <div class="tbk-reservation-form-container">
            <?php Functions\reservation_form_header($this) ?>
            <div class="tbk-content">
                <?php Functions\reservation_form_description($this) ?>
                <?php Functions\reservation_form_map($this) ?>
                <!-- form section -->
                <div class="ui horizontal tbk-divider">
                    <i class="info letter tb-icon"></i>
                </div>
                <?php
                /**
                 * Now we're setting up the form
                 */
                $form_enctype = $there_are_files ? 'enctype="multipart/form-data"' : '';
                ?>
                <form class="tbk-reservation-form" method="POST" action="" <?= $form_enctype ?>>
                    <?php wp_nonce_field('teambooking_submit_reservation', 'nonce') ?>
                    <input type="hidden" name="tickets" value="1">
                    <input type="hidden" name="service" value="<?= esc_attr($this->service_id) ?>">
                    <input type="hidden" name="post_id" value="">
                    <input type="hidden" name="service_location" value="<?= $this->service_location ?>">
                    <input type="hidden" name="customer_wp_id"
                           value="<?= isset($user) ? esc_attr($user->getId()) : '' ?>">
                    <?php if ($this->service->getClass() !== 'service') { ?>
                        <input type="hidden" name="owner" value="<?= esc_attr($this->coworker_id) ?>">
                        <input type="hidden" name="calendar_id"
                               value="<?= Toolkit\base64UrlEncode($this->calendar_id) ?>">
                        <input type="hidden" name="event_id" value="<?= $this->event_id ?>">
                        <input type="hidden" name="event_id_parent" value="<?= $this->event_id_parent ?>">
                        <input type="hidden" name="slot_start" value="<?= $this->slot_start ?>">
                        <input type="hidden" name="slot_end" value="<?= $this->slot_end ?>">
                    <?php } ?>
                    <?php
                    /**
                     * Let's render the hidden fields (pre-filled user data)
                     * if any
                     */
                    foreach ($form_fields_hidden_markup as $field_markup) {
                        echo $field_markup;
                    }
                    ?>
                    <?php
                    // Start grouping (max two, for now)
                    $group_limit = NULL;
                    $number_of_fields = count($form_fields_markup);
                    if ($number_of_fields > 3 && $number_of_fields < 700) {
                        $group_limit = 2;
                        $group_limit_textual = 'two';
                        echo "<div class='$group_limit_textual tbk-fields'>";
                    } elseif ($number_of_fields >= 700) {
                        $group_limit = 3;
                        $group_limit_textual = 'three';
                        echo "<div class='$group_limit_textual tbk-fields'>";
                    }
                    $i = 1;
                    ?>
                    <?php
                    foreach ($form_fields_markup as $field_markup) {
                        echo $field_markup;
                        // Close grouping
                        if ($i == $group_limit) {
                            $i = 1; // reset
                            echo "</div><div class='$group_limit_textual tbk-fields'>";
                        } else {
                            $i++;
                        }
                    }
                    if ($number_of_fields > 3) {
                        echo '</div>';
                    }
                    ?>
                </form>
            </div>
            <!-- modal footer -->
            <div class="tbk-reservation-form-footer">
                <!-- tickets and price section -->
                <?php if ($this->service->isClass('event') || $this->service->getPrice() > 0) {
                    $discounted_price = Functions\getDiscountedPrice($this->service);
                    ?>
                    <div class="tbk-tickets-price-section"
                         data-currency-format="<?= Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), 0, 0, TRUE) ?>"
                         data-currency-symbol="<?= htmlentities(Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), NULL)) ?>">
                        <?php
                        // Tickets number
                        if ($this->service->isClass('event')) { ?>
                            <span class="tbk-tickets-span" style="vertical-align: middle">
                                <?= esc_html($string_total_tickets) ?>
                                <?php
                                if ($this->service->getMaxReservationsPerUser() <= 1) {
                                    echo '1';
                                } else {
                                    if ($this->tickets_left < $this->service->getMaxReservationsPerUser()) {
                                        $i_max = $this->tickets_left;
                                    } else {
                                        $i_max = $this->service->getMaxReservationsPerUser();
                                    }
                                    ?>
                                    <input class="tbk-ticket-value" required type="number" min="1" value="1"
                                           max="<?= $i_max ?>"
                                           pattern="[0-9]"
                                           step="1" data-price-num="<?= $this->service->getPrice() ?>"
                                           data-price-disc-num="<?= $discounted_price ?>"
                                           style="width: 65px;padding: 6px;font-weight: 400;"/>
                                <?php } ?>
                            </span>
                        <?php } else {
                            ?>
                            <input class="tbk-ticket-value" type="hidden" value="1"
                                   data-price-num="<?= $this->service->getPrice() ?>"
                                   data-price-disc-num="<?= $discounted_price ?>"/>
                            <?php
                        } ?>
                        <?php if ($this->service->getPrice() > 0) {
                            if ($this->service->getPrice() != $discounted_price) {
                                $price_string = Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), NULL)
                                    . '<del>' . Functions\priceFormat($this->service->getPrice()) . '</del><span class="tbk-discounted-price"> '
                                    . Functions\priceFormat($discounted_price) . '</span>';
                            } else {
                                $price_string = Functions\currencyCodeToSymbol($this->settings->getCurrencyCode(), $this->service->getPrice());
                            }
                            ?>
                            <div style="
                                     font-weight: 300;
                                     font-style: italic;
                                     display: inline-block;
                                     ">
                                <div class="tbk-total-price-line"
                                     style="vertical-align: middle;display: inline-block;">
                                    <span class="tbk-total-price-line-string" style="font-style: italic;padding: 5px;">
                                      <?= esc_html($string_total_price) ?>
                                    </span>
                                    <span class="tbk-total-price-line-price"
                                          data-base="<?= htmlentities($price_string) ?>">
                                      <?= $price_string ?>
                                    </span>
                                </div>
                            </div>
                        <?php } ?>
                    </div>
                <?php } ?>
                <div class="tbk-book-now">
                    <!-- confirm button -->
                    <button class="tbk-book-now-button" type="submit" data-files="<?= $there_are_files ? 1 : 0 ?>">
                        <?= count($this->settings->getPaymentGatewaysActive()) == 1
                        && $this->service->getPaymentMustBeDone() === 'immediately'
                        && $this->service->getPrice() > 0
                            ? esc_html($string_book_and_pay)
                            : esc_html($string_book_now) ?>
                        <?= isset($customer_reservations_left) ? '<span class="tbk-services-left">(' . $customer_reservations_left . ' ' . esc_html__('left', 'team-booking') . ')</span>' : '' ?>
                    </button>
                </div>
            </div>
        </div>
        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    public function getContentRegisterAdvice()
    {
        $registration_url = $this->settings->getRegistrationUrl();
        ob_start();
        ?>
        <?php if ($this->service) echo '<div style="text-align:center;">' ?>
        <div style="margin:10px 0 20px 0">
            <?= esc_html__('You must be logged-in to book this!', 'team-booking') ?>
        </div>
        <div>
            <a href="<?= esc_url($registration_url) ?>"
               class="tbk-button tbk-red"><?= esc_html__("I'm not registered...", 'team-booking') ?></a>
        </div>
        <div>
            <a href="<?= wp_login_url(get_permalink()) ?>"
               class="tbk-button tbk-green"><?= esc_html__('Login', 'team-booking') ?></a>
        </div>
        <?php
        if (!$this->service) { ?>
            <div>
                <div class="tbk-button tbk-dimmer-off" tabindex="0">
                    <?= esc_html__('Go back', 'team-booking') ?>
                </div>
            </div>
        <?php } ?>
        <?php if ($this->service) echo '</div>' ?>
        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    public function getContentReservationsFinished()
    {
        ob_start();
        ?>
        <div style="margin:10px 0 20px 0;text-align: center;">
            <div class="tbk-reservation-limit-service-name"><?= $this->service->getName() ?></div>
            <?= esc_html__("You've reached the limit for that service!", 'team-booking') ?>
        </div>
        <?php
        return ob_get_clean();
    }

    /**
     * @param \TeamBooking\Frontend\Form $form
     */
    public function form_header($form)
    {
        echo Components\FormHeader::get($form->service->getName(), $form->start_end_times, $form->coworker_string);
    }

    /**
     * @param \TeamBooking\Frontend\Form $form
     */
    public function form_map($form)
    {
        Components\FormMap::render($form->service_location);
    }

    /**
     * @param \TeamBooking\Frontend\Form $form
     */
    public function form_description($form)
    {
        if ($form->service->getServiceInfo()) Components\FormServiceDescription::render($form->service->getServiceInfo());
    }

}
