<?php

namespace TeamBooking;
defined('ABSPATH') or die('No script kiddies please!');

class Loader
{
    /**
     * The main load method.
     *
     * It should be called from the plugin main file in order to init the plugin
     */
    public function load()
    {

        // load text domain for translations
        add_action('init', array(
            $this,
            'load_textdomain',
        ));

        // add oAuth handler
        add_action('wp_ajax_teambooking_oauth_callback', 'teambooking_oauth_callback');

        // add IPN listener
        add_action('wp_ajax_teambooking_ipn_listener', 'teambooking_ipn_listener');
        add_action('wp_ajax_nopriv_teambooking_ipn_listener', 'teambooking_ipn_listener');

        // add TinyMCE vars getter
        add_action('wp_ajax_teambooking_get_tinymce_vars', 'teambooking_get_tinymce_vars');

        // add e-mail reminders handler
        add_action('tb_email_reminder_handler', 'TeamBooking\\Functions\\tbSendEmailReminder');

        // add REST API handler
        add_action('wp_ajax_teambooking_rest_api', 'teambooking_rest_api');
        add_action('wp_ajax_nopriv_teambooking_rest_api', 'teambooking_rest_api');

        // register the widget
        add_action('widgets_init', function () {
            register_widget('TeamBooking_Widget');
        });

        // load fonts
        add_action('wp_enqueue_scripts', array(
            $this,
            'load_fonts',
        ));

        /*
         * load admin classes and resources
         */
        if (is_admin() && (!defined('DOING_AJAX') || !DOING_AJAX)) {

            // activation hook
            register_activation_hook(TEAMBOOKING_FILE_PATH, array(
                $this,
                'install',
            ));
            // deactivation hook
            register_deactivation_hook(TEAMBOOKING_FILE_PATH, array(
                $this,
                'deactivate',
            ));
            // uninstall hook ($this can't work, TeamBooking_Loader class must be called directly)
            register_uninstall_hook(TEAMBOOKING_FILE_PATH, array(
                'TeamBooking_Loader',
                'uninstall',
            ));

            /*
             * Check settings integrity
             */
            if (!Functions\getSettings() instanceof \TeamBookingSettings) {
                add_action('init', function () {
                    deactivate_plugins(plugin_basename(TEAMBOOKING_FILE_PATH));
                    wp_redirect(admin_url('plugins.php'));
                });
            } else {
                /*
                 * TGM Plugin Activation loader
                 */
                include_once dirname(TEAMBOOKING_FILE_PATH) . '/libs/tgm/class-tgm-plugin-activation.php';
                add_action('tgmpa_register', array(
                    $this,
                    'requiredPlugins',
                ));

                /*
                 * Update method
                 */
                if (version_compare(Functions\getSettings()->getVersion(), TEAMBOOKING_VERSION, '<')) {
                    add_action('init', array(
                        'TeamBooking\\Update',
                        'update',
                    ));
                }
            }

            // load admin class
            new Admin();
        } else {

            // hook the shortcode register method
            add_action('init', array(
                $this,
                'register_shortcodes',
            ));

            // enqueue resources
            add_action('wp_enqueue_scripts', array(
                $this,
                'tb_frontend_resources_enqueue',
            ));

            // Reservation form processing actions
            add_action(
                'wp_ajax_tb_submit_payment',
                array(
                    'TeamBooking\\ProcessReservation',
                    'submitPayment',
                ), 10, 0
            );
            add_action(
                'wp_ajax_nopriv_tb_submit_payment',
                array(
                    'TeamBooking\\ProcessReservation',
                    'submitPayment',
                ), 10, 0
            );
            add_action(
                'wp_ajax_tbajax_action_submit_form',
                array(
                    'TeamBooking\\ProcessReservation',
                    'submitReservation',
                )
            );
            add_action(
                'wp_ajax_nopriv_tbajax_action_submit_form',
                array(
                    'TeamBooking\\ProcessReservation',
                    'submitReservation',
                )
            );
            add_action(
                'wp_ajax_tbajax_action_prepare_form',
                array(
                    'TeamBooking\\ProcessReservation',
                    'prepareReservation',
                )
            );
            add_action(
                'wp_ajax_nopriv_tbajax_action_prepare_form',
                array(
                    'TeamBooking\\ProcessReservation',
                    'prepareReservation',
                )
            );
            add_action(
                'wp_ajax_tb_process_onsite_payment',
                array(
                    'TeamBooking\\ProcessReservation',
                    'processOnsite',
                )
            );
            add_action(
                'wp_ajax_nopriv_tb_process_onsite_payment',
                array(
                    'TeamBooking\\ProcessReservation',
                    'processOnsite',
                )
            );
            // ICAL file
            add_action(
                'admin_post_tb_get_ical',
                array(
                    'TeamBooking\\ProcessReservation',
                    'getIcalFile',
                )
            );
            add_action(
                'admin_post_nopriv_tb_get_ical',
                array(
                    'TeamBooking\\ProcessReservation',
                    'getIcalFile',
                )
            );

            // register Ajax Callbacks
            add_action('wp_ajax_tbajax_action_change_month', 'tbajax_action_change_month_callback');
            add_action('wp_ajax_tbajax_action_show_day_schedule', 'tbajax_action_show_day_schedule_callback');
            add_action('wp_ajax_tbajax_action_filter_calendar', 'tbajax_action_filter_calendar_callback');
            add_action('wp_ajax_tbajax_action_get_reservation_modal', 'tbajax_action_get_reservation_modal_callback');
            add_action('wp_ajax_tbajax_action_get_register_modal', 'tbajax_action_get_register_modal_callback');
            add_action('wp_ajax_tbajax_action_fast_month_selector', 'tbajax_action_fast_month_selector_callback');
            add_action('wp_ajax_tbajax_action_fast_year_selector', 'tbajax_action_fast_year_selector_callback');
            add_action('wp_ajax_tbajax_action_cancel_reservation', 'tbajax_action_cancel_reservation_callback');
            add_action('wp_ajax_tbajax_action_validate_coupon', 'tbajax_action_validate_coupon_callback');

            add_action('wp_ajax_nopriv_tbajax_action_change_month', 'tbajax_action_change_month_callback');
            add_action('wp_ajax_nopriv_tbajax_action_show_day_schedule', 'tbajax_action_show_day_schedule_callback');
            add_action('wp_ajax_nopriv_tbajax_action_filter_calendar', 'tbajax_action_filter_calendar_callback');
            add_action('wp_ajax_nopriv_tbajax_action_get_reservation_modal', 'tbajax_action_get_reservation_modal_callback');
            add_action('wp_ajax_nopriv_tbajax_action_get_register_modal', 'tbajax_action_get_register_modal_callback');
            add_action('wp_ajax_nopriv_tbajax_action_fast_month_selector', 'tbajax_action_fast_month_selector_callback');
            add_action('wp_ajax_nopriv_tbajax_action_fast_year_selector', 'tbajax_action_fast_year_selector_callback');
            add_action('wp_ajax_nopriv_tbajax_action_cancel_reservation', 'tbajax_action_cancel_reservation_callback');
            add_action('wp_ajax_nopriv_tbajax_action_validate_coupon', 'tbajax_action_validate_coupon_callback');
        }
    }

    //------------------------------------------------------------

    /**
     * Load fonts
     */
    public static function load_fonts()
    {
        wp_register_style('teambooking_fonts', '//fonts.googleapis.com/css?family=Oswald|Open+Sans:300italic,400,300,700|Josefin+Sans:400,700,300,300italic', array(), '1.0.0');
    }

    //------------------------------------------------------------

    /**
     * Load frontend resources
     */
    public function tb_frontend_resources_enqueue()
    {
        ////////////////////
        //      CSS       //
        ////////////////////
        if (Functions\getSettings()->getFix62dot5()) {
            wp_register_style('semantic-style', TEAMBOOKING_URL . 'libs/semantic/semantic-fix-min.css');
        } else {
            wp_register_style('semantic-style', TEAMBOOKING_URL . 'libs/semantic/semantic-min.css');
        }
        wp_register_style('teambooking-style-modal', TEAMBOOKING_URL . 'libs/remodal/remodal.css', array(), filemtime(TEAMBOOKING_PATH . 'libs/remodal/remodal.css'));
        wp_register_style('teambooking-style-modal-theme', TEAMBOOKING_URL . 'libs/remodal/remodal-default-theme.css', array(), filemtime(TEAMBOOKING_PATH . 'libs/remodal/remodal-default-theme.css'));
        wp_register_style('teambooking-style-frontend', TEAMBOOKING_URL . 'css/frontend.css', array(), filemtime(TEAMBOOKING_PATH . 'css/frontend.css'));

        //////////////////
        //  Dashicons   //
        //////////////////
        wp_enqueue_style('dashicons');

        //////////////
        // scripts  //
        //////////////
        wp_register_script('tb-base64-decoder', TEAMBOOKING_URL . 'libs/base64/base64decode.js', array('jquery'), filemtime(TEAMBOOKING_PATH . 'libs/base64/base64decode.js'), TRUE);
        wp_register_script('tb-modal-script', TEAMBOOKING_URL . 'libs/remodal/remodal.js', array('jquery'), filemtime(TEAMBOOKING_PATH . 'libs/remodal/remodal.js'), TRUE);
        wp_register_script('tb-frontend-script', TEAMBOOKING_URL . 'js/frontend.js', array('jquery'), filemtime(TEAMBOOKING_PATH . 'js/frontend.js'), TRUE);
        // in javascript, object properties are accessed as ajax_object.some_value
        wp_localize_script('tb-frontend-script', 'TB_Ajax', array('ajax_url' => admin_url('admin-ajax.php')));

        ////////////////////
        // Google Places  //
        ////////////////////
        if (!Functions\getSettings()->getGmapsApiKey()) {
            wp_register_script('google-places-script', 'https://maps.googleapis.com/maps/api/js?v=3.exp&libraries=places', array('jquery'));
        } else {
            wp_register_script('google-places-script', 'https://maps.googleapis.com/maps/api/js?v=3.exp&libraries=places&key=' . Functions\getSettings()->getGmapsApiKey(), array('jquery'));
        }
        wp_register_script('tb-geocomplete-script', TEAMBOOKING_URL . 'js/assets/jquery.geocomplete.min.js', array('jquery'), filemtime(TEAMBOOKING_PATH . 'js/assets/jquery.geocomplete.min.js'), TRUE);

        ////////////
        // GMap3  //
        ////////////
        wp_register_script('tb-gmap3-script', TEAMBOOKING_URL . 'libs/gmap3/gmap3.min.js', array('jquery'), filemtime(TEAMBOOKING_PATH . 'libs/gmap3/gmap3.min.js'), TRUE);

        /////////////////
        //   Stripe    //
        /////////////////
        if (Functions\getSettings()->getPaymentGatewaySettingObject('stripe') instanceof \TeamBooking_PaymentGateways_Stripe_Settings) {
            if (Functions\getSettings()->getPaymentGatewaySettingObject('stripe')->isActive()) {
                wp_register_script('tb-jquery-payment', TEAMBOOKING_URL . 'libs/jquery-payment/jquery.payment.min.js', array('jquery'), filemtime(TEAMBOOKING_PATH . 'libs/jquery-payment/jquery.payment.min.js'), TRUE);
                if (Functions\getSettings()->getPaymentGatewaySettingObject('stripe')->isLoadLibrary()) {
                    wp_register_script('stripejs', 'https://js.stripe.com/v2/');
                }
            }
        }

        $this->find_shortcodes();
        add_action('the_post', array($this, 'find_shortcodes'));
    }

    public function find_shortcodes()
    {
        if (defined('TEAMBOOKING_SHORTCODE_FOUND')) {
            return;
        }

        global $post;
        $post_meta = get_post_meta($post->ID);
        $post_content = $post->post_content;

        if (is_array($post_meta)) {
            $post_meta = json_encode($post_meta);
            $post_content .= $post_meta;
        }
        if (has_shortcode($post_content, 'tb-calendar') || has_shortcode($post_content, 'tb-reservations')) {
            define('TEAMBOOKING_SHORTCODE_FOUND', TRUE);
            if (!Functions\getSettings()->getSkipGmapLibs()) {
                wp_enqueue_script('google-places-script');
            }

            if (Functions\getSettings()->getPaymentGatewaySettingObject('stripe')->isActive()) {
                wp_enqueue_script('tb-jquery-payment');
                if (Functions\getSettings()->getPaymentGatewaySettingObject('stripe')->isLoadLibrary()) {
                    wp_enqueue_script('stripejs');
                }
            }

            wp_enqueue_script('tb-base64-decoder');
            wp_enqueue_script('tb-geocomplete-script');
            wp_enqueue_script('tb-gmap3-script');
            wp_enqueue_script('tb-frontend-script');
            wp_enqueue_style('teambooking_fonts');
            wp_enqueue_style('semantic-style');
            wp_enqueue_style('teambooking-style-frontend');

            if (has_shortcode($post_content, 'tb-reservations')) {
                wp_enqueue_script('tb-modal-script');
                wp_enqueue_style('teambooking-style-modal');
                wp_enqueue_style('teambooking-style-modal-theme');
            }

            if (has_shortcode($post_content, 'tb-calendar')) {
                wp_enqueue_script('semantic-script');
            }
        }
    }

    //------------------------------------------------------------

    /**
     * The plugin Install method
     * Creates an instance of the install-class and fires the installation method
     * It is used only during the activation of the plugin
     */
    public function install($networkwide)
    {
        $install = new \TeamBooking_Install();
        if (function_exists('is_multisite') && is_multisite() && $networkwide) {
            global $wpdb;
            $blogs = $wpdb->get_results("
                    SELECT blog_id
                    FROM {$wpdb->blogs}
                    WHERE site_id = '{$wpdb->siteid}'
                    AND spam = '0'
                    AND deleted = '0'
                    AND archived = '0'
                ");
            foreach ($blogs as $blog_id) {
                switch_to_blog($blog_id);
                $install->install();
                restore_current_blog();
            }
            $return = TRUE;
        } else {
            $return = $install->install();
        }

        return $return;
    }

    /**
     * The plugin Deactivate method
     * Creates an instance of the install-class and fires the deactivation method
     * It is used only during the deactivation of the plugin
     */
    public function deactivate($networkwide)
    {
        $install = new \TeamBooking_Install();
        if (function_exists('is_multisite') && is_multisite() && $networkwide) {
            global $wpdb;
            $blogs = $wpdb->get_results("
                    SELECT blog_id
                    FROM {$wpdb->blogs}
                    WHERE site_id = '{$wpdb->siteid}'
                    AND spam = '0'
                    AND deleted = '0'
                    AND archived = '0'
                ");
            foreach ($blogs as $blog_id) {
                switch_to_blog($blog_id);
                $install->deactivate();
                restore_current_blog();
            }
        } else {
            $install->deactivate();
        }
    }

    /**
     * The plugin Uninstall method
     * Creates an instance of the install-class and fires the unistallation method
     * It is used only during the uninstallation of the plugin
     */
    public static function uninstall($networkwide)
    {
        $install = new \TeamBooking_Install();
        if (function_exists('is_multisite') && is_multisite() && $networkwide) {
            global $wpdb;
            $blogs = $wpdb->get_results("
                    SELECT blog_id
                    FROM {$wpdb->blogs}
                    WHERE site_id = '{$wpdb->siteid}'
                    AND spam = '0'
                    AND deleted = '0'
                    AND archived = '0'
                ");
            foreach ($blogs as $blog_id) {
                switch_to_blog($blog_id);
                $install->uninstall();
                restore_current_blog();
            }
        } else {
            $install->uninstall();
        }
    }

    //------------------------------------------------------------

    /**
     * Load the text domain on plugin load.
     *
     * Hooked to the plugins_loaded via the load method
     */
    public function load_textdomain()
    {
        $domain = 'team-booking';
        $locale = apply_filters('plugin_locale', get_locale(), $domain);
        load_textdomain($domain, WP_LANG_DIR . '/team-booking/' . $locale . '.mo');
        load_plugin_textdomain($domain, FALSE, plugin_basename(TEAMBOOKING_PATH) . '/languages/');
    }

    //------------------------------------------------------------

    public function requiredPlugins()
    {
        /*
         * Array of plugin arrays. Required keys are name and slug.
         * If the source is NOT from the .org repo, then source is also required.
         */
        $plugins = array(
            array(
                'name'         => 'Envato Market',
                // The plugin name.
                'slug'         => 'envato-market',
                // The plugin slug (typically the folder name).
                'source'       => 'http://envato.github.io/wp-envato-market/dist/envato-market.zip',
                // The plugin source.
                'required'     => FALSE,
                // If false, the plugin is only 'recommended' instead of required.
                'external_url' => '',
            ),
        );

        $config = array(
            'id'           => 'tgmpa',
            // Unique ID for hashing notices for multiple instances of TGMPA.
            'default_path' => '',
            // Default absolute path to bundled plugins.
            'menu'         => 'tgmpa-install-plugins',
            // Menu slug.
            'parent_slug'  => 'plugins.php',
            // Parent menu slug.
            'capability'   => 'manage_options',
            // Capability needed to view plugin install page, should be a capability associated with the parent menu used.
            'has_notices'  => TRUE,
            // Show admin notices or not.
            'dismissable'  => TRUE,
            // If false, a user cannot dismiss the nag message.
            'dismiss_msg'  => '',
            // If 'dismissable' is false, this message will be output at top of nag.
            'is_automatic' => FALSE,
            // Automatically activate plugins after installation or not.
            'message'      => '',
            // Message to output right before the plugins table.
            'strings'      => array(
                'page_title'                     => __('Install Recommended Plugins', 'team-booking'),
                'menu_title'                     => __('Install Plugins', 'team-booking'),
                'installing'                     => __('Installing Plugin: %s', 'team-booking'),
                // %s = plugin name.
                'notice_can_install_recommended' => _n_noop(
                    'TeamBooking recommends the following plugin: %1$s.',
                    'TeamBooking recommends the following plugins: %1$s.',
                    'team-booking'
                ),
                // %1$s = plugin name(s).
                'notice_ask_to_update'           => _n_noop(
                    'The following plugin needs to be updated to its latest version to ensure maximum compatibility with TeamBooking: %1$s.',
                    'The following plugins need to be updated to their latest version to ensure maximum compatibility with TeamBooking: %1$s.',
                    'team-booking'
                ),
                // %1$s = plugin name(s).
                'notice_ask_to_update_maybe'     => _n_noop(
                    'There is an update available for: %1$s.',
                    'There are updates available for the following plugins: %1$s.',
                    'team-booking'
                ),
                // %1$s = plugin name(s).
            ),
        );

        tgmpa($plugins, $config);
    }

    //------------------------------------------------------------

    /**
     * Register Shortcodes handler
     */
    public function register_shortcodes()
    {
        // Main shortcode
        add_shortcode('tb-calendar', array(
            'TeamBooking\\Shortcodes\\Calendar',
            'render',
        ));

        // User reservation list
        add_shortcode('tb-reservations', array(
            'TeamBooking\\Shortcodes\\Reservations',
            'render',
        ));
    }
}
