<?php
// Blocks direct access to this file
defined('ABSPATH') or die("No script kiddies please!");

use TeamBooking\Functions,
    TeamBooking\Admin\Framework,
    TeamBooking\Database,
    TeamBooking\PaymentGateways;

/**
 * Option object for PayPal Gateway
 */
class TeamBooking_PaymentGateways_PayPal_Settings implements TeamBooking_PaymentGateways_Settings
{

    ///////////////
    //  GENERAL  //
    ///////////////
    private $use_gateway;
    private $gateway_id;
    /////////////////////////
    //  PAYPAL SPECIFIC    //
    /////////////////////////
    private $account_email;
    private $redirect_url;
    private $use_sandbox;
    private $save_ipn_logs;
    private $dump_ipn;
    private $use_curl;

    public function __construct()
    {
        /////////////////////////
        //  GENERAL (defaults) //
        /////////////////////////
        $this->gateway_id = 'paypal';
        $this->use_gateway = FALSE;
        ////////////////////////////////
        // PAYPAL SPECIFIC (defaults) //
        ////////////////////////////////
        $this->use_sandbox = FALSE;
        $this->save_ipn_logs = FALSE;
        $this->dump_ipn = FALSE;
        $this->redirect_url = site_url();
        if (extension_loaded('curl')) {
            $this->use_curl = TRUE;
        } else {
            $this->use_curl = FALSE;
        }
    }

    //------------------------------------------------------------

    public function isOffsite()
    {
        return TRUE;
    }

    //------------------------------------------------------------

    public function setUseGateway($bool)
    {
        $this->use_gateway = (bool)$bool;
    }

    public function isActive()
    {
        return $this->use_gateway;
    }

    //------------------------------------------------------------

    public function getGatewayId()
    {
        return $this->gateway_id;
    }

    //------------------------------------------------------------

    public function setAccountEmail($email)
    {
        $this->account_email = $email;
    }

    public function getAccountEmail()
    {
        return trim($this->account_email);
    }

    //------------------------------------------------------------

    public function setRedirectUrl($url)
    {
        $this->redirect_url = $url;
    }

    public function getRedirectUrl()
    {
        return $this->redirect_url;
    }

    //------------------------------------------------------------

    public function setUseCurl($boolean)
    {
        if ($boolean) {
            $this->use_curl = TRUE;
        } else {
            $this->use_curl = FALSE;
        }
    }

    public function getUseCurl()
    {
        if (extension_loaded('curl')) {
            if (is_null($this->use_curl)) {
                return TRUE;
            } else {
                return $this->use_curl;
            }
        } else {
            return FALSE;
        }
    }

    //------------------------------------------------------------

    public function setUseSandbox($boolean)
    {
        $this->use_sandbox = (bool)$boolean;
    }

    public function isUseSandbox()
    {
        return $this->use_sandbox;
    }

    //------------------------------------------------------------

    public function setSaveIpnLogs($boolean)
    {
        if ($boolean) {
            $this->save_ipn_logs = TRUE;
        } else {
            $this->save_ipn_logs = FALSE;
        }
    }

    public function isSaveIpnLogs()
    {
        return $this->save_ipn_logs;
    }

    public function setDumpIPN($boolean)
    {
        if ($boolean) {
            $this->dump_ipn = TRUE;
        } else {
            $this->dump_ipn = FALSE;
        }
    }

    public function getDumpIPN()
    {
        if (!$this->dump_ipn) {
            return FALSE;
        } else {
            return TRUE;
        }
    }

    //------------------------------------------------------------

    public function getLabel()
    {
        return __('via PayPal', 'team-booking');
    }

    //------------------------------------------------------------

    public function getPayButton()
    {
        ob_start();
        ?>
        <div class="tb-icon tbk-button tbk-blue tbk-pay-button" data-offsite="<?= $this->isOffsite() ?>"
             data-gateway="<?= $this->gateway_id ?>" type="submit" tabindex="0">
            <i class="paypal tb-icon"></i>

            <div class="tbk-content">
                <?= esc_html__('Pay with PayPal', 'team-booking') ?>
            </div>
        </div>

        <?php
        return ob_get_clean();
    }

    //------------------------------------------------------------

    public function getDataForm($data, $reservation_database_id)
    {
        return FALSE;
    }

    //------------------------------------------------------------

    public function prepareGateway(TeamBooking_ReservationData $data, $additional_parameter = NULL)
    {
        $paypal = new PaymentGateways\PayPal\Gateway();
        $paypal->setCurrency($data->getCurrencyCode());
        $paypal->setItemName($data->getServiceName());
        $paypal->setPrice($data->getPriceIncremented());
        $paypal->setQuantity($data->getTickets());
        $paypal->setIpnId($data->getIpnUniqueId());
        // should we override the redirect URL for this service?
        if (Functions\getSettings()->getService($data->getServiceId())->getRedirect()) {
            $paypal->setRedirectUrl(Functions\getSettings()->getService($data->getServiceId())->getRedirectUrl($data->getDatabaseId()));
        } else {
            $paypal->setRedirectUrl($this->getRedirectUrl());
        }

        return $paypal->processPayment();
    }

    //------------------------------------------------------------

    public function saveBackendSettings(array $settings, $new_currency_code)
    {
        if (isset($settings['use_gateway'])) {
            (!empty($settings['account_email']) && $this->verifyCurrency($new_currency_code)) ? $this->setUseGateway($settings['use_gateway']) : $this->setUseGateway(FALSE);
        }
        isset($settings['account_email']) ? $this->setAccountEmail(strtolower($settings['account_email'])) : $this->setAccountEmail('');
        isset($settings['redirect_url']) ? $this->setRedirectUrl($settings['redirect_url']) : $this->setRedirectUrl('');
        if (isset($settings['use_sandbox'])) {
            $this->setUseSandbox($settings['use_sandbox']);
        }
        isset($settings['save_ipn_logs']) ? $this->setSaveIpnLogs(TRUE) : $this->setSaveIpnLogs(FALSE);
        isset($settings['ipn_dump']) ? $this->setDumpIPN(TRUE) : $this->setDumpIPN(FALSE);
        if ($settings['use_curl'] === 'yes') {
            $this->setUseCurl(TRUE);
        } else {
            $this->setUseCurl(FALSE);
        }
    }

    //------------------------------------------------------------

    public function getBackendSettingsTab()
    {
        $panel = new Framework\PanelWithForm(NULL);
        $panel->setAction('option_submit');
        $panel->setNonce('team_booking_options_verify');
        $panel->addTitleContent(Framework\ElementFrom::content(Framework\Html::anchor(array(
            'text'   => Framework\Html::img(array('src' => 'https://www.paypalobjects.com/webstatic/i/logo/rebrand/ppcom.png', 'alt' => __('PayPal', 'team-booking'))),
            'escape' => FALSE,
            'target' => '_blank',
            'href'   => 'https://www.paypal.com'
        ))));

        // Use PayPal gateway
        $element = new Framework\PanelSettingYesOrNo(__('Use PayPal gateway', 'team-booking'));
        if (empty($this->account_email)) {
            $element->addDescription(__("You can't activate the PayPal gateway if the account e-mail field is empty.", 'team-booking'));
            $element->setDisabled(TRUE);
        }
        $element->addAlertDropcap(__('Note', 'team-booking'));
        if ($this->verifyCurrency(Functions\getSettings()->getCurrencyCode())) {
            $element->addNotice(__('be sure that your PayPal account actually holds the selected currency, otherwise your payments may require manual confirmation, depending on Payment Receiving Preferences in your PayPal profile.', 'team-booking'));
        } else {
            $element->addAlert(__("The selected currency is not supported by PayPal. PayPal gateway can't be activated.", 'team-booking'));
        }
        $element->addFieldname('gateway_settings[' . $this->gateway_id . '][use_gateway]');
        $element->setState($this->use_gateway);
        $element->appendTo($panel);

        // PayPal account email
        $element = new Framework\PanelSettingText(__('PayPal account email', 'team-booking'));
        $element->addFieldname('gateway_settings[' . $this->gateway_id . '][account_email]');
        $element->addDescription(__('Payments will be addressed to this account.', 'team-booking'));
        $element->addDefaultValue($this->account_email);
        $element->appendTo($panel);

        // Redirect URL
        $element = new Framework\PanelSettingText(__('Redirect URL', 'team-booking'));
        $element->addFieldname('gateway_settings[' . $this->gateway_id . '][redirect_url]');
        $element->addDescription(__('After PayPal payment, the customer will be redirected to this URL.', 'team-booking'));
        $element->addDefaultValue($this->redirect_url);
        $element->appendTo($panel);

        // IPN listener method
        $element = new Framework\PanelSettingRadios(__('IPN listener method', 'team-booking'));
        $element->addFieldname('gateway_settings[' . $this->gateway_id . '][use_curl]');
        $element->addDescription(__('cURL is default, if supported by your server. If you have issues like reservations kept in pending status, try to change method.', 'team-booking'));
        $element->addOption(array(
            'label'    => 'cURL' . (!extension_loaded('curl') ? ' (' . __('not supported', 'team-booking') . ')' : ''),
            'disabled' => (extension_loaded('curl')) ? FALSE : TRUE,
            'value'    => 'yes',
            'checked'  => $this->getUseCurl()
        ));
        $element->addOption(array(
            'label'   => 'fsockopen',
            'value'   => 'no',
            'checked' => !$this->getUseCurl()
        ));
        $element->appendTo($panel);

        // Use PayPal Sandbox
        $element = new Framework\PanelSettingYesOrNo(__('Use PayPal Sandbox for testing', 'team-booking'));
        $element->addFieldname('gateway_settings[' . $this->gateway_id . '][use_sandbox]');
        $element->addDescription(__('Fake payments. Activate this option only if you really know what the sandbox is.', 'team-booking'));
        $element->addAlertDropcap(__('Note', 'team-booking'));
        $element->addNotice(__("if you're testing the payments in a localhost environment, the PayPal IPN won't reach your computer, and the plugin will act as if payment was never made.", 'team-booking'));
        $element->setState($this->use_sandbox);
        $element->appendTo($panel);

        // Debugging IPN errors
        $element = new Framework\PanelSettingCheckboxes(__('Debugging IPN errors', 'team-booking'));
        $element->addCheckbox(array(
            'label'     => Framework\ElementFrom::content(
                esc_html__('Activate this option to save the IPN response error logs', 'team-booking') . ' (' . Framework\Html::anchor(array('id' => 'tb-paypal-ipn-log', 'text' => esc_html__('read', 'team-booking'))) . ')'
            ),
            'checked'   => $this->save_ipn_logs,
            'fieldname' => 'gateway_settings[' . $this->gateway_id . '][save_ipn_logs]',
            'escape'    => FALSE
        ));
        $element->addCheckbox(array(
            'label'     => Framework\ElementFrom::content(
                esc_html__('Activate this option to dump the last raw IPN response', 'team-booking') . ' (' . Framework\Html::anchor(array('id' => 'tb-paypal-ipn-dump', 'text' => esc_html__('read', 'team-booking'))) . ')'
            ),
            'checked'   => $this->dump_ipn,
            'fieldname' => 'gateway_settings[' . $this->gateway_id . '][ipn_dump]',
            'escape'    => FALSE
        ));
        $element->appendTo($panel);

        // Save changes
        $wildcard = new Framework\PanelSettingWildcard(NULL);
        $element = new Framework\PanelSaveButton(__('Save changes', 'team-booking'), 'team_booking_payment_options_submit');
        $wildcard->addContent($element);

        // IPN error log modal
        $element = new Framework\Modal('tb-paypal-ipn-log-modal');
        $element->setButtons(FALSE);
        $element->setHeaderText(array('main' => __('IPN response error logs', 'team-booking')));
        $element->addContent('<textarea style="width: 100%" rows="10" readonly="readonly">');
        if (file_exists(dirname(__FILE__) . '/ipn_errors.log')) {
            $element->addContent(file_get_contents(dirname(__FILE__) . '/ipn_errors.log'));
        }
        $element->addContent('</textarea>');
        $wildcard->addContent($element);

        // IPN last dump modal
        $element = new Framework\Modal('tb-paypal-ipn-dump-modal');
        $element->setButtons(FALSE);
        $element->setHeaderText(array('main' => __('Last IPN dump', 'team-booking')));
        $element->addContent('<textarea style="width: 100%" rows="10" readonly="readonly">');
        if (file_exists(dirname(__FILE__) . '/ipn_dump.log')) {
            $element->addContent(file_get_contents(dirname(__FILE__) . '/ipn_dump.log'));
        }
        $element->addContent('</textarea>');
        $wildcard->addContent($element);

        // Scripts
        ob_start();
        ?>
        <script>
            jQuery('#tb-paypal-ipn-log-modal')
                .uiModal('attach events', '#tb-paypal-ipn-log', 'show')
            ;
            jQuery('#tb-paypal-ipn-dump-modal')
                .uiModal('attach events', '#tb-paypal-ipn-dump', 'show')
            ;
            jQuery('#tb-paypal-ipn-log, #tb-paypal-ipn-dump').click(function (e) {
                e.preventDefault();
            });
        </script>
        <?php
        $element = Framework\ElementFrom::content(ob_get_clean());
        $wildcard->addContent($element);
        $panel->addElement($wildcard);

        return $panel;

    }

    //------------------------------------------------------------

    public function verifyCurrency($code)
    {
        $supported_currencies = array(
            'AUD' => array(
                'label'  => 'Australian Dollar',
                'format' => '$ %s',
            ),
            'BRL' => array(
                'label'  => 'Brazilian Real',
                'format' => 'R$ %s',
            ),
            'CAD' => array(
                'label'  => 'Canadian Dollar',
                'format' => '$ %s',
            ),
            'CZK' => array(
                'label'  => 'Czech Koruna',
                'format' => '%s Kč',
            ),
            'DKK' => array(
                'label'  => 'Danish Krone',
                'format' => '%s kr',
            ),
            'EUR' => array(
                'label'  => 'Euro',
                'format' => '€ %s',
            ),
            'HKD' => array(
                'label'  => 'Hong Kong Dollar',
                'format' => '$ %s',
            ),
            'HUF' => array(
                'label'  => 'Hungarian Forint',
                'format' => '%s Ft',
            ),
            'ILS' => array(
                'label'  => 'Israeli New Sheqel',
                'format' => '₪ %s',
            ),
            'JPY' => array(
                'label'  => 'Japanese Yen',
                'format' => '¥ %s',
            ),
            'MYR' => array(
                'label'  => 'Malaysian Ringgit',
                'format' => 'RM %s',
            ),
            'MXN' => array(
                'label'  => 'Mexican Peso',
                'format' => '$ %s',
            ),
            'NOK' => array(
                'label'  => 'Norwegian Krone',
                'format' => '%s kr',
            ),
            'NZD' => array(
                'label'  => 'N.Z. Dollar',
                'format' => '$ %s',
            ),
            'PHP' => array(
                'label'  => 'Philippine Peso',
                'format' => '₱ %s',
            ),
            'PLN' => array(
                'label'  => 'Polish Zloty',
                'format' => '%s zł',
            ),
            'GBP' => array(
                'label'  => 'Pound Sterling',
                'format' => '£ %s',
            ),
            'SGD' => array(
                'label'  => 'Singapore Dollar',
                'format' => '$ %s',
            ),
            'SEK' => array(
                'label'  => 'Swedish Krona',
                'format' => '%s kr',
            ),
            'CHF' => array(
                'label'  => 'Swiss Franc',
                'format' => '%s Fr',
            ),
            'TWD' => array(
                'label'  => 'New Taiwan Dollar',
                'format' => 'NT$ %s',
            ),
            'THB' => array(
                'label'  => 'Thai Baht',
                'format' => '฿ %s',
            ),
            'TRY' => array(
                'label'  => 'Turkish Lira',
                'format' => 'TRY %s',
            ),
            'USD' => array(
                'label'  => 'U.S. Dollar',
                'format' => '$ %s',
            ),
        );
        if (in_array($code, array_keys($supported_currencies))) {
            return TRUE;
        } else {
            return FALSE;
        }
    }

    //------------------------------------------------------------

    public function listenerIPN($ipn_data)
    {
        // Log IPN errors
        if ($this->isSaveIpnLogs()) {
            ini_set('log_errors', TRUE);
            ini_set('error_log', dirname(__FILE__) . '/ipn_errors.log');
        }
        // instantiate the IPN listener
        include_once dirname(__FILE__) . '/lib/tb_ipnlistener.php';
        $listener = new IpnListener();

        // tell the IPN listener whether to use the PayPal test sandbox or not
        if ($this->isUseSandbox()) {
            $listener->use_sandbox = TRUE;
        } else {
            $listener->use_sandbox = FALSE;
        }

        // tell the IPN listener whether to use cURL or fsockopen
        if ($this->getUseCurl()) {
            $listener->use_curl = TRUE;
        } else {
            $listener->use_curl = FALSE;
        }

        // try to process the IPN POST
        try {
            $listener->requirePostMethod();
            $verified = $listener->processIpn($ipn_data);
        } catch (Exception $e) {
            // Log IPN errors
            if ($this->isSaveIpnLogs()) {
                error_log($e->getMessage());
            }
            // Dump IPN
            if ($this->getDumpIPN()) {
                ob_start();
                var_dump($ipn_data);
                $output = ob_get_clean();
                $outputFile = dirname(__FILE__) . '/ipn_dump.log';
                $filehandle = fopen($outputFile, 'a') or die();
                fwrite($filehandle, $output);
                fclose($filehandle);
            }
            exit;
        }

        if ($verified) {
            if (isset($ipn_data['charset'])) {
                $charset = $ipn_data['charset'];
                // If not UTF-8, convert all the values
                if (!$charset == 'utf-8') {
                    foreach ($ipn_data as $key => &$value) {
                        $value = mb_convert_encoding($value, 'utf-8', $charset);
                    }
                }
                // Store the charset values for future implementation
                $ipn_data['charset'] = 'utf-8';
                $ipn_data['charset_original'] = $charset;
            }

            $reservations = Database\Reservations::getAll();
            $reservation = FALSE;
            foreach ($reservations as $reservation_id => $res) {
                /* @var $res TeamBooking_ReservationData */
                if ($res->getIpnUniqueId() == $ipn_data['custom']) {
                    $reservation = $res;
                    $reservation_id = $reservation_id;
                    break;
                }
            }

            $errmsg = '';   // stores errors from fraud checks

            if (!$reservation) { // Reservation is not found!
                $errmsg .= "'reservation_unique_id' not found: ";
                $errmsg .= $ipn_data['custom'] . "\n";
                $price = 0;
            } else {
                if ($reservation->getServiceClass() == 'event') {
                    $price = number_format($reservation->getPriceIncremented() * $reservation->getTickets(), 2, '.', '');
                } else {
                    $price = number_format($reservation->getPriceIncremented(), 2, '.', '');
                }
            }

            // 1. Make sure the payment status is "Completed" 
            if ($ipn_data['payment_status'] != 'Completed') {
                if ($ipn_data['payment_status'] == 'Pending' && $ipn_data['pending_reason'] == 'multi_currency') {
                    // Warning: the PayPal account
                    // settings requires manual confirmation of payments
                    // with currency that merchant don't holds
                    $body = "WARNING: You are asking for payments in a currency that your PayPal account doesn't hold! Please either change the currency, or change your PayPal -> Profile -> Payment receiving preferences for 'Allow payments sent to me in a currency I do not hold' to 'Yes, accept and convert them'.\n\n";
                    $body .= $listener->getTextReport();
                    wp_mail(get_bloginfo('admin_email'), 'Currency not held by your PayPal account', $body);
                    $multi_currency = TRUE;
                    unset($body);
                } else {
                    exit;
                }
            }
            // 2. Make sure seller email matches your primary account email.
            if ($ipn_data['receiver_email'] != $this->getAccountEmail()) {
                $errmsg .= "'receiver_email' does not match: ";
                $errmsg .= $ipn_data['receiver_email'] . "\n";
                $errmsg .= $this->getAccountEmail() . "\n";
            }
            // 3. Make sure the amount(s) paid match
            $paid_amount = $ipn_data['mc_gross'];
            if (isset($ipn_data['tax']) && is_numeric($ipn_data['tax'])) {
                $taxes = $ipn_data['tax'];
            } else {
                $taxes = 0.00;
            }
            if (($paid_amount - $taxes) != $price) {
                $errmsg .= "'mc_gross' does not match: ";
                $errmsg .= $paid_amount . "\n";
            }
            // 4. Make sure the currency code matches
            if ($ipn_data['mc_currency'] != Functions\getSettings()->getCurrencyCode()) {
                $errmsg .= "'mc_currency' does not match: ";
                $errmsg .= $ipn_data['mc_currency'] . "\n";
            }
            // 5. Ensure the transaction is not a duplicate.
            $txn_id = $ipn_data['txn_id'];
            $exists = FALSE;
            foreach ($reservations as $res) {
                /* @var $res TeamBooking_ReservationData */
                if ($res->getPaymentGateway() == 'paypal') {
                    $payment_details = $res->getPaymentDetails();
                    if ($payment_details['transaction id'] == $txn_id) {
                        $exists = TRUE;
                    }
                }
            }
            if ($exists) {
                $errmsg .= "'txn_id' has already been processed: " . $txn_id . "\n";
            }

            if (!empty($errmsg)) {
                // manually investigate errors from the fraud checking
                $body = "IPN failed fraud checks: \n$errmsg\n\n";
                $body .= $listener->getTextReport();
                wp_mail(get_bloginfo('admin_email'), 'IPN Fraud Warning', $body);
            } else {
                if ($multi_currency) {
                    $reservation->setPaymentMustBeManuallyConfirmedOnPayPal(TRUE);
                } else {
                    $reservation->setPaid(TRUE);
                }
                $reservation->setPaymentGateway($this->gateway_id);
                $payment_details_array = array();
                $payment_details_array['transaction id'] = $txn_id;
                $payment_details_array['paid amount'] = $paid_amount;
                $payment_details_array['payer email'] = $ipn_data['payer_email'];
                $reservation->setPaymentDetails($payment_details_array);
                if (!$reservation->isPending()) {
                    // Reservation was not pending, updating the placed one
                    Database\Reservations::update($reservation);
                } else {
                    // Reservation was pending, do the reservation...
                    $var = new TeamBooking_Reservation($reservation);
                    $attempted = $var->doReservation();
                    // Check for errors
                    if ($attempted instanceof TeamBooking_Error) {
                        // At this point, payment is made but
                        // the reservation attempt throws errors.
                        $reservation->setStatusPending();
                        $errmsg = $attempted->getMessage();
                        $reservation->setPendingReason($errmsg);
                        Database\Reservations::update($reservation);
                        $body = "Error message: \n$errmsg\n\n";
                        $body .= $listener->getTextReport();
                        wp_mail(get_bloginfo('admin_email'), 'Reservation error after payment is done.', $body);
                    } else {
                        // No errors
                        $reservation->setStatusConfirmed();
                        Database\Reservations::update($reservation);
                        // Send e-mail messages
                        if ($var->getServiceObj()->getNotify()) {
                            $var->sendNotificationEmail();
                        }
                        if (Functions\getSettings()->getCoworkerData($reservation->getCoworker())->getCustomEventSettings($reservation->getServiceId())->getGetDetailsByEmail()) {
                            $var->sendNotificationEmailToCoworker();
                        }
                        if ($var->getServiceObj()->getConfirm() && $reservation->getCustomerEmail()) {
                            $var->sendConfirmationEmail();
                        }
                    }
                }
            }
            if ($this->getDumpIPN()) {
                ob_start();
                var_dump($errmsg);
                echo '<br>';
                var_dump($ipn_data);
                $output = ob_get_clean();
                $outputFile = dirname(__FILE__) . '/ipn_dump.log';
                $filehandle = fopen($outputFile, 'w') or die();
                fwrite($filehandle, $output);
                fclose($filehandle);
            }
            exit;
        } else {
            // manually investigate the invalid IPN
            wp_mail(get_bloginfo('admin_email'), 'Invalid IPN', $listener->getTextReport());
            exit;
        }
    }

}
