<?php

namespace TeamBooking\PaymentGateways\Stripe;

// Blocks direct access to this file
defined('ABSPATH') or die("No script kiddies please!");

use TeamBooking\Functions;

class Gateway implements \TeamBooking\PaymentGateways\Gateway
{
    private $settings;
    private $api_key;
    private $token;
    private $currency;
    private $quantity;
    private $price;
    private $item_name;
    private $receipt_email;

    public function __construct()
    {
        $this->settings = Functions\getSettings();
    }

    //------------------------------------------------------------

    public function setApiKey($key)
    {
        $this->api_key = $key;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function setCurrency($code)
    {
        $this->currency = $code;
    }

    public function setQuantity($quantity)
    {
        $this->quantity = $quantity;
    }

    public function setPrice($price)
    {
        $this->price = $price;
    }

    public function setItemName($name)
    {
        $this->item_name = $name;
    }

    public function setReceiptEmail($email)
    {
        $this->receipt_email = $email;
    }

    //------------------------------------------------------------

    public function processPayment()
    {
        require_once dirname(__FILE__) . '/lib/init.php';
        \Stripe\Stripe::setApiKey($this->api_key);
        $parameters = array(
            "amount"      => $this->generateAmount(),
            // amount in cents, again
            "currency"    => $this->currency,
            "source"      => $this->token,
            "description" => $this->item_name,
        );
        if (!is_null($this->receipt_email)) {
            $parameters['receipt_email'] = $this->receipt_email;
        }
        try {
            $charge = \Stripe\Charge::create($parameters);
            $charge_array = $charge->__toArray();
            $return_array['charge id'] = $charge_array['id'];
            $return_array['receipt number'] = $charge_array['receipt_number'];
            $return_array['invoice id'] = $charge_array['invoice'];

            return $return_array;
        } catch (\Stripe\Error\Card $e) {
            // The card has been declined
            $body = $e->getJsonBody();
            $code = $body['error']['code'];
            $message = $body['error']['message'];
        } catch (\Stripe\Error\RateLimit $e) {
            // Too many requests made to the API too quickly
            $body = $e->getJsonBody();
            $code = $body['error']['code'];
            $message = $body['error']['message'];
        } catch (\Stripe\Error\InvalidRequest $e) {
            // Invalid parameters were supplied to Stripe's API
            $body = $e->getJsonBody();
            $code = $body['error']['code'];
            $message = $body['error']['message'];
        } catch (\Stripe\Error\Authentication $e) {
            // Authentication with Stripe's API failed
            // (maybe you changed API keys recently)
            $body = $e->getJsonBody();
            $code = $body['error']['code'];
            $message = $body['error']['message'];
        } catch (\Stripe\Error\ApiConnection $e) {
            // Network communication with Stripe failed
            $body = $e->getJsonBody();
            $code = $body['error']['code'];
            $message = $body['error']['message'];
        } catch (\Stripe\Error\Base $e) {
            // Display a very generic error to the user
            $body = $e->getJsonBody();
            $code = $body['error']['code'];
            $message = $body['error']['message'];
        } catch (\Exception $e) {
            // Something else happened, completely unrelated to Stripe
            $code = $e->getCode();
            $message = $e->getMessage();
        }

        $error = new \TeamBooking_Error($code);
        $error->setMessage($message);

        return $error;
    }

    //------------------------------------------------------------

    public function generateAmount()
    {
        $currency_code = Functions\getSettings()->getCurrencyCode();
        $zero_decimal_currencies = array(
            'BIF' => 'Burundian Franc',
            'CLP' => 'Chilean Peso',
            'DJF' => 'Djiboutian Franc',
            'GNF' => 'Guinean Franc',
            'JPY' => 'Japanese Yen',
            'KMF' => 'Comorian Franc',
            'KRW' => 'South Korean Won',
            'MGA' => 'Malagasy Ariary',
            'PYG' => 'Paraguayan Guaraní',
            'RWF' => 'Rwandan Franc',
            'VND' => 'Vietnamese Đồng',
            'VUV' => 'Vanuatu Vatu',
            'XAF' => 'Central African Cfa Franc',
            'XOF' => 'West African Cfa Franc',
            'XPF' => 'Cfp Franc',
        );
        if (in_array($currency_code, array_keys($zero_decimal_currencies))) {
            return $this->price * $this->quantity;
        } else {
            return $this->price * $this->quantity * 100;
        }
    }

}
