<?php

defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Toolkit;

class TeamBooking_ReservationData
{
    //////////////////////////
    //   COWORKER DATA      //
    //////////////////////////
    private $coworker_id;
    //////////////////////////
    //   CUSTOMER DATA      //
    //////////////////////////
    private $customer_user_id; // WordPress user id, if registered
    private $customer_timezone;
    private $enumerable_for_customer_limit;
    //////////////////////////
    //    GOOGLE DATA       //
    //////////////////////////
    private $google_calendar_event_parent;
    private $google_calendar_event;
    private $google_calendar_id;
    private $hangout_link;
    private $event_html_link;
    //////////////////////////
    //    SERVICE DATA      //
    //////////////////////////
    private $service_id;
    private $service_name;
    private $service_class;
    private $service_location;
    //////////////////////////
    // COLLECTED FORM DATA  //
    //////////////////////////
    /* @var $form_fields TeamBooking_ReservationFormField[] */
    private $form_fields;
    //////////////////////////
    //  RESERVATION DATA    //
    //////////////////////////
    private $start; // unix time
    private $end;   // unix time
    private $slot_start; // ATOM time
    private $slot_end;   // ATOM time
    private $description = ''; //TODO
    private $uri = ''; //TODO
    private $tickets;
    private $price; // unitary
    private $price_discounted; // unitary
    private $discount; // array with id,name,value
    private $log_key;
    private $status;
    private $cancellation_reason;
    private $cancellation_who;
    private $confirmation_who;
    private $pending_reason;
    private $email_reminder_sent;
    private $database_id;
    //////////////////////////
    //    PAYMENT DATA      //
    //////////////////////////
    private $payment_gateway;
    private $paid;
    private $payment_must_be_manually_confimed_on_paypal;
    private $payment_details; // array, gateway specific
    private $currency_code;
    private $ipn_unique_id; // Internally generated for IPN recatch
    //////////////////////////
    //  FILES REFERENCES    //
    //////////////////////////
    private $files;
    //////////////////////////
    //  ADDITIONAL DATA     //
    //////////////////////////
    private $post_id;
    private $post_title;
    private $token;

    public function __construct()
    {
        $this->customer_timezone = Toolkit\getTimezone()->getName();
        $this->token = Toolkit\generateToken();
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //   COWORKER DATA      //
    //                      //
    //////////////////////////

    public function setCoworker($id)
    {
        $this->coworker_id = (int)$id;
    }

    public function getCoworker()
    {
        return $this->coworker_id;
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //   CUSTOMER DATA      //
    //                      //
    //////////////////////////

    public function setCustomerUserId($id)
    {
        $this->customer_user_id = $id;
    }

    public function getCustomerUserId()
    {
        return NULL !== $this->customer_user_id ? $this->customer_user_id : 0;
    }

    public function getCustomerEmail()
    {
        $return = '';
        foreach ($this->form_fields as $field) {
            if ($field->getName() == 'email') {
                $return = $field->getValue();
            }
        }

        return $return;
    }

    public function getCustomerDisplayName()
    {
        $return = '';
        foreach ($this->form_fields as $field) {
            if ($field->getName() == 'first_name') {
                $return .= $field->getValue();
            }
        }
        foreach ($this->form_fields as $field) {
            if ($field->getName() == 'second_name') {
                $return .= ' ' . $field->getValue();
            }
        }

        return $return;
    }

    public function getCustomerAddress()
    {
        $return = FALSE;
        foreach ($this->form_fields as $field) {
            if ($field->getName() == 'address') {
                $return = $field->getValue();
            }
        }

        return $return;
    }

    public function setCustomerTimezone($timezone_name)
    {
        $this->customer_timezone = $timezone_name;
    }

    public function getCustomerTimezone()
    {
        return NULL !== $this->customer_timezone ? $this->customer_timezone : Toolkit\getTimezone()->getName();
    }

    public function setEnumerableForCustomerLimits($bool)
    {
        $this->enumerable_for_customer_limit = (bool)$bool;
    }

    public function isEnumerableForCustomerLimits()
    {
        return NULL !== $this->enumerable_for_customer_limit ? $this->enumerable_for_customer_limit : TRUE;
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //     GOOGLE DATA      //
    //                      //
    //////////////////////////

    public function setGoogleCalendarId($id)
    {
        $this->google_calendar_id = $id;
    }

    public function getGoogleCalendarId()
    {
        return NULL !== $this->google_calendar_id ? $this->google_calendar_id : FALSE;
    }

    public function setGoogleCalendarEvent($id)
    {
        $this->google_calendar_event = $id;
    }

    public function getGoogleCalendarEvent()
    {
        return $this->google_calendar_event;
    }

    public function setGoogleCalendarEventParent($id)
    {
        $this->google_calendar_event_parent = $id;
    }

    public function getGoogleCalendarEventParent()
    {
        return $this->google_calendar_event_parent ?: FALSE;
    }

    public function setHangoutLink($link)
    {
        $this->hangout_link = $link;
    }

    public function getHangoutLink()
    {
        return $this->hangout_link;
    }

    public function setEventHtmlLink($link)
    {
        $this->event_html_link = $link;
    }

    public function getEventHtmlLink()
    {
        return $this->event_html_link;
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //    SERVICE DATA      //
    //                      //
    //////////////////////////

    public function setServiceId($id)
    {
        $this->service_id = $id;
    }

    public function getServiceId()
    {
        return $this->service_id;
    }

    public function setServiceClass($class)
    {
        $this->service_class = $class;
    }

    public function getServiceClass()
    {
        if (!$this->service_class) {
            try {
                $return = Functions\getSettings()->getService($this->service_id)->getClass();
            } catch (Exception $ex) {
                $return = FALSE;
            }

            return $return;
        } else {
            return $this->service_class;
        }
    }

    public function setServiceName($name)
    {
        $this->service_name = $name;
    }

    public function getServiceName()
    {
        return $this->service_name;
    }

    public function setServiceLocation($location)
    {
        $this->service_location = $location;
    }

    public function getServiceLocation()
    {
        return $this->service_location;
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    // COLLECTED FORM DATA  //
    //                      //
    //////////////////////////

    public function setFormFields(array $fields)
    {
        $this->form_fields = $fields;
    }

    /**
     * @return TeamBooking_ReservationFormField[]
     */
    public function getFormFields()
    {
        return $this->form_fields;
    }

    public function getFieldsArray()
    {
        $return = array();
        foreach ($this->form_fields as $field) {
            $return[ $field->getName() ] = $field->getValue();
        }

        return $return;
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //   RESERVATION DATA   //
    //                      //
    //////////////////////////

    public function setSlotStart($google_event_time)
    {
        $this->slot_start = $google_event_time;
    }

    public function getSlotStart()
    {
        return $this->slot_start;
    }

    public function setSlotEnd($google_event_time)
    {
        $this->slot_end = $google_event_time;
    }

    public function getSlotEnd()
    {
        return $this->slot_end;
    }

    public function setStart($time) // Unix
    {
        $this->start = $time;
    }

    public function getStart()
    {
        return $this->start;
    }

    public function setEnd($time)
    {
        $this->end = $time;
    }

    public function getEnd()
    {
        return $this->end;
    }

    public function setUri($url)
    {
        $this->uri = $url;
    }

    public function getUri()
    {
        return $this->uri;
    }

    public function setServiceDescription($description)
    {
        $this->description = $description;
    }

    public function getServiceDescription()
    {
        return $this->description;
    }

    public function setTickets($number)
    {
        $this->tickets = $number;
    }

    public function getTickets()
    {
        return NULL !== $this->tickets ? $this->tickets : 1;
    }

    public function setPrice($price)
    {
        $this->price = $price;
    }

    public function getPrice()
    {
        return $this->price;
    }

    public function setPriceDiscounted($price)
    {
        $this->price_discounted = $price;
    }

    public function getPriceDiscounted()
    {
        return NULL === $this->price_discounted ? FALSE : $this->price_discounted;
    }

    public function setDiscount(array $discount)
    {
        $this->discount = $discount;
    }

    public function addDiscount(array $discount)
    {
        $this->discount[] = $discount;
    }

    /**
     * @return array|bool
     */
    public function getDiscount()
    {
        return is_array($this->discount) ? $this->discount : FALSE;
    }

    public function setCreationInstant($now)
    {
        $this->log_key = $now;
    }

    public function getCreationInstant()
    {
        return $this->log_key;
    }

    public function setStatusPending()
    {
        $this->status = 'pending';
    }

    public function setStatusWaitingApproval()
    {
        $this->status = 'waiting_approval';
    }

    public function setStatusConfirmed()
    {
        $this->status = 'confirmed';
    }

    /**
     * This is for Unscheduled Services
     */
    public function setStatusDone()
    {
        $this->status = 'done';
    }

    public function setStatusCancelled()
    {
        $this->status = 'cancelled';
    }

    public function isCancelled()
    {
        return ($this->status == 'cancelled');
    }

    /**
     * This is for Unscheduled Services
     */
    public function isDone()
    {
        return ($this->status == 'done');
    }

    public function isConfirmed()
    {
        return ($this->status == 'confirmed' || NULL === $this->status);
    }

    public function isPending()
    {
        return ($this->status == 'pending');
    }

    public function isWaitingApproval()
    {
        return ($this->status == 'waiting_approval');
    }

    public function getStatus()
    {
        return (!$this->status) ? 'confirmed' : $this->status;
    }

    public function setCancellationReason($text)
    {
        $this->cancellation_reason = $text;
    }

    public function getCancellationReason()
    {
        return $this->cancellation_reason ?: '';
    }

    public function setCancellationWho($who)
    {
        $this->cancellation_who = $who;
    }

    public function getCancellationWho()
    {
        return NULL !== $this->cancellation_who ? $this->cancellation_who : FALSE;
    }

    public function setConfirmationWho($who)
    {
        $this->confirmation_who = $who;
    }

    public function getConfirmationWho()
    {
        return NULL !== $this->confirmation_who ? $this->confirmation_who : FALSE;
    }

    public function setPendingReason($text)
    {
        $this->pending_reason = $text;
    }

    public function getPendingReason()
    {
        return $this->pending_reason ?: '';
    }

    public function isEmailReminderSent()
    {
        return ($this->email_reminder_sent === TRUE);
    }

    public function setEmailReminderSent($bool)
    {
        $this->email_reminder_sent = (bool)$bool;
    }

    public function getDatabaseId()
    {
        return $this->database_id;
    }

    public function setDatabaseId($id)
    {
        $this->database_id = (int)$id;
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //     PAYMENT DATA     //
    //                      //
    //////////////////////////

    public function setPaymentGateway($gateway_id)
    {
        $this->payment_gateway = $gateway_id;
    }

    public function getPaymentGateway()
    {
        return $this->payment_gateway;
    }

    /**
     * @param boolean $bool
     */
    public function setPaid($bool)
    {
        $bool = (bool)$bool;
        if ($bool) {
            if ($this->payment_must_be_manually_confimed_on_paypal) {
                $this->payment_must_be_manually_confimed_on_paypal = FALSE;
            }
            $this->paid = TRUE;
        } else {
            $this->paid = FALSE;
        }
    }

    public function isPaid()
    {
        return (bool)$this->paid;
    }

    public function setPaymentDetails(array $details)
    {
        $this->payment_details = $details;
    }

    public function getPaymentDetails()
    {
        return $this->payment_details;
    }

    public function setPaymentMustBeManuallyConfirmedOnPayPal($boolean)
    {
        $this->payment_must_be_manually_confimed_on_paypal = (bool)$boolean;
    }

    public function getPaymentMustBeManuallyConfirmedOnPayPal()
    {
        return ($this->payment_must_be_manually_confimed_on_paypal === TRUE);
    }

    public function setIpnUniqueId($unique_id)
    {
        $this->ipn_unique_id = $unique_id;
    }

    public function getIpnUniqueId()
    {
        return $this->ipn_unique_id;
    }

    public function setCurrencyCode($code)
    {
        $this->currency_code = $code;
    }

    public function getCurrencyCode()
    {
        return $this->currency_code;
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //   FILES REFERENCES   //
    //                      //
    //////////////////////////

    public function addFileReference($hook, array $returned_handle)
    {
        $this->files[ $hook ] = $returned_handle;
    }

    public function removeFileReference($hook)
    {
        if (is_array($this->files)) {
            unset($this->files[ $hook ]);
        }
    }

    public function getFilesReferences()
    {
        return is_array($this->files) ? $this->files : array();
    }

    //------------------------------------------------------------

    //////////////////////////
    //                      //
    //   ADDITIONAL DATA    //
    //                      //
    //////////////////////////

    public function isFromContainer()
    {
        return ($this->getGoogleCalendarEventParent() !== FALSE);
    }

    public function setPostId($id)
    {
        $this->post_id = $id;
    }

    public function getPostId()
    {
        return $this->post_id;
    }

    public function setPostTitle($title)
    {
        $this->post_title = $title;
    }

    public function getPostTitle()
    {
        return $this->post_title;
    }

    public function setToken($token)
    {
        $this->token = $token;
    }

    public function getToken()
    {
        return $this->token;
    }

    //------------------------------------------------------------

    /**
     * Returns the hooks array
     *
     * @param bool $customer_timezone
     *
     * @return array
     */
    public function getHooksArray($customer_timezone = FALSE)
    {
        $hooks_values_array = array(
            'tickets_quantity'  => $this->getTickets(),
            'unit_price'        => Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), $this->getPriceIncremented()),
            'total_price'       => Functions\currencyCodeToSymbol(Functions\getSettings()->getCurrencyCode(), $this->getPriceIncremented() * $this->getTickets()),
            'post_id'           => $this->getPostId(),
            'post_title'        => $this->getPostTitle(),
            'service_location'  => $this->getServiceLocation(),
            'coworker_name'     => Functions\getSettings()->getCoworkerData($this->getCoworker())->getDisplayName(),
            'coworker_page'     => Functions\getSettings()->getCoworkerUrl($this->getCoworker()),
            'hangout_link'      => $this->getHangoutLink(),
            'reason'            => $this->getCancellationReason(),
            'cancellation_link' => admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=cancel&id=' . $this->getDatabaseId()
                . '&checksum=' . $this->getToken()
                . '&show_confirm=1',
            'pay_link'          => admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=pay&id=' . $this->getDatabaseId()
                . '&checksum=' . $this->getToken(),
            'ics_link'          => admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=ics&id=' . $this->getDatabaseId()
                . '&checksum=' . $this->getToken(),
            'decline_link'      => admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=decline&id=' . $this->getDatabaseId()
                . '&checksum=' . $this->getToken()
                . '&ctoken=' . Functions\getSettings()->getCoworkerData($this->getCoworker())->getApiToken()
                . '&show_confirm=1',
            'approve_link'      => admin_url() . 'admin-ajax.php?action=teambooking_rest_api&operation=approve&id=' . $this->getDatabaseId()
                . '&checksum=' . $this->getToken()
                . '&ctoken=' . Functions\getSettings()->getCoworkerData($this->getCoworker())->getApiToken()
                . '&show_confirm=1'
        );
        if ($this->getServiceClass() !== 'service') {
            if ($customer_timezone) {
                $timezone = Toolkit\getTimezone($this->getCustomerTimezone());
            } else {
                $timezone = Toolkit\getTimezone();
            }
            $hooks_values_array['timezone'] = $timezone->getName();
            $start_time_obj = DateTime::createFromFormat('U', $this->getStart());
            $start_time_obj->setTimezone($timezone);
            $hooks_values_array['start_date'] = Functions\date_i18n_tb(get_option('date_format'), $start_time_obj->getTimestamp() + $start_time_obj->getOffset());
            $hooks_values_array['start_time'] = Functions\date_i18n_tb(get_option('time_format'), $start_time_obj->getTimestamp() + $start_time_obj->getOffset());
            $hooks_values_array['start_datetime'] = $hooks_values_array['start_date'] . ' ' . $hooks_values_array['start_time'];
            $end_time_obj = DateTime::createFromFormat('U', $this->getEnd());
            $end_time_obj->setTimezone($timezone);
            $hooks_values_array['end_date'] = Functions\date_i18n_tb(get_option('date_format'), $end_time_obj->getTimestamp() + $end_time_obj->getOffset());
            $hooks_values_array['end_time'] = Functions\date_i18n_tb(get_option('time_format'), $end_time_obj->getTimestamp() + $end_time_obj->getOffset());
            $hooks_values_array['end_datetime'] = $hooks_values_array['end_date'] . ' ' . $hooks_values_array['end_time'];
        }

        return array_merge($hooks_values_array, $this->getFieldsArray());
    }

    //------------------------------------------------------------

    public function getPriceIncremented()
    {
        $fields = $this->getFormFields();
        $increment = 0;
        foreach ($fields as $field) {
            $increment += $field->getPriceIncrement();
        }
        if ($this->getDiscount()) {
            return $this->getPriceDiscounted() + $increment;
        } else {
            return $this->getPrice() + $increment;
        }

    }

    //------------------------------------------------------------

    public function getChecksum()
    {
        $back_start = $this->start;
        $back_end = $this->end;
        $this->start = NULL;
        $this->end = NULL;
        $return = md5(serialize($this));
        $this->start = $back_start;
        $this->end = $back_end;

        return $return;
    }

    //------------------------------------------------------------

    public function verifyChecksum(TeamBooking_ReservationData $reservation)
    {
        $back_start = $reservation->getStart();
        $back_end = $reservation->getEnd();
        $reservation->setStart(NULL);
        $reservation->setEnd(NULL);
        $hash_guest = md5(serialize($reservation));
        $reservation->setStart($back_start);
        $reservation->setEnd($back_end);
        if ($hash_guest === $this->getChecksum()) {
            return TRUE;
        } else {
            return FALSE;
        }
    }

    //------------------------------------------------------------

    public function getApiResource()
    {
        $return = array(
            'type'             => 'reservation',
            'id'               => $this->database_id,
            'coworkerID'       => $this->coworker_id,
            'customerUserID'   => $this->customer_user_id,
            'customerTimezone' => $this->customer_timezone,
            'service'          => array(
                'id'       => $this->service_id,
                'location' => $this->service_location
            ),
            'datetime'         => array(
                'start' => $this->start,
                'end'   => $this->end
            ),
            'payment'          => array(
                'amount'   => $this->getPriceIncremented(),
                'isPaid'   => $this->isPaid(),
                'currency' => $this->currency_code,
                'gateway'  => $this->payment_gateway
            ),
            'status'           => $this->getStatus(),
            'tickets'          => $this->tickets,
            'creationDateTime' => $this->log_key,
            'reminderSent'     => $this->isEmailReminderSent(),
            'referer'          => array(
                'postID'    => $this->post_id,
                'postTitle' => $this->post_title

            )
        );
        $fields = array();
        foreach ($this->form_fields as $field) {
            $fields[] = array(
                'type'           => 'form_field',
                'name'           => $field->getName(),
                'value'          => Toolkit\unfilterInput($field->getValue()),
                'priceIncrement' => $field->getPriceIncrement()
            );
        }
        $return['formFields'] = $fields;

        return $return;
    }

}
