<?php

namespace TeamBooking\Shortcodes;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Toolkit,
    TeamBooking\Functions,
    TeamBooking\Database;

class Reservations
{
    /**
     * TeamBooking Reservations Shortcode
     *
     * @param $atts
     *
     * @return string
     */
    public static function render($atts)
    {
        if (!is_user_logged_in()) {
            return FALSE;
        }

        // Set attributes
        extract(shortcode_atts(array(
            'read_only' => FALSE,
        ), $atts, 'tb-reservations'));

        $user_id = get_current_user_id();
        $timezone = Toolkit\getTimezone();
        $now = current_time('timestamp', TRUE);
        $random_append = Toolkit\randomNumber(6);
        $reservations = Database\Reservations::getAll();
        uasort($reservations, function ($a, $b) {
            /* @var $a \TeamBooking_ReservationData */
            /* @var $b \TeamBooking_ReservationData */
            return ($a->getStart() < $b->getStart()) ? -1 : (($a->getStart() > $b->getStart()) ? 1 : 0);
        });

        ob_start();
        echo "<div class='tbk-reservations-list-container'>";
        echo "<table class='tb-reservations-list'>";
        echo '<thead>';
        echo '<tr>';
        echo '<th>' . esc_html__('Service', 'team-booking') . '</th>';
        echo '<th>' . esc_html__('When', 'team-booking') . '</th>';
        echo '<th>' . esc_html__('Who', 'team-booking') . '</th>';
        echo '<th>' . esc_html__('Status', 'team-booking') . '</th>';
        if (!$read_only) {
            echo '<th>' . esc_html__('Actions', 'team-booking') . '</th>';
        }
        echo '</tr>';
        echo '</thead>';
        echo '<tbody>';
        foreach ($reservations as $id => $reservation) {
            if ($reservation->getCustomerUserId() == $user_id) {
                if ($reservation->getServiceClass() === 'service') {
                    $date_time_object = \DateTime::createFromFormat('U', $reservation->getCreationInstant());
                    if ($reservation->isDone()) {
                        continue;
                    }
                } else {
                    $date_time_object = \DateTime::createFromFormat('U', $reservation->getStart());
                    if ($date_time_object->getTimestamp() < $now) {
                        continue;
                    }
                }
                $date_time_object->setTimezone($timezone);
                $slot_starting_time_in_seconds = $date_time_object->getTimestamp() + $date_time_object->getOffset();
                $when_value = Functions\date_i18n_tb(get_option('date_format'), $slot_starting_time_in_seconds)
                    . ' '
                    . Functions\date_i18n_tb(get_option('time_format'), $slot_starting_time_in_seconds);
                $coworker_name = ucwords(Functions\getSettings()->getCoworkerData($reservation->getCoworker())->getDisplayName());
                $allow_customer_canc = FALSE;
                try {
                    $service = Functions\getSettings()->getService($reservation->getServiceId());
                    if (($slot_starting_time_in_seconds - $now) >= $service->getAllowCustomerCancellationUntil()) {
                        $allow_customer_canc = $service->getAllowCustomerCancellation();
                    }
                } catch (\Exception $e) {
                    // nothing
                }
                switch ($reservation->getStatus()) {
                    case 'confirmed' :
                        if ($reservation->getServiceClass() === 'service') {
                            $status_label = '<div class="tbk-slot-label blue">' . esc_html__('todo', 'team-booking') . '</div>';
                        } else {
                            $status_label = '<div class="tbk-slot-label green">' . esc_html__('confirmed', 'team-booking') . '</div>';
                        }
                        break;
                    case 'done':
                        $status_label = '<div class="tbk-slot-label green">' . esc_html__('done', 'team-booking') . '</div>';
                        break;
                    case 'waiting_approval' :
                        $status_label = '<div class="tbk-slot-label yellow">' . esc_html__('waiting approval', 'team-booking') . '</div>';
                        break;
                    case 'pending' :
                        $status_label = '<div class="tbk-slot-label yellow">' . esc_html__('pending', 'team-booking') . ' </div>';
                        break;
                    case 'cancelled' :
                        $status_label = '<div class="tbk-slot-label red">' . esc_html__('cancelled', 'team-booking') . '</div>';
                        break;
                }

                echo '<tr>';
                echo '<td>' . $reservation->getServiceName() . '</td>';
                echo '<td>' . $when_value . '</td>';
                echo '<td>' . $coworker_name . '</td>';
                echo '<td>' . $status_label . '</td>';
                if (!$read_only) {
                    echo '<td>'
                        . ((!$reservation->isCancelled()
                            && $allow_customer_canc
                            && $reservation->getServiceClass() !== 'service') ?
                            '<a href="#" class="tb-cancel-reservation" data-id="'
                            . $id . '" data-hash="' . $reservation->getToken() . '"'
                            . ' data-modal="' . $random_append . '">'
                            . esc_html__('cancel', 'team-booking')
                            . '</a>'
                            : '')
                        . '</td>';
                }
                echo '</tr>';
            }
        }
        echo '</tbody>';
        echo '</table>';

        ?>
        <div class="tbk-reservation-cancel-modal remodal" id="tbk-modal-<?= $random_append ?>"
             data-remodal-options="hashTracking: false, closeOnConfirm: false">
            <h1><?= esc_html__('Are you sure?', 'team-booking') ?></h1>
            <p>
                <?= esc_html__("You're going to cancel this reservation, the action is irreversible.", 'team-booking') ?>
            </p>
            <br>
            <button data-remodal-action="cancel"
                    class="remodal-cancel"><?= esc_html__('Cancel', 'team-booking') ?></button>
            <button data-remodal-action="confirm"
                    class="remodal-confirm"><?= esc_html__('Proceed', 'team-booking') ?></button>
        </div>
        <?php
        echo '</div>';

        return ob_get_clean();
    }
}