<?php

namespace TeamBooking\Shortcodes;
defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Toolkit,
    TeamBooking\Functions;

class Calendar
{
    /**
     * TeamBooking Calendar Shortcode
     *
     * @param $atts
     *
     * @return string
     * @throws \Exception
     */
    public static function render($atts)
    {
        // Set attributes
        extract(shortcode_atts(array(
            'booking'     => NULL,
            'coworker'    => NULL,
            'read_only'   => FALSE,
            'logged_only' => FALSE,
            'nofilter'    => FALSE, // Hide filtering buttons
            'notimezone'  => FALSE, // Hide timezone selector
        ), $atts, 'tb-calendar'));

        // Read-only mode is identified by lenght of istance id
        // This is the fastest way to keep things safe from exploits
        if (!$read_only) {
            $unique_id = Toolkit\randomNumber(8);
        } else {
            $unique_id = Toolkit\randomNumber(6);
        }

        if (!$logged_only || ($logged_only && is_user_logged_in())) {
            // Picking all active services
            $all_services = array_keys(Functions\getSettings()->getServices());
            // Remove inactive services
            foreach ($all_services as $key => $service) {
                if (!Functions\getSettings()->getService($service)->isActive()) {
                    unset($all_services[ $key ]);
                }
            }
            if (NULL !== $booking) {
                $services = array_map('trim', explode(',', $booking));
                foreach ($services as $key => $booking) {
                    try {
                        // Remove inactive services
                        if (!Functions\getSettings()->getService($booking)->isActive()) {
                            unset($services[ $key ]);
                        }
                    } catch (\Exception $exc) {
                        unset($services[ $key ]);
                        continue;
                    }
                }
                if (empty($services)) {
                    return esc_html__('WARNING: service ID(s) not found. Please check the shortcode syntax and ensure at least one of the specified services is active.', 'team-booking');
                }
            } else {
                if (NULL !== Functions\getSettings()->getServices()) {
                    // Service(s) not specified, picking all of them
                    $services = $all_services;
                } else {
                    // Service(s) not specified, but no service available
                    return esc_html__('WARNING: no service(s) found. Please add one first.', 'team-booking');
                }
            }
            $coworkers = NULL !== ($coworker) ? array_map('trim', explode(',', $coworker)) : array();
            $calendar = new \TeamBooking\Calendar();
            $parameters = new \TeamBooking\RenderParameters();
            $parameters->setServiceIds($services);
            $parameters->setRequestedServiceIds($services);
            $parameters->setCoworkerIds($coworkers);
            $parameters->setRequestedCoworkerIds($coworkers);
            $parameters->setInstance($unique_id);
            $parameters->setTimezone(Toolkit\getTimezone());
            $parameters->setIsAjaxCallNo();
            $parameters->setNoFilter($nofilter);
            $parameters->setNoTimezone($notimezone);
            ob_start();
            ?>

            <div class="ui calendar_main_container" aria-live="polite" data-postid="<?= get_the_ID() ?>">
                <?php
                $calendar->getCalendar($parameters);
                ?>
            </div>
            <br>

            <?php
            return ob_get_clean();
        }
    }
}