<?php

defined('ABSPATH') or die('No script kiddies please!');
use TeamBooking\Functions,
    TeamBooking\Toolkit;

class TeamBooking_Widget extends WP_Widget
{

    /**
     * Sets up the widgets name etc
     */
    public function __construct()
    {
        parent::__construct(
            'teambooking_widget', // Base ID
            __('Team Booking Calendar', 'team-booking'), // Name
            array(
                'description'                 => __('Shows a booking calendar', 'team-booking'),
                'customize_selective_refresh' => TRUE,
            ) // Args
        );
        // enqueue CSS and JS only if widget is active
        if (!defined('TEAMBOOKING_WIDGET_FOUND') && is_active_widget(FALSE, FALSE, $this->id_base)) {
            add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
            define('TEAMBOOKING_WIDGET_FOUND', TRUE);
        }
    }

    //------------------------------------------------------------

    /**
     * Outputs the content of the widget
     *
     * @param array $args
     * @param array $instance
     *
     * @return bool
     * @throws Exception
     */
    public function widget($args, $instance)
    {
        if (!defined('TEAMBOOKING_WIDGET_FOUND') && is_active_widget(FALSE, FALSE, $this->id_base)) {
            add_action('wp_footer', array($this, 'enqueue_scripts'));
            define('TEAMBOOKING_WIDGET_FOUND', TRUE);
        }
        $settings = Functions\getSettings();
        // Read-only mode is identified by lenght of istance id
        // This is the fastest way to keep things safe from exploits
        if ($instance['read_only'] === 'true') {
            $unique_id = Toolkit\randomNumber(6);
        } else {
            $unique_id = Toolkit\randomNumber(8);
        }
        $filter = FALSE;
        if (isset($instance['no_filter']) && $instance['no_filter'] === 'true') {
            $filter = TRUE;
        }
        $timezone = FALSE;
        if (isset($instance['no_timezone']) && $instance['no_timezone'] === 'true') {
            $timezone = TRUE;
        }
        $title = apply_filters('widget_title', $instance['title']);
        echo $args['before_widget'];
        if (!empty($title)) {
            echo $args['before_title'] . $title . $args['after_title'];
        }

        // Picking all active services
        $all_services = $settings->getServices();
        // Remove inactive services
        foreach ($all_services as $key => $service) {
            if (!$service->isActive()) {
                unset($all_services[ $key ]);
            }
        }
        if (!empty($all_services)) {
            $checkboxes = array();
            foreach ($all_services as $booking_id => $booking) {
                $checkboxes[ $booking_id ] = isset($instance[ $booking->getId() ]) ? TRUE : FALSE;
            }
        } else {
            return _e('WARNING: no service(s) found. Please add one first.', 'team-booking');
        }
        $widget_service_list = array_keys($checkboxes, TRUE);
        if (empty($widget_service_list)) {
            return _e('WARNING: no service(s) selected. Please check this widget settings.', 'team-booking');
        }
        $calendar = new TeamBooking\Calendar();
        $parameters = new TeamBooking\RenderParameters();
        if (count($widget_service_list) === 1 && $settings->getService(reset($widget_service_list))->isClass('service')) {
            $parameters->setServiceIds($widget_service_list);
        } else {
            $parameters->setServiceIds(array_keys($all_services));
        }
        $parameters->setRequestedServiceIds($widget_service_list);
        $parameters->setCoworkerIds(array());
        $parameters->setRequestedCoworkerIds(array());
        $parameters->setTimezone(Toolkit\getTimezone());
        $parameters->setInstance($unique_id);
        $parameters->setIsAjaxCallNo();
        $parameters->setNoFilter($filter);
        $parameters->setNoTimezone($timezone);
        $parameters->setIsWidgetYes();
        $post_id = get_the_ID();
        echo "<div class='ui calendar_widget_container' aria-live='polite' data-postid='" . $post_id . "'>";
        $calendar->getCalendar($parameters);
        echo '</div>';
        echo $args['after_widget'];

        return TRUE;
    }

    //------------------------------------------------------------

    /**
     * Outputs the options form on admin
     *
     * @param array $instance The widget options
     */
    public function form($instance)
    {
        if (isset($instance['title'])) {
            $title = $instance['title'];
        } else {
            $title = __('Widget title', 'team-booking');
        }
        if (!isset($instance['read_only'])) {
            $instance['read_only'] = 'false';
        }
        if (!isset($instance['no_filter'])) {
            $instance['no_filter'] = 'false';
        }
        if (!isset($instance['no_timezone'])) {
            $instance['no_timezone'] = 'false';
        }
        ?>
        <p>
            <label for="<?= $this->get_field_id('title') ?>"><?= __('Title:') ?></label>
            <input class="widefat" id="<?= $this->get_field_id('title') ?>" name="<?= $this->get_field_name('title') ?>"
                   type="text" value="<?= esc_attr($title) ?>">
        </p>
        <p><?= __('Services:', 'team-booking') ?></p>
        <?php
        foreach (Functions\getSettings()->getServices() as $booking) {
            $id = $this->get_field_id($booking->getId());
            ?>
            <label for="<?= $id ?>"></label>
            <input type="checkbox" id="<?= $id ?>" name="<?= $this->get_field_name($booking->getId()) ?>" <?php
            if (isset($instance[ $booking->getId() ])) {
                checked($instance[ $booking->getId() ], 'on');
            }
            ?>><span><?= $booking->getName() ?></span>
            <br>
        <?php } ?>
        <p><?= __('Read-only', 'team-booking') ?></p>
        <input type="radio" name="<?= $this->get_field_name('read_only') ?>" value="true" <?=
    checked($instance['read_only'], 'true')
        ?>><?= __('Yes', 'team-booking') ?>
        <input type="radio" name="<?= $this->get_field_name('read_only') ?>" value="false" <?=
    checked($instance['read_only'], 'false')
        ?>><?= __('No', 'team-booking') ?>
        <p><?= __('Hide filter buttons', 'team-booking') ?></p>
        <input type="radio" name="<?= $this->get_field_name('no_filter') ?>" value="true" <?=
    checked($instance['no_filter'], 'true')
        ?>><?= __('Yes', 'team-booking') ?>
        <input type="radio" name="<?= $this->get_field_name('no_filter') ?>" value="false" <?=
    checked($instance['no_filter'], 'false')
        ?>><?= __('No', 'team-booking') ?>
        <p><?= __('Hide timezone selector', 'team-booking') ?></p>
        <input type="radio" name="<?= $this->get_field_name('no_timezone') ?>" value="true" <?=
    checked($instance['no_timezone'], 'true')
        ?>><?= __('Yes', 'team-booking') ?>
        <input type="radio" name="<?= $this->get_field_name('no_timezone') ?>" value="false" <?=
    checked($instance['no_timezone'], 'false')
        ?>><?= __('No', 'team-booking') ?>
        <br>
        <br>
        <?php
    }

    //------------------------------------------------------------

    /**
     * Processing widget options on save
     *
     * @param array $new_instance The new options
     * @param array $old_instance The previous options
     *
     * @return array
     */
    public function update($new_instance, $old_instance)
    {
        $instance = array();
        $instance['title'] = (!empty($new_instance['title'])) ? strip_tags($new_instance['title']) : '';
        foreach (Functions\getSettings()->getServices() as $service) {
            $instance[ $service->getId() ] = $new_instance[ $service->getId() ];
        }
        $instance['read_only'] = $new_instance['read_only'];
        $instance['no_filter'] = $new_instance['no_filter'];
        $instance['no_timezone'] = $new_instance['no_timezone'];

        return $instance;
    }

    //------------------------------------------------------------

    /**
     * Enqueue the scripts and styles
     */

    public function enqueue_scripts()
    {
        if (!Functions\getSettings()->getSkipGmapLibs()) {
            wp_enqueue_script('google-places-script');
        }
        if (Functions\getSettings()->getPaymentGatewaySettingObject('stripe')->isActive()) {
            wp_enqueue_script('tb-jquery-payment');
            if (Functions\getSettings()->getPaymentGatewaySettingObject('stripe')->isLoadLibrary()) {
                wp_enqueue_script('stripejs');
            }
        }
        wp_enqueue_script('semantic-script');
        wp_enqueue_script('tb-base64-decoder');
        wp_enqueue_script('tb-geocomplete-script');
        wp_enqueue_script('tb-gmap3-script');
        wp_enqueue_script('tb-frontend-script');
        wp_enqueue_style('teambooking_fonts');
        wp_enqueue_style('semantic-style');
        wp_enqueue_style('teambooking-style-frontend');
    }

}
